@@ -12,16 +12,13 @@ import time, sys, os
 import argparse
 import shutil
 import dmgbuild
+import argparse
 
 thisFolder = Path(__file__).parent
 finalDistFolder = thisFolder.parent.parent/'dist'
 
-with Path().home()/ 'keys/apple_ost_id' as p:
-    IDENTITY = p.read_text().strip()
-with Path().home()/ 'keys/apple_psychopy_app_specific' as p:
-    PWORD = p.read_text().strip()
-
-ENTITLEMENTS = thisFolder / "entitlements.plist"
+ENTITLEMENTS = (thisFolder / "entitlements.plist").absolute()
+assert ENTITLEMENTS.exists()
 BUNDLE_ID = "org.opensciencetools.psychopy"
 USERNAME = "admin@opensciencetools.org"
 
@@ -29,6 +26,8 @@ SIGN_ALL = True
 
 # handy resources for info:
 #
+# to get a new APple app-specific password:
+#   https://appleid.apple.com/account/manage NOT developer.apple.com
 # why use zip file to notarize as well as dmg:
 #   https://deciphertools.com/blog/notarizing-dmg/
 # notarize from Python:
@@ -38,15 +37,20 @@ SIGN_ALL = True
 
 
 class AppSigner:
-    def __init__(self, appFile, version, destination=None):
+    def __init__(self, appFile, version, identity='', pword='', destination=None, verbose=False):
         self.appFile = Path(appFile)
         self.version = version
         self.destination = destination
         self._zipFile = None #'/Users/lpzjwp/code/psychopy/git/dist/PsychoPy3_2020.2.3.zip'
         self._appNotarizeUUID = None
         self._dmgBuildFile = None
+        self._pword = pword
+        self.verbose = verbose
+        self._identity = identity
 
-    def signAll(self):
+    def signAll(self, verbose=None):
+        if verbose is None:
+            verbose = self.verbose
         # remove files that we know will fail the signing:
         for filename in self.appFile.glob("**/Frameworks/SDL*"):
             shutil.rmtree(filename)
@@ -66,7 +70,7 @@ class AppSigner:
         # PyQt
         files.extend(self.appFile.glob('**/Versions/5/Qt*'))
         files.extend(self.appFile.glob('**/Contents/MacOS/QtWebEngineProcess'))
-        files.extend(self.appFile.glob('**/Resources/lib/python3.6/lib-dynload/*.so'))
+        files.extend(self.appFile.glob('**/Resources/lib/python3.8/lib-dynload/*.so'))
         files.extend(self.appFile.glob('**/Frameworks/Python.framework/Versions/3.6/Python'))
         files.extend(self.appFile.glob('**/Frameworks/Python.framework'))
         files.extend(self.appFile.glob('**/Contents/MacOS/python'))
@@ -88,10 +92,14 @@ class AppSigner:
         print(f'...done signing app in {time.time()-t0:.03f}s')
         sys.stdout.flush()
 
-    def signSingleFile(self, filename, removeFailed=False, verbose=True,
+    def signSingleFile(self, filename, removeFailed=False, verbose=None,
                        appFile=False):
+        if verbose is None:
+            verbose = self.verbose
+        if not self._identity:
+            raise ValueError('No identity provided for signing')
         cmd = ['codesign', str(filename),
-               '--sign',  IDENTITY,
+               '--sign',  self._identity,
                '--entitlements', str(ENTITLEMENTS),
                '--force',
                '--timestamp',
@@ -108,7 +116,7 @@ class AppSigner:
         # if failed and removing then remove
         if (exitcode != 0 or 'failed' in output) and removeFailed:
             Path(filename).unlink()
-            print(f"REMOVED FILE {filename}: failed to codesign")
+            print(f"FILE {filename}: failed to codesign")
         return self.signCheck(filename, verbose=False, removeFailed=removeFailed)
 
     def signCheck(self, filepath=None, verbose=False, strict=True,
@@ -141,12 +149,15 @@ class AppSigner:
         return warnings
 
     def upload(self, fileToNotarize):
+        """Uploads a file to Apple for notarizing"""
+        if not self._pword:
+            raise ValueError('No app-specific password provided for notarizing')
         filename = Path(fileToNotarize).name
         print(f'Sending {filename} to apple for notarizing')
         cmdStr = (f"xcrun altool --notarize-app -t osx -f {fileToNotarize} "
                   f"--primary-bundle-id {BUNDLE_ID} -u {USERNAME} ")
         print(cmdStr)
-        cmdStr += f"-p {PWORD}"
+        cmdStr += f"-p {self._pword}"
         t0 = time.time()
         exitcode, output = subprocess.getstatusoutput(cmdStr)
         m = re.match('.*RequestUUID = (.*)\n', output, re.S)
@@ -197,7 +208,7 @@ class AppSigner:
 
     def checkStatus(self, uuid):
         cmd = ['xcrun', 'altool', '--notarization-info', self._appNotarizeUUID,
-               '-u', USERNAME, '-p', PWORD]
+               '-u', USERNAME, '-p', self._pword]
         cmdStr = ' '.join(cmd)
         exitcode, output = subprocess.getstatusoutput(cmdStr)
 
@@ -285,7 +296,22 @@ class AppSigner:
                 break  # succeeded so stop
         if exitcode != 0:
             print(f'*********Failed to detach {volName} (wrong name?) *************')
-            exit(1)
+            time.sleep(10)  # wait for 10s and then try more forcefully
+            import diskutil_parser.cmd
+            import sh
+            disks = diskutil_parser.cmd.diskutil_list()
+            for disk in disks:
+                print(f"checking /dev/{disk.device_id} ({disk.partition_scheme})")
+                for part in disk.partitions:
+                    if "PsychoPy" in part.name:
+                        print("Ejecting - ", part.name, part.mount_point)
+                        try:
+                            sh.diskutil("unmountDisk", "force", f"/dev/{disk.device_id}")
+                            sh.diskutil("eject", f"/dev/{disk.device_id}")
+                        except sh.ErrorReturnCode_1:
+                            print("still can't eject that disk")
+                            exit(1)
+
 
     def dmgCompress(self):
         dmgFilename = str(self.appFile).replace(".app", "_rw.dmg")
@@ -325,6 +351,10 @@ def main():
                         action='store', required=False, default='true')
     parser.add_argument("--runPostDmgBuild", help="Runs up until dmg is built (and notarised) then exits",
                         action='store', required=False, default='true')
+    parser.add_argument("--id", help="ost id for codesigning",
+                        action='store', required=False, default=None)
+    parser.add_argument("--pwd", help="password for app-specific password",
+                        action='store', required=False, default=None)
     args = parser.parse_args()
     args.runPreDmgBuild = args.runPreDmgBuild.lower() in ['true', 'True', '1', 'y', 'yes']
     args.runDmgBuild = args.runDmgBuild.lower() in ['true', 'True', '1', 'y', 'yes']
@@ -335,10 +365,21 @@ def main():
     else:
         NOTARIZE = True
 
+    # codesigning identity from CLI args?
+    if args.id:
+        IDENTITY = args.id
+    else:
+        with Path().home()/ 'keys/apple_ost_id' as p:
+            IDENTITY = p.read_text().strip()
+    if args.pwd:
+        PWORD = args.pwd
+    else:
+        with Path().home()/ 'keys/apple_psychopy_app_specific' as p:
+            PWORD = p.read_text().strip()
+            
     if args.file:  # not the whole app - just sign one file
         distFolder = (thisFolder / '../dist').resolve()
-        signer = AppSigner(appFile='',
-                           version=None)
+        signer = AppSigner(appFile='', version=None, pword=PWORD, identity=IDENTITY)
         signer.signSingleFile(args.file, removeFailed=False, verbose=True)
         signer.signCheck(args.file, verbose=True)
 
@@ -350,8 +391,7 @@ def main():
 
     else:  # full app signing and notarization
         distFolder = (thisFolder / '../dist').resolve()
-        signer = AppSigner(appFile=distFolder/args.app,
-                        version=args.version)
+        signer = AppSigner(appFile=distFolder/args.app, version=args.version, pword=PWORD, identity=IDENTITY)
 
         if args.runPreDmgBuild:
             if SIGN_ALL:
