@@ -1,11 +1,9 @@
-from __future__ import print_function
-from builtins import next
-import csv, codecs
+import csv
 from psychopy import gui
 
 filename = gui.fileOpenDlg('.', allowed='*.csv')[0]
         
-#use csv from python (not from numpy) due to handling newlines within quote char
+# use csv from python (not from numpy) due to handling newlines within quote char
 with open(filename, 'rU') as csvFile:
     spamreader = csv.reader(csvFile, delimiter=',', quotechar='"', dialect=csv.excel)
     headers = next(spamreader)
@@ -18,9 +16,9 @@ with open(filename, 'rU') as csvFile:
             thisEntry[thisFieldName] = thisRow[fieldN]
         entries.append(thisEntry)
 
-companHead="Your Company or Institution"
-nameHead='Your name (or anon, but a name is nicer)'
-testimHead='Your thoughts on PsychoPy'
+companHead = "Your Company or Institution"
+nameHead = 'Your name (or anon, but a name is nicer)'
+testimHead = 'Your thoughts on PsychoPy'
 posnHead = 'Your position'
 
 
@@ -34,4 +32,3 @@ with open('testimonialsText.html', 'wb') as outFile:
             nameStr += ', %s' %thisEntry[companHead]
         nameStr += ' </em><br>\n'
         outFile.write(nameStr)
-
