@@ -15,7 +15,7 @@ import wx
 from collections import OrderedDict
 from psychopy.experiment.components.code import CodeComponent
 from ..validators import WarningManager
-from ...themes import ThemeMixin
+from ...themes import handlers
 
 from importlib.util import find_spec as loader
 hasMetapensiero = loader("metapensiero") is not None
@@ -33,7 +33,8 @@ class DlgCodeComponentProperties(wx.Dialog):
     def __init__(self, frame, element, experiment,
                  helpUrl=None, suppressTitles=True, size=(1000,600),
                  style=_style, editing=False, depends=[],
-                 timeout=None, type="Code"):
+                 timeout=None, type="Code",
+                 openToPage=None):
 
         # translate title
         if "name" in element.params:
@@ -55,6 +56,7 @@ class DlgCodeComponentProperties(wx.Dialog):
         self.frame = frame
         self.app = frame.app
         self.helpUrl = helpUrl
+        self.component = element
         self.params = element.params  # dict
         self.order = element.order
         self.title = title
@@ -74,8 +76,6 @@ class DlgCodeComponentProperties(wx.Dialog):
         # in FlatNoteBook the tab controls (left,right,close) are ugly on mac
         #   and also can't be killed
 
-        openToPage = None
-        tabN = -1
         for paramN, paramName in enumerate(self.order):
             param = self.params.get(paramName)
             if paramName == 'name':
@@ -116,9 +116,12 @@ class DlgCodeComponentProperties(wx.Dialog):
                     _panel = self.tabs[tabName]
                 else:
                     _panel = wx.Panel(self.codeNotebook, wx.ID_ANY)
+                    _panel.tabN = len(self.tabs)
                     _panel.app = self.app
                     self.tabs[tabName] = _panel
-                    tabN += 1
+                # if openToPage refers to this page by name, convert to index
+                if openToPage == paramName:
+                    openToPage = _panel.tabN
 
                 self.codeBoxes[paramName] = CodeBox(_panel, wx.ID_ANY,
                                                     pos=wx.DefaultPosition,
@@ -129,9 +132,10 @@ class DlgCodeComponentProperties(wx.Dialog):
                 self.codeBoxes[paramName].AddText(param.val)
                 self.codeBoxes[paramName].Bind(wx.EVT_KEY_UP, self.onKeyUp)  # For real time translation
 
-                if len(param.val.strip()) and openToPage is None:
-                    # index of first non-blank page
-                    openToPage = tabN
+                if len(param.val.strip()) and hasattr(_panel, "tabN") and not isinstance(openToPage, str):
+                    if openToPage is None or openToPage > _panel.tabN:
+                        # index of first non-blank page
+                        openToPage = _panel.tabN
 
         if self.helpUrl is not None:
             self.helpButton = wx.Button(self, wx.ID_HELP,
@@ -145,7 +149,7 @@ class DlgCodeComponentProperties(wx.Dialog):
         self.warnings = WarningManager(self)  # to store warnings for all fields
         self.__do_layout()
         if openToPage is None:
-            openToPage = 1
+            openToPage = 0
         self.codeNotebook.SetSelection(openToPage)
         self.Update()
         self.Bind(wx.EVT_BUTTON, self.helpButtonHandler, self.helpButton)
@@ -308,7 +312,7 @@ class DlgCodeComponentProperties(wx.Dialog):
 
         try:
             if pythonCode:
-                jsCode = translatePythonToJavaScript(pythonCode)
+                jsCode = translatePythonToJavaScript(pythonCode, namespace=None)
 
             if codeChangeTest:
                 return jsCode
@@ -400,7 +404,10 @@ class DlgCodeComponentProperties(wx.Dialog):
             # Add a visual indicator when tab contains code
             emptyCodeComp = CodeComponent('', '') # Spawn empty code component
             # If code tab is not empty and not the same as in empty code component, add an asterisk to tab name
-            if (self.params.get(pyName).val or self.params.get(jsName).val) and not (self.params.get(pyName).val == emptyCodeComp.params.get(pyName).val or self.params.get(jsName).val == emptyCodeComp.params.get(jsName).val):
+            hasContents = self.params.get(pyName).val or self.params.get(jsName).val
+            pyEmpty = self.params.get(pyName).val == emptyCodeComp.params.get(pyName).val
+            jsEmpty = self.params.get(jsName).val == emptyCodeComp.params.get(jsName).val
+            if hasContents and not (pyEmpty and jsEmpty):
                 tabLabel += ' *'
             self.codeNotebook.AddPage(panel, tabLabel)
 
@@ -467,7 +474,7 @@ class DlgCodeComponentProperties(wx.Dialog):
         self.app.followLink(url=self.helpUrl)
 
 
-class CodeBox(BaseCodeEditor, ThemeMixin):
+class CodeBox(BaseCodeEditor, handlers.ThemeMixin):
     # this comes mostly from the wxPython demo styledTextCtrl 2
 
     def __init__(self, parent, ID, prefs,
