@@ -36,6 +36,7 @@ __all__ = [
 import os.path
 
 import wx
+import os
 from subprocess import Popen, PIPE
 from threading import Thread, Event
 from queue import Queue, Empty
@@ -173,6 +174,9 @@ class Job:
         Callback function called when `poll` is invoked and the error pipe has
         data. Data is passed to the first argument of the callable object. You
         may set `inputCallback` and `errorCallback` using the same function.
+    extra : dict or None
+        Dict of extra variables to be accessed by callback functions, use None
+        for a blank dict.
 
     Examples
     --------
@@ -187,7 +191,7 @@ class Job:
 
     """
     def __init__(self, parent, command='', terminateCallback=None,
-                 inputCallback=None, errorCallback=None):
+                 inputCallback=None, errorCallback=None, extra=None):
 
         # command to be called, cannot be changed after spawning the process
         self.parent = parent
@@ -205,6 +209,10 @@ class Job:
         self.inputCallback = inputCallback
         self.errorCallback = errorCallback
         self.terminateCallback = terminateCallback
+        # user defined additional info
+        if extra is None:
+            extra = {}
+        self.extra = extra
 
         # non-blocking pipe reading threads and FIFOs
         self._stdoutReader = None
@@ -248,7 +256,8 @@ class Job:
                 cwd=cwd,
                 env=None,
                 universal_newlines=True,  # gives us back a string instead of bytes
-                creationflags=0
+                creationflags=0,
+                text=True
             )
         except FileNotFoundError:
             return -1  # negative PID means failure
@@ -286,6 +295,7 @@ class Job:
 
         # isOk = wx.Process.Kill(self._pid, signal, flags) is wx.KILL_OK
         self._process.kill()  # kill the process
+        #self._pollTimer.Stop()
 
         # Wait for the process to exit completely, return code will be incorrect
         # if we don't.
