@@ -26,6 +26,7 @@ import wx
 from wx.lib import scrolledpanel as scrlpanel
 
 from .. import utils
+from ..themes import icons
 from ...projects.pavlovia import PavloviaProject
 
 try:
@@ -189,12 +190,12 @@ class ProjectEditor(wx.Dialog):
 class DetailsPanel(wx.Panel):
 
     class StarBtn(wx.Button):
-        def __init__(self, parent, iconCache, value=False):
+        def __init__(self, parent, value=False):
             wx.Button.__init__(self, parent, label=_translate("Star"))
             # Setup icons
             self.icons = {
-                True: iconCache.getBitmap(name="starred", size=16),
-                False: iconCache.getBitmap(name="unstarred", size=16),
+                True: icons.ButtonIcon(stem="starred", size=16, theme="light").bitmap,
+                False: icons.ButtonIcon(stem="unstarred", size=16, theme="light").bitmap,
             }
             self.SetBitmapDisabled(self.icons[False])  # Always appear empty when disabled
             # Set start value
@@ -217,14 +218,15 @@ class DetailsPanel(wx.Panel):
             self.value = (not self.value)
 
     def __init__(self, parent, project=None,
-                 size=(650, 550),
+                 size=(650, 650),
                  style=wx.NO_BORDER):
 
         wx.Panel.__init__(self, parent, -1,
                           size=size,
                           style=style)
         self.SetBackgroundColour("white")
-        iconCache = parent.app.iconCache
+        self.parent = parent
+        self._updateQueue = []
         # Setup sizer
         self.contentBox = wx.BoxSizer()
         self.SetSizer(self.contentBox)
@@ -236,7 +238,7 @@ class DetailsPanel(wx.Panel):
         # Icon
         self.icon = utils.ImageCtrl(self, bitmap=wx.Bitmap(), size=(128, 128))
         self.icon.SetBackgroundColour("#f2f2f2")
-        self.icon.Bind(wx.EVT_FILEPICKER_CHANGED, self.updateProject)
+        self.icon.Bind(wx.EVT_FILEPICKER_CHANGED, self.queueUpdate)
         self.headSizer.Add(self.icon, border=6, flag=wx.ALL)
         self.icon.SetToolTip(_translate(
             "An image to represent this project, this helps it stand out when browsing on Pavlovia."
@@ -248,7 +250,7 @@ class DetailsPanel(wx.Panel):
         self.title = wx.TextCtrl(self,
                                  size=(-1, 30 if sys.platform == 'darwin' else -1),
                                  value="")
-        self.title.Bind(wx.EVT_KILL_FOCUS, self.updateProject)
+        self.title.Bind(wx.EVT_TEXT, self.queueUpdate)
         self.title.SetFont(
             wx.Font(24, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD)
         )
@@ -276,7 +278,7 @@ class DetailsPanel(wx.Panel):
         # Star button
         self.starLbl = wx.StaticText(self, label="-")
         self.btnSizer.Add(self.starLbl, border=6, flag=wx.LEFT | wx.TOP | wx.BOTTOM | wx.ALIGN_CENTER_VERTICAL)
-        self.starBtn = self.StarBtn(self, iconCache=iconCache)
+        self.starBtn = self.StarBtn(self)
         self.starBtn.Bind(wx.EVT_BUTTON, self.star)
         self.btnSizer.Add(self.starBtn, border=6, flag=wx.ALL | wx.EXPAND)
         self.starBtn.SetToolTip(_translate(
@@ -287,7 +289,7 @@ class DetailsPanel(wx.Panel):
         self.forkLbl = wx.StaticText(self, label="-")
         self.btnSizer.Add(self.forkLbl, border=6, flag=wx.LEFT | wx.TOP | wx.BOTTOM | wx.ALIGN_CENTER_VERTICAL)
         self.forkBtn = wx.Button(self, label=_translate("Fork"))
-        self.forkBtn.SetBitmap(iconCache.getBitmap(name="fork", size=16))
+        self.forkBtn.SetBitmap(icons.ButtonIcon(stem="fork", size=16, theme="light").bitmap)
         self.forkBtn.Bind(wx.EVT_BUTTON, self.fork)
         self.btnSizer.Add(self.forkBtn, border=6, flag=wx.ALL | wx.EXPAND)
         self.forkBtn.SetToolTip(_translate(
@@ -296,7 +298,7 @@ class DetailsPanel(wx.Panel):
         ))
         # Create button
         self.createBtn = wx.Button(self, label=_translate("Create"))
-        self.createBtn.SetBitmap(iconCache.getBitmap(name="plus", size=16))
+        self.createBtn.SetBitmap(icons.ButtonIcon(stem="plus", size=16, theme="light").bitmap)
         self.createBtn.Bind(wx.EVT_BUTTON, self.create)
         self.btnSizer.Add(self.createBtn, border=6, flag=wx.RIGHT | wx.TOP | wx.BOTTOM | wx.ALIGN_CENTER_VERTICAL)
         self.createBtn.SetToolTip(_translate(
@@ -304,7 +306,7 @@ class DetailsPanel(wx.Panel):
         ))
         # Sync button
         self.syncBtn = wx.Button(self, label=_translate("Sync"))
-        self.syncBtn.SetBitmap(iconCache.getBitmap(name="view-refresh", size=16))
+        self.syncBtn.SetBitmap(icons.ButtonIcon(stem="view-refresh", size=16, theme="light").bitmap)
         self.syncBtn.Bind(wx.EVT_BUTTON, self.sync)
         self.btnSizer.Add(self.syncBtn, border=6, flag=wx.ALL | wx.EXPAND)
         self.syncBtn.SetToolTip(_translate(
@@ -313,7 +315,7 @@ class DetailsPanel(wx.Panel):
         ))
         # Get button
         self.downloadBtn = wx.Button(self, label=_translate("Download"))
-        self.downloadBtn.SetBitmap(iconCache.getBitmap(name="download", size=16))
+        self.downloadBtn.SetBitmap(icons.ButtonIcon(stem="download", size=16, theme="light").bitmap)
         self.downloadBtn.Bind(wx.EVT_BUTTON, self.sync)
         self.btnSizer.Add(self.downloadBtn, border=6, flag=wx.ALL | wx.EXPAND)
         self.downloadBtn.SetToolTip(_translate(
@@ -335,7 +337,7 @@ class DetailsPanel(wx.Panel):
         self.localRootLabel = wx.StaticText(self, label="Local root:")
         self.rootSizer.Add(self.localRootLabel, border=6, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL)
         self.localRoot = utils.FileCtrl(self, dlgtype="dir")
-        self.localRoot.Bind(wx.EVT_FILEPICKER_CHANGED, self.updateProject)
+        self.localRoot.Bind(wx.EVT_FILEPICKER_CHANGED, self.queueUpdate)
         self.rootSizer.Add(self.localRoot, proportion=1, border=6, flag=wx.EXPAND | wx.LEFT | wx.RIGHT | wx.BOTTOM)
         self.localRoot.SetToolTip(_translate(
             "Folder in which local files are stored for this project. Changes to files in this folder will be tracked "
@@ -344,8 +346,8 @@ class DetailsPanel(wx.Panel):
         # Sep
         self.sizer.Add(wx.StaticLine(self, -1), border=6, flag=wx.EXPAND | wx.ALL)
         # Description
-        self.description = wx.TextCtrl(self, size=(-1, -1), value="", style=wx.TE_MULTILINE)
-        self.description.Bind(wx.EVT_KILL_FOCUS, self.updateProject)
+        self.description = utils.MarkdownCtrl(self, size=(-1, -1), value="", file=None)
+        self.description.Bind(wx.EVT_TEXT, self.queueUpdate)
         self.sizer.Add(self.description, proportion=1, border=6, flag=wx.ALL | wx.EXPAND)
         self.description.SetToolTip(_translate(
             "Description of the project to be shown on Pavlovia. Note: This is different than a README file!"
@@ -358,7 +360,7 @@ class DetailsPanel(wx.Panel):
         self.visLbl = wx.StaticText(self, label=_translate("Visibility:"))
         self.visSizer.Add(self.visLbl, border=6, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL)
         self.visibility = wx.Choice(self, choices=["Private", "Public"])
-        self.visibility.Bind(wx.EVT_CHOICE, self.updateProject)
+        self.visibility.Bind(wx.EVT_CHOICE, self.queueUpdate)
         self.visSizer.Add(self.visibility, proportion=1, border=6, flag=wx.EXPAND | wx.ALL)
         self.visibility.SetToolTip(_translate(
             "Visibility of the current project; whether its visible only to its creator (Private) or to any user "
@@ -370,7 +372,7 @@ class DetailsPanel(wx.Panel):
         self.statusLbl = wx.StaticText(self, label=_translate("Status:"))
         self.statusSizer.Add(self.statusLbl, border=6, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL)
         self.status = wx.Choice(self, choices=["Running", "Piloting", "Inactive"])
-        self.status.Bind(wx.EVT_CHOICE, self.updateProject)
+        self.status.Bind(wx.EVT_CHOICE, self.queueUpdate)
         self.statusSizer.Add(self.status, proportion=1, border=6, flag=wx.EXPAND | wx.ALL)
         self.status.SetToolTip(_translate(
             "Project status; whether it can be run to collect data (Running), run by its creator without saving "
@@ -382,17 +384,25 @@ class DetailsPanel(wx.Panel):
         self.tagLbl = wx.StaticText(self, label=_translate("Keywords:"))
         self.tagSizer.Add(self.tagLbl, border=6, flag=wx.ALIGN_CENTER_VERTICAL | wx.ALL)
         self.tags = utils.ButtonArray(self, orient=wx.HORIZONTAL, items=[], itemAlias=_translate("tag"))
-        self.tags.Bind(wx.EVT_LIST_INSERT_ITEM, self.updateProject)
-        self.tags.Bind(wx.EVT_LIST_DELETE_ITEM, self.updateProject)
+        self.tags.Bind(wx.EVT_LIST_INSERT_ITEM, self.queueUpdate)
+        self.tags.Bind(wx.EVT_LIST_DELETE_ITEM, self.queueUpdate)
         self.tagSizer.Add(self.tags, proportion=1, border=6, flag=wx.EXPAND | wx.ALL)
         self.tags.SetToolTip(_translate(
             "Keywords associated with this project, helping others to find it. For example, if your experiment is "
             "useful to psychophysicists, you may want to add the keyword 'psychophysics'."
         ))
+        # Update button
+        self.updateBtn = wx.Button(self, style=wx.BU_EXACTFIT)
+        self.updateBtn.SetBitmap(icons.ButtonIcon(stem="view-refresh", size=16, theme="light").bitmap)
+        self.sizer.Add(self.updateBtn, flag=wx.ALIGN_RIGHT | wx.ALL)
+        self.updateBtn.Bind(wx.EVT_BUTTON, self.doUpdate)
+        self.updateBtn.Disable()
         # Populate
         if project is not None:
             project.refresh()
         self.project = project
+        # Bind close function
+        self.Bind(wx.EVT_WINDOW_DESTROY, self.close)
 
     @property
     def project(self):
@@ -405,7 +415,7 @@ class DetailsPanel(wx.Panel):
         # Populate fields
         if project is None:
             # Icon
-            self.icon.SetBitmap(wx.Bitmap())
+            self.icon.setImage(wx.Bitmap())
             self.icon.SetBackgroundColour("#f2f2f2")
             self.icon.Disable()
             # Title
@@ -444,7 +454,7 @@ class DetailsPanel(wx.Panel):
             wx.TextCtrl.SetValue(self.localRoot, "")  # use base method to avoid callback
             self.localRoot.Disable()
             # Description
-            self.description.SetValue("")
+            self.description.setValue("")
             self.description.Disable()
             # Visibility
             self.visibility.SetSelection(wx.NOT_FOUND)
@@ -455,6 +465,20 @@ class DetailsPanel(wx.Panel):
             # Tags
             self.tags.clear()
             self.tags.Disable()
+        elif project.project is None:
+            # If project has been deleted, prompt to unlink
+            dlg = wx.MessageDialog(
+                self,
+                message=_translate(
+                    "Could not find GitLab project with id {}.\n"
+                    "\n"
+                    "Please check that the project exists on Pavlovia, that you are logged in as the correct user in "
+                    "the PsychoPy app, and that your account has access to the project."
+                ).format(project.id),
+                style=wx.ICON_ERROR
+            )
+            dlg.ShowModal()
+            self.project = None
         else:
             # Refresh project to make sure it has info
             if not hasattr(project, "_info"):
@@ -462,13 +486,13 @@ class DetailsPanel(wx.Panel):
             # Icon
             if 'avatarUrl' in project.info:
                 try:
-                    content = requests.get(project['avatar_url']).content
-                    icon = wx.Bitmap(wx.Image(io.BytesIO(content)))
+                    content = self.session.session.get(project['avatar_url']).content
+                    icon = io.BytesIO(content)
                 except requests.exceptions.MissingSchema:
                     icon = wx.Bitmap()
             else:
                 icon = wx.Bitmap()
-            self.icon.SetBitmap(icon)
+            self.icon.setImage(icon)
             self.icon.SetBackgroundColour("#f2f2f2")
             self.icon.Enable(project.editable)
             # Title
@@ -509,7 +533,7 @@ class DetailsPanel(wx.Panel):
             self.localRootLabel.Enable(project.editable)
             self.localRoot.Enable(project.editable)
             # Description
-            self.description.SetValue(project['description'])
+            self.description.setValue(project['description'])
             self.description.Enable(project.editable)
             # Visibility
             self.visibility.SetStringSelection(project['visibility'])
@@ -523,6 +547,9 @@ class DetailsPanel(wx.Panel):
 
         # Layout
         self.Layout()
+        # Clear update queue as we've just set from online
+        self._updateQueue = []
+        self.updateBtn.Disable()
 
     @property
     def session(self):
@@ -551,10 +578,13 @@ class DetailsPanel(wx.Panel):
         # Enable ctrl now that there is a local root
         self.localRoot.Enable()
         self.localRootLabel.Enable()
-        # Show sync dlg (does sync)
-        dlg = sync.SyncDialog(self, self.project)
-        dlg.sync()
-        functions.showCommitDialog(self, self.project, initMsg="", infoStream=dlg.status)
+        # Get filename if available
+        if hasattr(self.GetTopLevelParent(), "filename"):
+            file = self.parent.filename
+        else:
+            file = ""
+        # Do sync
+        syncProject(self, self.project, file=file)
         # Update project
         self.project.refresh()
         # Update last sync date & show
@@ -595,32 +625,48 @@ class DetailsPanel(wx.Panel):
         # Toggle button
         self.starBtn.toggle()
         # Star/unstar project
-        self.updateProject(evt)
+        self.queueUpdate(evt)
         # todo: Refresh stars count
 
-    def updateProject(self, evt=None):
+    def queueUpdate(self, evt=None):
         # Skip if no project
         if self.project is None or evt is None:
             return
         # Get object
         obj = evt.GetEventObject()
+        # Mark object as needing update
+        if obj not in self._updateQueue:
+            self._updateQueue.append(obj)
+        # Enable update button
+        self.updateBtn.Enable()
 
+    def doUpdate(self, evt=None):
+        # Update each object in queue
+        success = []
+        for obj in self._updateQueue:
+            success.append(self.updateProject(obj))
+        # Disable update button
+        self.updateBtn.Enable(not all(success))
+
+    def updateProject(self, obj):
+        success = False
         # Update project attribute according to supplying object
         if obj == self.title and self.project.editable:
             self.project['name'] = self.title.Value
-            self.project.save()
+            success = self.project.save()
         if obj == self.icon:
             # Create temporary image file
             _, temp = tempfile.mkstemp(suffix=".png")
             self.icon.BitmapFull.SaveFile(temp, wx.BITMAP_TYPE_PNG)
             # Load and upload from temp file
             self.project['avatar'] = open(temp, "rb")
-            self.project.save()
+            success = self.project.save()
             # Delete temp file
             #os.remove(temp)
         if obj == self.starBtn:
             self.project.starred = self.starBtn.value
             self.starLbl.SetLabel(str(self.project.info['nbStars']))
+            success = True
         if obj == self.localRoot:
             if Path(self.localRoot.Value).is_dir():
                 self.project.localRoot = self.localRoot.Value
@@ -637,20 +683,37 @@ class DetailsPanel(wx.Panel):
                 dlg.ShowModal()
             # Set project again to trigger a refresh
             self.project = self.project
+            success = True
         if obj == self.description and self.project.editable:
-            self.project['description'] = self.description.Value
-            self.project.save()
+            self.project['description'] = self.description.getValue()
+            success = self.project.save()
         if obj == self.visibility and self.project.editable:
             self.project['visibility'] = self.visibility.GetStringSelection().lower()
-            self.project.save()
+            success = self.project.save()
         if obj == self.status and self.project.editable:
-            requests.put(f"https://pavlovia.org/api/v2/experiments/{self.project.id}",
-                         data={"status2": self.status.GetStringSelection()},
-                         headers={'OauthToken': self.session.getToken()})
+            retval = self.session.session.put(
+                f"https://pavlovia.org/api/v2/experiments/{self.project.id}",
+                json={'status2': self.status.GetStringSelection().upper()}
+            )
+            success = True
         if obj == self.tags and self.project.editable:
-            requests.put(f"https://pavlovia.org/api/v2/experiments/{self.project.id}",
-                         data={"keywords": self.tags.GetValue()},
-                         headers={'OauthToken': self.session.getToken()})
+            retval = self.session.session.put(
+                f"https://pavlovia.org/api/v2/experiments/{self.project.id}",
+                json={"keywords": self.tags.GetValue()}
+            )
+            success = True
+        # Clear from update queue
+        if obj in self._updateQueue:
+            self._updateQueue.remove(obj)
+
+        return success
+
+    def close(self, evt=None):
+        if len(self._updateQueue):
+            wx.MessageDialog(self, message=_translate(
+                "Project info has changed, update online before closing?"
+            ), style=wx.YES_NO | wx.CANCEL)
+
 
 
 class ProjectFrame(wx.Dialog):
@@ -691,8 +754,34 @@ def syncProject(parent, project, file="", closeFrameWhenDone=False):
         0 for fail
         -1 for cancel at some point in the process
     """
+    # Error catch logged out
+    session = pavlovia.getCurrentSession()
+    if not session or not session.user:
+        # If not logged in, prompt to login
+        dlg = wx.MessageDialog(None, message=_translate(
+            "You are not logged in to Pavlovia. Please log in to sync project."
+        ), style=wx.ICON_AUTH_NEEDED | wx.OK | wx.CANCEL)
+        dlg.SetOKLabel(_translate("Login..."))
+        if dlg.ShowModal() == wx.ID_OK:
+            # If they click Login, open login screen
+            user = functions.logInPavlovia(None)
+            # If they cancelled out of login screen, cancel sync
+            if not user:
+                return
+        else:
+            # If they cancel out of login prompt, cancel sync
+            return
+
     # If not in a project, make one
     if project is None:
+        # Try to get project id from git files
+        projName = pavlovia.getNameWithNamespace(file)
+        if projName is not None:
+            # If successful, make PavloviaProject from local info
+            project = PavloviaProject(projName, localRoot=file)
+
+    if project is None or project.project is None:
+        # If project is still None
         msgDlg = wx.MessageDialog(parent,
                                message=_translate("This file doesn't belong to any existing project."),
                                style=wx.OK | wx.CANCEL | wx.CENTER)
@@ -716,29 +805,52 @@ def syncProject(parent, project, file="", closeFrameWhenDone=False):
                 return
         else:
             return
-    # If no local root, prompt to make one
-    if not project.localRoot:
+    # If no local root or dead local root, prompt to make one
+    if not project.localRoot or not Path(project.localRoot).is_dir():
         defaultRoot = Path(file).parent
-        # Ask user if they want to
-        dlg = wx.MessageDialog(parent, message=_translate("Project root folder is not yet specified, specify one now?"), style=wx.YES_NO)
-        # Open folder picker
-        if dlg.ShowModal() == wx.ID_YES:
-            dlg = wx.DirDialog(parent, message=_translate("Specify folder..."), defaultPath=str(defaultRoot))
-            if dlg.ShowModal() == wx.ID_OK:
-                localRoot = Path(dlg.GetPath())
-                project.localRoot = str(localRoot)
+
+        # Handle missing or invalid local root
+        if not project.localRoot or not Path(project.localRoot).is_dir():
+            if file and defaultRoot.is_dir():
+                # If we have a reference to the current folder, use it
+                project.localRoot = defaultRoot
             else:
-                # If cancelled, cancel sync
-                return
-        else:
-            # If they don't want to specify, cancel sync
-            return
+                # Otherwise, ask user to choose a local root
+                if not project.localRoot:
+                    # If there is no local root at all, prompt user to make one
+                    msg = _translate("Project root folder is not yet specified, specify project root now?")
+                else:
+                    # If there is a local root but the folder is gone, prompt user to change it
+                    msg = _translate("Project root folder does not exist, change project root now?")
+                dlg = wx.MessageDialog(parent, message=msg, style=wx.OK | wx.CANCEL)
+                # Get response
+                if dlg.ShowModal() == wx.ID_OK:
+                    dlg = wx.DirDialog(parent, message=_translate("Specify folder..."), defaultPath=str(defaultRoot))
+                    if dlg.ShowModal() == wx.ID_OK:
+                        project.localRoot = str(dlg.GetPath())
+                    else:
+                        # If cancelled, cancel sync
+                        return
+                else:
+                    # If they don't want to specify, cancel sync
+                    return
     # Assign project to parent frame
     parent.project = project
     # If there is (now) a project, do sync
     if project is not None:
+        # Show sync dlg
         dlg = sync.SyncDialog(parent, project)
-        functions.showCommitDialog(parent, project, initMsg="", infoStream=dlg.status)
+        # Commit changes
+        committed = functions.showCommitDialog(parent, project, initMsg="", infoStream=dlg.status)
+        # Cancel sync if commit cancelled
+        if committed == -1:
+            dlg.status.write(_translate(
+                "\n"
+                "Sync cancelled by user."
+            ))
+            dlg.OKbtn.Enable(True)
+            return
+        # Do sync
         dlg.sync()
 
 
