@@ -2,6 +2,8 @@
 # -*- coding: utf-8 -*-
 import json
 import sys
+from pathlib import Path
+
 import wx
 import wx.propgrid as pg
 import wx.py
@@ -9,6 +11,7 @@ import platform
 import re
 import os
 
+from psychopy.app.themes import icons
 from . import dialogs
 from psychopy import localization, prefs
 from psychopy.localization import _translate
@@ -256,7 +259,7 @@ class PrefPropGrid(wx.Panel):
             if s not in self.sections.keys():
                 self.sections[s] = dict()
 
-        nbBitmap = self.app.iconCache.getBitmap(bitmap)
+        nbBitmap = icons.ButtonIcon(stem=bitmap, size=(48, 48)).bitmap
         if nbBitmap.IsOk():
             self.prefsImages.Add(nbBitmap)
 
@@ -388,6 +391,10 @@ class PrefPropGrid(wx.Panel):
             for s in sections:
                 _ = pagePtr.Append(pg.PropertyCategory(_localized[s], s))
                 for name, prop in self.sections[s].items():
+                    if name in prefs.legacy:
+                        # If this is included in the config file only for legacy, don't show it
+                        continue
+
                     item = pagePtr.Append(prop)
 
                     # set the appropriate control to edit the attribute
@@ -473,18 +480,18 @@ class PreferencesDlg(wx.Dialog):
         # add property pages to the manager
         self.proPrefs.addPage(
             'General', 'general', ['general'],
-            'preferences-general48.png')
+            'preferences-general')
         self.proPrefs.addPage(
             'Application', 'app', ['app', 'builder', 'coder'],
-            'preferences-app48.png')
+            'preferences-app')
         self.proPrefs.addPage(
             'Key Bindings', 'keyBindings', ['keyBindings'],
-            'preferences-keyboard48.png')
+            'preferences-keyboard')
         self.proPrefs.addPage(
-            'Hardware', 'hardware', ['hardware'], 'preferences-hardware48.png')
+            'Hardware', 'hardware', ['hardware'], 'preferences-hardware')
         self.proPrefs.addPage(
             'Connections', 'connections', ['connections'],
-            'preferences-conn48.png')
+            'preferences-conn')
         self.proPrefs.populateGrid()
 
         sbPrefs.Add(self.proPrefs, 1, wx.EXPAND)
@@ -496,15 +503,15 @@ class PreferencesDlg(wx.Dialog):
 
         # dialog controls, have builtin localization
         sdbControls = wx.BoxSizer(wx.HORIZONTAL)
-        self.sdbControlsHelp = wx.Button(self.pnlMain, wx.ID_HELP)
+        self.sdbControlsHelp = wx.Button(self.pnlMain, wx.ID_HELP, _translate(" Help "))
         sdbControls.Add(self.sdbControlsHelp, 0,
                         wx.LEFT | wx.ALL | wx.ALIGN_CENTER_VERTICAL,
                         border=3)
         sdbControls.AddStretchSpacer()
         # Add Okay and Cancel buttons
-        self.sdbControlsApply = wx.Button(self.pnlMain, wx.ID_APPLY)
-        self.sdbControlsOK = wx.Button(self.pnlMain, wx.ID_OK)
-        self.sdbControlsCancel = wx.Button(self.pnlMain, wx.ID_CANCEL)
+        self.sdbControlsApply = wx.Button(self.pnlMain, wx.ID_APPLY, _translate(" Apply "))
+        self.sdbControlsOK = wx.Button(self.pnlMain, wx.ID_OK, _translate(" OK "))
+        self.sdbControlsCancel = wx.Button(self.pnlMain, wx.ID_CANCEL, _translate(" Cancel "))
         if sys.platform == "win32":
             btns = [self.sdbControlsOK, self.sdbControlsApply, self.sdbControlsCancel]
         else:
@@ -542,17 +549,8 @@ class PreferencesDlg(wx.Dialog):
         # valid themes
         themePath = self.GetTopLevelParent().app.prefs.paths['themes']
         self.themeList = []
-        for themeFile in os.listdir(themePath):
-            try:
-                # Load theme from json file
-                with open(os.path.join(themePath, themeFile), "rb") as fp:
-                    theme = json.load(fp)
-                # Add themes to list only if min spec is defined
-                base = theme['base']
-                if all(key in base for key in ['bg', 'fg', 'font']):
-                    self.themeList += [themeFile.replace('.json', '')]
-            except:
-                pass
+        for file in Path(themePath).glob("*.json"):
+            self.themeList.append(file.stem)
 
         # get sound devices for "audioDevice" property
         try:
@@ -670,6 +668,18 @@ class PreferencesDlg(wx.Dialog):
                     self.proPrefs.addFileItem(
                         sectionName, pLabel, prefName, thisPref,
                         helpText=helpText)
+                # window backend items
+                elif prefName == 'winType':
+                    from psychopy.visual.backends import getAvailableWinTypes
+                    labels = getAvailableWinTypes()
+                    default = labels.index('pyglet')  # is always included
+                    self.proPrefs.addEnumItem(
+                            sectionName,
+                            pLabel,
+                            prefName,
+                            labels=labels,
+                            values=[i for i in range(len(labels))],
+                            value=default, helpText=helpText)
                 # # audio latency mode for the PTB driver
                 elif prefName == 'audioLatencyMode':
                     # get the labels from above
@@ -744,7 +754,7 @@ class PreferencesDlg(wx.Dialog):
                         item = self.proPrefs.sections[sectionName][prefName]
                         for i in range(len(item.GetChoices())):
                             choice = item.GetChoices()[i]
-                            icon = self.app.iconCache.getBitmap(choice.Text)
+                            icon = icons.ButtonIcon(stem=choice.Text).bitmap
                             choice.SetBitmap(icon)
                 # # lists are given a property that can edit and reorder items
                 elif thisSpec.startswith('list'):  # list
@@ -782,8 +792,7 @@ class PreferencesDlg(wx.Dialog):
                         self.fontList[thisPref]
                     continue
                 if prefName in ('theme',):
-                    self.prefsCfg[sectionName][prefName] = \
-                        self.themeList[thisPref]
+                    self.app.theme = self.prefsCfg[sectionName][prefName] = self.themeList[thisPref]
                     continue
                 elif prefName == 'audioDevice':
                     self.prefsCfg[sectionName][prefName] = \
@@ -861,7 +870,6 @@ class PreferencesDlg(wx.Dialog):
             self.app.builder.updateAllViews()
 
         # after validation, update the UI
-        self.app.theme = self.app.theme
         self.updateFramesUI()
 
     def updateFramesUI(self):
