@@ -38,7 +38,8 @@ colorExamples = {
 
 # Dict of named colours
 colorNames = {
-    "none": (0, 0, 0),
+    "none": (0, 0, 0, 0),
+    "transparent": (0, 0, 0, 0),
     "aliceblue": (0.882352941176471, 0.945098039215686, 1),
     "antiquewhite": (0.96078431372549, 0.843137254901961, 0.686274509803922),
     "aqua": (-1, 1, 1),
@@ -258,6 +259,7 @@ class Color:
     """
     def __init__(self, color=None, space=None, contrast=None, conematrix=None):
         self._cache = {}
+        self._renderCache = {}
         self.contrast = contrast if isinstance(contrast, (int, float)) else 1
         self.alpha = 1
         self.valid = False
@@ -376,6 +378,9 @@ class Color:
         """
         if space not in colorSpaces:
             raise ValueError(f"{space} is not a valid color space")
+        # If value is cached, return it rather than doing calculations again
+        if space in self._renderCache:
+            return self._renderCache[space]
         # Transform contrast to match rgb
         contrast = self.contrast
         contrast = np.reshape(contrast, (-1, 1))
@@ -488,6 +493,42 @@ class Color:
         dupe.valid = self.valid
         return dupe
 
+    def getReadable(self, contrast=4.5/21):
+        """
+        Get a color which will stand out and be easily readable against this
+        one. Useful for choosing text colors based on background color.
+
+        Parameters
+        ----------
+        contrast : float
+            Desired perceived contrast between the two colors, between 0 (the
+            same color) and 1 (as opposite as possible). Default is the
+            w3c recommended minimum of 4.5/21 (dividing by 21 to adjust for
+            sRGB units).
+
+        Returns
+        -------
+        colors.Color
+            A contrasting color to this color.
+        """
+        # adjust contrast to sRGB
+        contrast *= 21
+        # get value as rgb1
+        rgb = self.rgb1
+        # convert to srgb
+        srgb = rgb**2.2 * [0.2126, 0.7151, 0.0721]
+        # apply contrast adjustment
+        if np.sum(srgb) < 0.5:
+            srgb = (srgb + 0.05) * contrast
+        else:
+            srgb = (srgb + 0.05) / contrast
+        # convert back
+        rgb = (srgb / [0.2126, 0.7151, 0.0721])**(1/2.2)
+        # cap
+        rgb = np.clip(rgb, 0, 1)
+        # Return new color
+        return Color(rgb, "rgb1")
+
     @property
     def alpha(self):
         """How opaque (1) or transparent (0) this color is. Synonymous with
@@ -516,6 +557,20 @@ class Color:
             value = max(value, 0)
         # Set value
         self._alpha = value
+        # Clear render cache
+        self._renderCache = {}
+
+    @property
+    def contrast(self):
+        if hasattr(self, "_contrast"):
+            return self._contrast
+
+    @contrast.setter
+    def contrast(self, value):
+        # Set value
+        self._contrast = value
+        # Clear render cache
+        self._renderCache = {}
 
     @property
     def opacity(self):
@@ -573,12 +628,13 @@ class Color:
         # Validate
         color, space = self.validate(color, space='rgb')
         if space != 'rgb':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Set color
         self._franca = color
         # Clear outdated values from cache
         self._cache = {'rgb': color}
+        self._renderCache = {}
 
     @property
     def rgba255(self):
@@ -607,12 +663,13 @@ class Color:
         # Validate
         color, space = self.validate(color, space='rgb255')
         if space != 'rgb255':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Iterate through values and do conversion
         self.rgb = 2 * (color / 255 - 0.5)
         # Clear outdated values from cache
         self._cache = {'rgb255': color}
+        self._renderCache = {}
 
     @property
     def rgba1(self):
@@ -641,12 +698,13 @@ class Color:
         # Validate
         color, space = self.validate(color, space='rgb1')
         if space != 'rgb1':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Iterate through values and do conversion
         self.rgb = 2 * (color - 0.5)
         # Clear outdated values from cache
         self._cache = {'rgb1': color}
+        self._renderCache = {}
 
     @property
     def hex(self):
@@ -687,7 +745,7 @@ class Color:
         # Validate
         color, space = self.validate(color, space='hex')
         if space != 'hex':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         if len(color) > 1:
             # Handle arrays
@@ -718,6 +776,7 @@ class Color:
         self.rgb255 = rgb255
         # Clear outdated values from cache
         self._cache = {'hex': color}
+        self._renderCache = {}
 
     @property
     def named(self):
@@ -760,7 +819,7 @@ class Color:
         # Validate
         color, space = self.validate(color=color, space='named')
         if space != 'named':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Retrieve named colour
         if len(color) > 1:
@@ -779,6 +838,7 @@ class Color:
                 self.alpha = 0
         # Clear outdated values from cache
         self._cache = {'named': color}
+        self._renderCache = {}
 
     @property
     def hsva(self):
@@ -803,12 +863,13 @@ class Color:
         # Validate
         color, space = self.validate(color=color, space='hsv')
         if space != 'hsv':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Apply via rgba255
         self.rgb = ct.hsv2rgb(color)
         # Clear outdated values from cache
         self._cache = {'hsv': color}
+        self._renderCache = {}
 
     @property
     def lmsa(self):
@@ -833,12 +894,13 @@ class Color:
         # Validate
         color, space = self.validate(color=color, space='lms')
         if space != 'lms':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Apply via rgba255
         self.rgb = ct.lms2rgb(color, self.conematrix)
         # Clear outdated values from cache
         self._cache = {'lms': color}
+        self._renderCache = {}
 
     @property
     def dkla(self):
@@ -864,12 +926,13 @@ class Color:
         # Validate
         color, space = self.validate(color=color, space='dkl')
         if space != 'dkl':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Apply via rgba255
         self.rgb = ct.dkl2rgb(color, self.conematrix)
         # Clear outdated values from cache
         self._cache = {'dkl': color}
+        self._renderCache = {}
 
     @property
     def dklaCart(self):
@@ -895,12 +958,13 @@ class Color:
         # Validate
         color, space = self.validate(color=color, space='dklCart')
         if space != 'dkl':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Apply via rgba255
         self.rgb = ct.dklCart2rgb(color, self.conematrix)
         # Clear outdated values from cache
         self._cache = {'dklCart': color}
+        self._renderCache = {}
 
     @property
     def srgb(self):
@@ -916,12 +980,13 @@ class Color:
         # Validate
         color, space = self.validate(color=color, space='srgb')
         if space != 'srgb':
-            setattr(self, space)
+            setattr(self, space, color)
             return
         # Apply via rgba255
         self.rgb = ct.srgbTF(color, reverse=True)
         # Clear outdated values from cache
         self._cache = {'srgb': color}
+        self._renderCache = {}
 
     # removing for now
     # @property
@@ -935,12 +1000,13 @@ class Color:
     #     # Validate
     #     color, space = self.validate(color=color, space='rec709TF')
     #     if space != 'rec709TF':
-    #         setattr(self, space)
+    #         setattr(self, space, color)
     #         return
     #     # Apply via rgba255
     #     self.rgb = ct.rec709TF(color, reverse=True)
     #     # Clear outdated values from cache
     #     self._cache = {'rec709TF': color}
+    #     self._renderCache = {}
 
 
 # ------------------------------------------------------------------------------
