@@ -31,12 +31,6 @@
 # OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 # DAMAGE.
 
-from __future__ import absolute_import, division, print_function
-
-from builtins import input
-from builtins import zip
-from builtins import range
-from builtins import object
 __all__ = ['QuestObject']
 
 import math
@@ -52,7 +46,7 @@ def getinf(x):
     return num.nonzero( num.isinf( num.atleast_1d(x) ) )
 
 
-class QuestObject(object):
+class QuestObject():
 
     """Measure threshold using a Weibull psychometric function.
 
@@ -337,8 +331,8 @@ class QuestObject(object):
             raise RuntimeError('prior pdf is not finite')
 
         # recompute the pdf from the historical record of trials
-        for intensity, response in zip(self.intensity,self.response):
-            inten = max(-1e10,min(1e10,intensity)) # make intensity finite
+        for intensity, response in zip(self.intensity, self.response):
+            inten = max(-1e10,min(1e10, intensity)) # make intensity finite
             ii = len(self.pdf) + self.i-round((inten-self.tGuess)/self.grain)-1
             if ii[0]<0:
                 ii = ii-ii[0]
@@ -348,7 +342,7 @@ class QuestObject(object):
             if not num.allclose(ii,iii):
                 raise ValueError('truncation error')
             self.pdf = self.pdf*self.s2[response,iii]
-            if self.normalizePdf and k%100==0:
+            if self.normalizePdf and ii % 100 == 0:
                 self.pdf = self.pdf/num.sum(self.pdf) # avoid underflow; keep the pdf normalized
         if self.normalizePdf:
             self.pdf = self.pdf/num.sum(self.pdf) # avoid underflow; keep the pdf normalized
@@ -429,19 +423,17 @@ def demo():
 
     tActual = None
     while tActual is None:
-        sys.stdout.write('Specify true threshold of simulated observer: ')
-        input = input()
+        inputStr = input('Specify true threshold of simulated observer: ')
         try:
-            tActual = float(input)
+            tActual = float(inputStr)
         except Exception:
             pass
 
     tGuess = None
     while tGuess is None:
-        sys.stdout.write('Estimate threshold: ')
-        input = input()
+        inputStr = input('Estimate threshold: ')
         try:
-            tGuess = float(input)
+            tGuess = float(inputStr)
         except Exception:
             pass
 
