@@ -188,7 +188,7 @@ class StairHandler(_BaseTrialHandler):
         self.currentDirection = 'start'
         # correct since last stim change (minus are incorrect):
         self.correctCounter = 0
-        self._nextIntensity = self.startVal
+        self.intensity = self.startVal
         self.minVal = minVal
         self.maxVal = maxVal
         self.autoLog = autoLog
@@ -202,6 +202,16 @@ class StairHandler(_BaseTrialHandler):
     def __iter__(self):
         return self
 
+    @property
+    def intensity(self):
+        """The intensity (level) of the current staircase"""
+        return self._nextIntensity
+
+    @intensity.setter
+    def intensity(self, intensity):
+        """The intensity (level) of the current staircase"""
+        self._nextIntensity = intensity
+
     def addResponse(self, result, intensity=None):
         """Add a 1 or 0 to signify a correct / detected or
         incorrect / missed trial.
@@ -857,6 +867,7 @@ class QuestHandler(StairHandler):
         self.startValSd = startValSd
         self.pThreshold = pThreshold
         self.stopInterval = stopInterval
+        # NB there is also _nextIntensity
         self._questNextIntensity = startVal
         self._range = range
 
@@ -873,6 +884,8 @@ class QuestHandler(StairHandler):
         self._exp = None
         self.autoLog = autoLog
 
+    # NB we inherit self.intensity from StairHandler
+
     @property
     def beta(self):
         return self._quest.beta
@@ -1952,6 +1965,8 @@ class MultiStairHandler(_BaseTrialHandler):
                     exp.addData(self.name + '.stepType', stair.stepType)
 
                 exp.addData(self.name + '.intensity', self._nextIntensity)
+
+            self._trialAborted = False  # reset this flag
             return self._nextIntensity, self.currentStaircase.condition
         else:
             raise StopIteration
@@ -1980,6 +1995,55 @@ class MultiStairHandler(_BaseTrialHandler):
         else:
             raise ValueError('Unknown randomization method requested.')
 
+    @property
+    def intensity(self):
+        """The intensity (level) of the current staircase"""
+        return self.currentStaircase._nextIntensity
+
+    @intensity.setter
+    def intensity(self, intensity):
+        """The intensity (level) of the current staircase"""
+        self.currentStaircase._nextIntensity = intensity
+
+    def abortCurrentTrial(self, action='random'):
+        """Abort the current trial (staircase).
+
+        Calling this during an experiment abort the current staircase used this
+        trial. The current staircase will be reshuffled into available 
+        staircases depending on the `action` parameter.
+
+        Parameters
+        ----------
+        action : str
+            Action to take with the aborted trial. Can be either of `'random'`,
+            or `'append'`. The default action is `'random'`.
+
+        Notes
+        -----
+        * When using `action='random'`, the RNG state for the trial handler is
+          not used.
+
+        """
+        # check if value for parameter `action` is valid
+        if not isinstance(action, str):  # type checks for params
+            raise TypeError(
+                "Parameter `action` specified incorrect type, must be `str`.")
+        
+        # reinsert the current staircase into the list of running staircases
+        if action == 'append':
+            self.thisPassRemaining.append(self.currentStaircase)
+        elif action == 'random':
+            self.thisPassRemaining.append(self.currentStaircase)
+            # shuffle using the numpy RNG to preserve state
+            np.random.shuffle(self.thisPassRemaining)
+        else:
+            raise ValueError(
+                "Value for parameter `action` must be either 'random' or "
+                "'append'.")
+
+        # set flag to indicate that the trial was aborted
+        self._trialAborted = True  
+
     def addResponse(self, result, intensity=None):
         """Add a 1 or 0 to signify a correct / detected or
         incorrect / missed trial
@@ -2175,3 +2239,7 @@ class MultiStairHandler(_BaseTrialHandler):
             label = thisStair.condition['label']
             thisStair.saveAsText(fileName='stdout', delim=delim,
                                  matrixOnly=thisMatrixOnly)
+
+
+if __name__ == "__main__":
+    pass
