@@ -39,6 +39,7 @@ class ButtonComponent(BaseVisualComponent):
     """
     categories = ['Responses']
     targets = ['PsychoPy', 'PsychoJS']
+    version = "2021.1.0"
     iconFile = Path(__file__).parent / 'button.png'
     tooltip = _translate('Button: A clickable textbox')
     beta = True
@@ -48,7 +49,7 @@ class ButtonComponent(BaseVisualComponent):
                  stopType='duration (s)', stopVal=1.0,
                  startEstim='', durationEstim='',
                  text=_translate("Click here"), font='Arvo',
-                 pos=(0, 0), size="", padding="", anchor='center', units='from exp settings', ori=0,
+                 pos=(0, 0), size=(0.5, 0.5), padding="", anchor='center', units='from exp settings', ori=0,
                  color="white", fillColor="darkgrey", borderColor="None", borderWidth=0, colorSpace='rgb', opacity="",
                  letterHeight=0.05, bold=True, italic=False,
                  callback="", save='every click', timeRelativeTo='button onset', forceEndRoutine=True, oncePerClick=True):
@@ -74,14 +75,14 @@ class ButtonComponent(BaseVisualComponent):
         ]
         # params
         _allow3 = ['constant', 'set every repeat', 'set every frame']  # list
-        self.params['color'].label = _translate("Text Color")
+        self.params['color'].label = _translate("Text color")
 
         self.params['forceEndRoutine'] = Param(
             forceEndRoutine, valType='bool', inputType="bool", categ='Basic',
             updates='constant', direct=False,
             hint=_translate("Should a response force the end of the Routine "
                             "(e.g end the trial)?"),
-            label=_localized['forceEndRoutine'])
+            label=_translate("Force end of Routine"))
 
         # If force end routine, then once per click doesn't make sense
         self.depends += [
@@ -98,44 +99,44 @@ class ButtonComponent(BaseVisualComponent):
             oncePerClick, valType='bool', inputType="bool", allowedTypes=[], categ='Basic',
             updates='constant',
             hint=_translate("Should the callback run once per click (True), or each frame until click is released (False)"),
-            label=_localized['oncePerClick']
+            label=_translate("Run once per click")
         )
         self.params['callback'] = Param(
             callback, valType='extendedCode', inputType="multi", allowedTypes=[], categ='Basic',
             updates='constant',
             hint=_translate("Code to run when button is clicked"),
-            label=_localized['callback'])
+            label=_translate("Callback function"))
         self.params['text'] = Param(
             text, valType='str', inputType="single", allowedTypes=[], categ='Basic',
             updates='constant', allowedUpdates=_allow3[:],  # copy the list
             hint=_translate("The text to be displayed"),
-            label=_localized['text'])
+            label=_translate("Button text"))
         self.params['font'] = Param(
             font, valType='str', inputType="single", allowedTypes=[], categ='Formatting',
             updates='constant', allowedUpdates=_allow3[:],  # copy the list
             hint=_translate("The font name (e.g. Comic Sans)"),
-            label=_localized['font'])
+            label=_translate("Font"))
         self.params['letterHeight'] = Param(
             letterHeight, valType='num', inputType="single", allowedTypes=[], categ='Formatting',
             updates='constant', allowedUpdates=_allow3[:],  # copy the list
             hint=_translate("Specifies the height of the letter (the width"
                             " is then determined by the font)"),
-            label=_localized['letterHeight'])
+            label=_translate("Letter height"))
         self.params['italic'] = Param(
             italic, valType='bool', inputType="bool", allowedTypes=[], categ='Formatting',
             updates='constant',
             hint=_translate("Should text be italic?"),
-            label=_localized['italic'])
+            label=_translate("Italic"))
         self.params['bold'] = Param(
             bold, valType='bool', inputType="bool", allowedTypes=[], categ='Formatting',
             updates='constant',
             hint=_translate("Should text be bold?"),
-            label=_localized['bold'])
+            label=_translate("Bold"))
         self.params['padding'] = Param(
             padding, valType='num', inputType="single", allowedTypes=[], categ='Layout',
             updates='constant', allowedUpdates=_allow3[:],
             hint=_translate("Defines the space between text and the textbox border"),
-            label=_localized['padding'])
+            label=_translate("Padding"))
         self.params['anchor'] = Param(
             anchor, valType='str', inputType="choice", categ='Layout',
             allowedVals=['center',
@@ -150,19 +151,19 @@ class ButtonComponent(BaseVisualComponent):
                          ],
             updates='constant',
             hint=_translate("Should text anchor to the top, center or bottom of the box?"),
-            label=_localized['anchor'])
+            label=_translate("Anchor"))
         self.params['borderWidth'] = Param(
             borderWidth, valType='num', inputType="single", allowedTypes=[], categ='Appearance',
             updates='constant', allowedUpdates=_allow3[:],
             hint=_translate("Textbox border width"),
-            label=_localized['borderWidth'])
+            label=_translate("Border width"))
         self.params['save'] = Param(
             save, valType='str', inputType="choice", categ='Data',
             allowedVals=['first click', 'last click', 'every click', 'none'],
             hint=_translate(
                 "What clicks on this button should be saved to the data output?"),
             direct=False,
-            label=_localized['save'])
+            label=_translate("Record clicks"))
         self.params['timeRelativeTo'] = Param(
             timeRelativeTo, valType='str', inputType="choice", categ='Data',
             allowedVals=['button onset', 'experiment', 'routine'],
@@ -171,7 +172,7 @@ class ButtonComponent(BaseVisualComponent):
             hint=_translate(
                 "What should the values of mouse.time should be "
                 "relative to?"),
-            label=_localized['timeRelativeTo'])
+            label=_translate("Time relative to"))
 
 
     def writeInitCode(self, buff):
@@ -182,6 +183,7 @@ class ButtonComponent(BaseVisualComponent):
             unitsStr = "units=%(units)s," % self.params
         # do writing of init
         inits = getInitVals(self.params, 'PsychoPy')
+        inits['depth'] = -self.getPosInRoutine()
         code = (
                 "%(name)s = visual.ButtonStim(win, \n"
         )
@@ -198,7 +200,8 @@ class ButtonComponent(BaseVisualComponent):
                     "bold=%(bold)s, italic=%(italic)s,\n"
                     "padding=%(padding)s,\n"
                     "anchor=%(anchor)s,\n"
-                    "name='%(name)s'\n"
+                    "name='%(name)s',\n"
+                    "depth=%(depth)s\n"
         )
         buff.writeIndentedLines(code % inits)
         buff.setIndentLevel(-1, relative=True)
@@ -210,6 +213,7 @@ class ButtonComponent(BaseVisualComponent):
 
     def writeInitCodeJS(self, buff):
         inits = getInitVals(self.params, 'PsychoJS')
+        inits['depth'] = -self.getPosInRoutine()
 
         code = (
             "%(name)s = new visual.ButtonStim({\n"
@@ -226,7 +230,8 @@ class ButtonComponent(BaseVisualComponent):
                 "colorSpace: %(colorSpace)s,\n"
                 "pos: %(pos)s,\n"
                 "letterHeight: %(letterHeight)s,\n"
-                "size: %(size)s\n"
+                "size: %(size)s,\n"
+                "depth: %(depth)s\n"
         )
         buff.writeIndentedLines(code % inits)
         buff.setIndentLevel(-1, relative=True)
@@ -236,99 +241,118 @@ class ButtonComponent(BaseVisualComponent):
         )
         buff.writeIndentedLines(code % inits)
 
+    def writeRoutineStartCode(self, buff):
+        # Write base code
+        BaseVisualComponent.writeRoutineStartCode(self, buff)
+        # If mouse is on button and already clicked, mark as `wasClicked` so button knows click is not new
+        code = (
+            "# reset %(name)s to account for continued clicks & clear times on/off\n"
+            "%(name)s.reset()\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+    def writeRoutineStartCodeJS(self, buff):
+        # Write base code
+        BaseVisualComponent.writeRoutineStartCodeJS(self, buff)
+        # If mouse is on button and already clicked, mark as `wasClicked` so button knows click is not new
+        code = (
+            "// reset %(name)s to account for continued clicks & clear times on/off\n"
+            "%(name)s.reset()\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
     def writeFrameCode(self, buff):
-        BaseVisualComponent.writeFrameCode(self, buff)
-        # do writing of init
-        inits = getInitVals(self.params, 'PsychoPy')
         # Get callback from params
-        callback = inits['callback']
-        if inits['callback']:
-            callback = str(callback)
+        if self.params['callback'].val:
+            callback = str(self.params['callback'].val)
         else:
-            callback = ""
+            callback = "pass"
         # String to get time
-        if inits['timeRelativeTo'] == 'button onset':
+        if self.params['timeRelativeTo'] == 'button onset':
             timing = "%(name)s.buttonClock.getTime()"
-        elif inits['timeRelativeTo'] == 'experiment':
+        elif self.params['timeRelativeTo'] == 'experiment':
             timing = "globalClock.getTime()"
-        elif inits['timeRelativeTo'] == 'routine':
+        elif self.params['timeRelativeTo'] == 'routine':
             timing = "routineTimer.getTime()"
         else:
             timing = "globalClock.getTime()"
-        # Assemble code
+
+        # Write comment
         code = (
-            f"if %(name)s.status == STARTED:\n"
+            "# *%(name)s* updates\n"
         )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(1, relative=True)
-        code = (
-                f"# check whether %(name)s has been pressed\n"
-                f"if %(name)s.isClicked:\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(1, relative=True)
-        code = (
-                    f"if not %(name)s.wasClicked:\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(1, relative=True)
-        code = (
-                        f"%(name)s.timesOn.append({timing}) # store time of first click\n"
-                        f"%(name)s.timesOff.append({timing}) # store time clicked until\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(-1, relative=True)
-        code = (
-                    f"else:\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(1, relative=True)
-        code = (
-                        f"%(name)s.timesOff[-1] = {timing} # update time clicked until\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(-1, relative=True)
-        if self.params['oncePerClick'].val or self.params['forceEndRoutine'].val:
+        buff.writeIndentedLines(code % self.params)
+
+        # Start code
+        indented = self.writeStartTestCode(buff)
+        if indented:
             code = (
-                f"if not %(name)s.wasClicked:\n"
+                "%(name)s.setAutoDraw(True)\n"
             )
-            buff.writeIndentedLines(code % inits)
+            buff.writeIndentedLines(code % self.params)
+        buff.setIndentLevel(-indented, relative=True)
+
+        # Active code
+        indented = self.writeActiveTestCode(buff)
+        if indented:
+            code = (
+                    f"# check whether %(name)s has been pressed\n"
+                    f"if %(name)s.isClicked:\n"
+            )
+            buff.writeIndentedLines(code % self.params)
+            # If clicked...
             buff.setIndentLevel(1, relative=True)
-            if self.params['forceEndRoutine'].val:
+            code = (
+                        f"if not %(name)s.wasClicked:\n"
+                        f"    # if this is a new click, store time of first click and clicked until\n"
+                        f"    %(name)s.timesOn.append({timing})\n"
+                        f"    %(name)s.timesOff.append({timing})\n"
+                        f"elif len(%(name)s.timesOff):\n"
+                        f"    # if click is continuing from last frame, update time of clicked until\n"
+                        f"    %(name)s.timesOff[-1] = {timing}\n"
+            )
+            buff.writeIndentedLines(code % self.params)
+            # Handle force end routine
+            if self.params['forceEndRoutine']:
                 code = (
-                        f"continueRoutine = False  # end routine when %(name)s is clicked\n"
+                        f"if not %(name)s.wasClicked:\n"
+                        f"    # end routine when %(name)s is clicked\n"
+                        f"    continueRoutine = False\n"
                 )
-                buff.writeIndentedLines(code % inits)
-            if self.params['oncePerClick'].val:
-                buff.writeIndentedLines(callback % inits)
+                buff.writeIndentedLines(code % self.params)
+            # Callback code
+            if self.params['oncePerClick']:
+                code = (
+                        f"if not %(name)s.wasClicked:\n"
+                )
+                buff.writeIndentedLines(code % self.params)
+                buff.setIndentLevel(1, relative=True)
+            code = (
+                        f"# run callback code when %(name)s is clicked\n"
+            )
+            buff.writeIndentedLines(code % self.params)
+            buff.writeIndentedLines(callback % self.params)
+            if self.params['oncePerClick']:
                 buff.setIndentLevel(-1, relative=True)
-        if not self.params['oncePerClick'].val:
-            buff.writeIndentedLines(callback % inits)
+            buff.setIndentLevel(-1, relative=True)
+        buff.setIndentLevel(-indented, relative=True)
+
+        # Update wasClicked
         code = (
-                    f"%(name)s.wasClicked = True  # if %(name)s is still clicked next frame, it is not a new click\n"
+            f"# take note of whether %(name)s was clicked, so that next frame we know if clicks are new\n"
+            f"%(name)s.wasClicked = %(name)s.isClicked and %(name)s.status == STARTED\n"
         )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(-1, relative=True)
-        code = (
-                f"else:\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(1, relative=True)
-        code = (
-                    f"%(name)s.wasClicked = False  # if %(name)s is clicked next frame, it is a new click\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(-2, relative=True)
-        code = (
-            f"else:\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(1, relative=True)
-        code = (
-            f"%(name)s.wasClicked = False  # if %(name)s is clicked next frame, it is a new click\n"
-        )
-        buff.writeIndentedLines(code % inits)
-        buff.setIndentLevel(-1, relative=True)
+        buff.writeIndentedLines(code % self.params)
+
+        # Stop code
+        indented = self.writeStopTestCode(buff)
+        if indented:
+            code = (
+                "%(name)s.setAutoDraw(False)\n"
+            )
+            buff.writeIndentedLines(code % self.params)
+            # to get out of the if statement
+            buff.setIndentLevel(-indented, relative=True)
 
     def writeFrameCodeJS(self, buff):
         BaseVisualComponent.writeFrameCodeJS(self, buff)
@@ -336,7 +360,7 @@ class ButtonComponent(BaseVisualComponent):
         inits = getInitVals(self.params, 'PsychoJS')
         # Get callback from params
         callback = inits['callback']
-        if inits['callback'].val:
+        if inits['callback'].val not in [None, "None", "none", "undefined"]:
             callback = translatePythonToJavaScript(str(callback))
         else:
             callback = ""
