@@ -62,7 +62,7 @@ class JoyButtonsComponent(BaseComponent):
             updates='constant',
             allowedUpdates=['constant', 'set every repeat'],
             hint=(msg),
-            label=_localized['allowedKeys'])
+            label=_translate("Allowed buttons"))
 
         msg = _translate("Choose which (if any) responses to store at the "
                          "end of a trial")
@@ -71,7 +71,7 @@ class JoyButtonsComponent(BaseComponent):
             allowedVals=['last key', 'first key', 'all keys', 'nothing'],
             updates='constant', direct=False,
             hint=msg,
-            label=_localized['store'])
+            label=_translate("Store"))
 
         msg = _translate("Should a response force the end of the Routine "
                          "(e.g end the trial)?")
@@ -79,7 +79,7 @@ class JoyButtonsComponent(BaseComponent):
             forceEndRoutine, valType='bool', inputType="bool", allowedTypes=[], categ='Basic',
             updates='constant',
             hint=msg,
-            label=_localized['forceEndRoutine'])
+            label=_translate("Force end of Routine"))
 
         msg = _translate("Do you want to save the response as "
                          "correct/incorrect?")
@@ -87,7 +87,16 @@ class JoyButtonsComponent(BaseComponent):
             storeCorrect, valType='bool', inputType="bool", allowedTypes=[], categ='Data',
             updates='constant',
             hint=msg,
-            label=_localized['storeCorrect'])
+            label=_translate("Store correct"))
+
+        self.depends += [  # allows params to turn each other off/on
+            {"dependsOn": "storeCorrect",  # must be param name
+             "condition": "== True",  # val to check for
+             "param": "correctAns",  # param property to alter
+             "true": "enable",  # what to do with param if condition is True
+             "false": "disable",  # permitted: hide, show, enable, disable
+             }
+        ]
 
         msg = _translate(
             "What is the 'correct' key? Might be helpful to add a "
@@ -97,7 +106,7 @@ class JoyButtonsComponent(BaseComponent):
             correctAns, valType='list', inputType="single", allowedTypes=[], categ='Data',
             updates='constant',
             hint=msg,
-            label=_localized['correctAns'])
+            label=_translate("Correct answer"))
 
         msg = _translate(
             "A reaction time to a visual stimulus should be based on when "
@@ -106,7 +115,7 @@ class JoyButtonsComponent(BaseComponent):
             syncScreenRefresh, valType='bool', inputType="bool", categ='Data',
             updates='constant',
             hint=msg,
-            label=_localized['syncScreenRefresh'])
+            label=_translate("Sync RT with screen"))
 
         msg = _translate(
             "Device number, if you have multiple devices which"
@@ -115,7 +124,7 @@ class JoyButtonsComponent(BaseComponent):
             deviceNumber, valType='int', inputType="int", allowedTypes=[], categ='Hardware',
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized['deviceNumber'])
+            label=_translate("Device number"))
 
     def writeStartCode(self, buff):
         code = ("from psychopy.hardware import joystick as joysticklib  "
@@ -231,66 +240,61 @@ class JoyButtonsComponent(BaseComponent):
         buff.writeIndented("\n")
         buff.writeIndented("# *%s* updates\n" % self.params['name'])
         # writes an if statement to determine whether to draw etc
-        self.writeStartTestCode(buff)
-        buff.writeIndented("%(name)s.status = STARTED\n" % self.params)
+        allowedKeysIsVar = (valid_var_re.match(str(allowedKeys)) and not allowedKeys == 'None')
+        indented = self.writeStartTestCode(buff)
+        if indented:
+            if allowedKeysIsVar:
+                # if it looks like a variable, check that the variable is suitable
+                # to eval at run-time
+                code = ("# AllowedKeys looks like a variable named `{0}`\n"
+                        "if not type({0}) in [list, tuple, np.ndarray]:\n")
 
-        allowedKeysIsVar = (valid_var_re.match(str(allowedKeys)) and not
-                            allowedKeys == 'None')
+                buff.writeIndentedLines(code.format(allowedKeys))
 
-        if allowedKeysIsVar:
-            # if it looks like a variable, check that the variable is suitable
-            # to eval at run-time
-            code = ("# AllowedKeys looks like a variable named `{0}`\n"
-                    "if not type({0}) in [list, tuple, np.ndarray]:\n")
+                buff.setIndentLevel(1, relative=True)
+                code = ("if type({0}) == int:\n")
+                buff.writeIndentedLines(code.format(allowedKeys))
 
-            buff.writeIndentedLines(code.format(allowedKeys))
+                buff.setIndentLevel(1, relative=True)
+                code = ("{0} = [{0}]\n")
+                buff.writeIndentedLines(code.format(allowedKeys))
+                buff.setIndentLevel(-1, relative=True)
 
-            buff.setIndentLevel(1, relative=True)
-            code = ("if type({0}) == int:\n")
-            buff.writeIndentedLines(code.format(allowedKeys))
+                code = ("elif not (isinstance({0}, str) "
+                        "or isinstance({0}, unicode)):\n")
+                buff.writeIndentedLines(code.format(allowedKeys))
 
-            buff.setIndentLevel(1, relative=True)
-            code = ("{0} = [{0}]\n")
-            buff.writeIndentedLines(code.format(allowedKeys))
-            buff.setIndentLevel(-1, relative=True)
+                buff.setIndentLevel(1, relative=True)
+                code = ("logging.error('AllowedKeys variable `{0}` is "
+                        "not string- or list-like.')\n"
+                        "core.quit()\n")
+                buff.writeIndentedLines(code.format(allowedKeys))
+                buff.setIndentLevel(-1, relative=True)
 
-            code = ("elif not (isinstance({0}, str) "
-                    "or isinstance({0}, unicode)):\n")
-            buff.writeIndentedLines(code.format(allowedKeys))
+                code = (
+                    "elif not ',' in {0}: {0} = eval(({0},))\n"
+                    "else: {0} = eval({0})\n")
+                buff.writeIndentedLines(code.format(allowedKeys))
+                buff.setIndentLevel(-1, relative=True)
 
-            buff.setIndentLevel(1, relative=True)
-            code = ("logging.error('AllowedKeys variable `{0}` is "
-                    "not string- or list-like.')\n"
-                    "core.quit()\n")
-            buff.writeIndentedLines(code.format(allowedKeys))
-            buff.setIndentLevel(-1, relative=True)
+            buff.writeIndented("# joyButtons checking is just starting\n")
 
-            code = (
-                "elif not ',' in {0}: {0} = eval(({0},))\n"
-                "else: {0} = eval({0})\n")
-            buff.writeIndentedLines(code.format(allowedKeys))
-            buff.setIndentLevel(-1, relative=True)
+            if store != 'nothing':
+                if self.params['syncScreenRefresh'].val:
+                    code = ("win.callOnFlip(%(name)s.clock.reset)  # t=0 on next"
+                            " screen flip\n") % self.params
+                else:
+                    code = "%(name)s.clock.reset()  # now t=0\n" % self.params
 
-        buff.writeIndented("# joyButtons checking is just starting\n")
-
-        if store != 'nothing':
-            if self.params['syncScreenRefresh'].val:
-                code = ("win.callOnFlip(%(name)s.clock.reset)  # t=0 on next"
-                        " screen flip\n") % self.params
-            else:
-                code = "%(name)s.clock.reset()  # now t=0\n" % self.params
-
-            buff.writeIndented(code)
+                buff.writeIndented(code)
 
         # to get out of the if statement
-        buff.setIndentLevel(-1, relative=True)
+        buff.setIndentLevel(-indented, relative=True)
+
         # test for stop (only if there was some setting for duration or stop)
-        if self.params['stopVal'].val not in ['', None, -1, 'None']:
-            # writes an if statement to determine whether to draw etc
-            self.writeStopTestCode(buff)
-            buff.writeIndented("%(name)s.status = FINISHED\n" % self.params)
-            # to get out of the if statement
-            buff.setIndentLevel(-2, relative=True)
+        indented = self.writeStopTestCode(buff)
+        # to get out of the if statement
+        buff.setIndentLevel(-indented, relative=True)
 
         buff.writeIndented("if %(name)s.status == STARTED:\n" % self.params)
         buff.setIndentLevel(1, relative=True)  # to get out of if statement
