@@ -62,9 +62,9 @@ class JoystickComponent(BaseComponent):
             save, valType='str', inputType="choice", categ='Data',
             allowedVals=['final', 'on click', 'every frame', 'never'],
             hint=msg, direct=False,
-            label=_localized['saveJoystickState'])
+            label=_translate("Save joystick state"))
 
-        msg = _translate("Should a button press force the end of the routine"
+        msg = _translate("Should a button press force the end of the Routine"
                          " (e.g end the trial)?")
         if forceEndRoutineOnPress is True:
             forceEndRoutineOnPress = 'any click'
@@ -75,7 +75,7 @@ class JoystickComponent(BaseComponent):
             allowedVals=['never', 'any click', 'valid click'],
             updates='constant',
             hint=msg, direct=False,
-            label=_localized['forceEndRoutineOnPress'])
+            label=_translate("End Routine on press"))
 
         msg = _translate("What should the values of joystick.time should be "
                          "relative to?")
@@ -84,7 +84,7 @@ class JoystickComponent(BaseComponent):
             allowedVals=['joystick onset', 'experiment', 'routine'],
             updates='constant', direct=False,
             hint=msg,
-            label=_localized['timeRelativeTo'])
+            label=_translate("Time relative to"))
 
         msg = _translate('A comma-separated list of your stimulus names that '
                          'can be "clicked" by the participant. '
@@ -94,7 +94,7 @@ class JoystickComponent(BaseComponent):
             '', valType='list', inputType="single", categ='Data',
             updates='constant',
             hint=msg,
-            label=_localized['Clickable stimuli'])
+            label=_translate("Clickable stimuli"))
 
         msg = _translate('The params (e.g. name, text), for which you want '
                          'to store the current value, for the stimulus that was'
@@ -105,7 +105,7 @@ class JoystickComponent(BaseComponent):
             'name,', valType='list', inputType="single", categ='Data',
             updates='constant', allowedUpdates=[],
             hint=msg, direct=False,
-            label=_localized['Store params for clicked'])
+            label=_translate("Store params for clicked"))
 
         msg = _translate('Device number, if you have multiple devices which'
                          ' one do you want (0, 1, 2...)')
@@ -114,7 +114,7 @@ class JoystickComponent(BaseComponent):
             deviceNumber, valType='int', inputType="single", allowedTypes=[], categ='Hardware',
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized['deviceNumber'])
+            label=_translate("Device number"))
 
         msg = _translate('Buttons to be read (blank for any) numbers separated by '
                          'commas')
@@ -123,7 +123,7 @@ class JoystickComponent(BaseComponent):
             allowedButtons, valType='list', inputType="single", allowedTypes=[], categ='Data',
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized['allowedButtons'])
+            label=_translate("Allowed buttons"))
 
     @property
     def _clickableParamsList(self):
@@ -376,21 +376,19 @@ class JoystickComponent(BaseComponent):
         buff.writeIndented("# *%s* updates\n" % self.params['name'])
 
         # writes an if statement to determine whether to draw etc
-        self.writeStartTestCode(buff)
-        code = ("{name}.status = STARTED\n")
-        if self.params['timeRelativeTo'].val.lower() == 'joystick onset':
-            code += "{name}.joystickClock.reset()\n"
-        buff.writeIndentedLines(code.format(**self.params))
+        indented = self.writeStartTestCode(buff)
+        if indented:
+            code = ("{name}.status = STARTED\n")
+            if self.params['timeRelativeTo'].val.lower() == 'joystick onset':
+                code += "{name}.joystickClock.reset()\n"
+            buff.writeIndentedLines(code.format(**self.params))
         # to get out of the if statement
-        buff.setIndentLevel(-1, relative=True)
+        buff.setIndentLevel(-indented, relative=True)
 
         # test for stop (only if there was some setting for duration or stop)
-        if self.params['stopVal'].val not in ['', None, -1, 'None']:
-            # writes an if statement to determine whether to draw etc
-            self.writeStopTestCode(buff)
-            buff.writeIndented("%(name)s.status = FINISHED\n" % self.params)
-            # to get out of the if statement
-            buff.setIndentLevel(-2, relative=True)
+        indented = self.writeStopTestCode(buff)
+        # to get out of the if statement
+        buff.setIndentLevel(-indented, relative=True)
 
         # if STARTED and not FINISHED!
         code = ("if %(name)s.status == STARTED:  "
@@ -441,7 +439,7 @@ class JoystickComponent(BaseComponent):
                     #"print({name}.pressedButtons)\n"
                     #"print({name}.newPressedButtons)\n"
                     "[logging.data(\"joystick_{{}}_button: {{}}, pos=({{:1.4f}},{{:1.4f}})\".format("
-                    "{name}.device_number, i, {name}.getX(), {name}.getY()) for i in {name}.pressedButtons]\n"
+                    "{name}.device_number, i, {name}.getX(), {name}.getY())) for i in {name}.pressedButtons]\n"
             )
             buff.writeIndentedLines(code.format(**self.params))
 
