@@ -18,9 +18,6 @@ _localized.update({'movie': _translate('Movie file'),
                    'backend': _translate('backend'),
                    'No audio': _translate('No audio')})
 
-if _localized['backend'] == 'backend': # this is the only non-capitals label
-    _localized['backend'] = 'Backend'
-
 class MovieComponent(BaseVisualComponent):
     """An event class for presenting movie-based stimuli"""
 
@@ -31,13 +28,14 @@ class MovieComponent(BaseVisualComponent):
 
     def __init__(self, exp, parentName, name='movie', movie='',
                  units='from exp settings',
-                 pos=(0, 0), size='', anchor="center", ori=0,
+                 pos=(0, 0), size=(0.5, 0.5), anchor="center", ori=0,
                  startType='time (s)', startVal=0.0,
                  stopType='duration (s)', stopVal=1.0,
                  startEstim='', durationEstim='',
-                 forceEndRoutine=False, backend='moviepy',
-                 loop=False,
-                 noAudio=False):
+                 forceEndRoutine=False, backend='ffpyplayer',
+                 loop=False, volume=1, noAudio=False,
+                 stopWithRoutine=True
+                 ):
         super(MovieComponent, self).__init__(
             exp, parentName, name=name, units=units,
             pos=pos, size=size, ori=ori,
@@ -54,7 +52,7 @@ class MovieComponent(BaseVisualComponent):
 
         # params
         self.params['stopVal'].hint = _translate(
-            "When does the component end? (blank to use the duration of "
+            "When does the Component end? (blank to use the duration of "
             "the media)")
 
         msg = _translate("A filename for the movie (including path)")
@@ -62,36 +60,57 @@ class MovieComponent(BaseVisualComponent):
             movie, valType='file', inputType="file", allowedTypes=[], categ='Basic',
             updates='constant', allowedUpdates=['constant', 'set every repeat'],
             hint=msg,
-            label=_localized['movie'])
+            label=_translate("Movie file"))
 
         msg = _translate("What underlying lib to use for loading movies")
         self.params['backend'] = Param(
             backend, valType='str', inputType="choice", categ='Playback',
-            allowedVals=['moviepy', 'avbin', 'opencv', 'vlc'],
+            allowedVals=['ffpyplayer', 'moviepy', 'opencv', 'vlc'],
             hint=msg, direct=False,
-            label=_localized['backend'])
+            label=_translate("Backend"))
 
         msg = _translate("Prevent the audio stream from being loaded/processed "
                "(moviepy and opencv only)")
         self.params["No audio"] = Param(
             noAudio, valType='bool', inputType="bool", categ='Playback',
             hint=msg,
-            label=_localized['No audio'])
+            label=_translate("No audio"))
+
+        self.depends.append(
+            {"dependsOn": "No audio",  # must be param name
+             "condition": "==True",  # val to check for
+             "param": "volume",  # param property to alter
+             "true": "hide",  # what to do with param if condition is True
+             "false": "show",  # permitted: hide, show, enable, disable
+             }
+        )
+
+        msg = _translate("How loud should audio be played?")
+        self.params["volume"] = Param(
+            volume, valType='num', inputType="float", categ='Playback',
+            hint=msg,
+            label=_translate("Volume"))
 
         msg = _translate("Should the end of the movie cause the end of "
-                         "the routine (e.g. trial)?")
+                         "the Routine (e.g. trial)?")
         self.params['forceEndRoutine'] = Param(
             forceEndRoutine, valType='bool', inputType="bool", allowedTypes=[], categ='Basic',
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized['forceEndRoutine'])
+            label=_translate("Force end of Routine"))
 
         msg = _translate("Whether the movie should loop back to the beginning "
                          "on completion.")
         self.params['loop'] = Param(
             loop, valType='bool', inputType="bool", categ='Playback',
             hint=msg,
-            label=_translate('Loop playback'))
+            label=_translate("Loop playback"))
+        self.params['stopWithRoutine'] = Param(
+            stopWithRoutine, valType='bool', inputType="bool", updates='constant', categ='Playback',
+            hint=_translate(
+                "Should playback cease when the Routine ends? Untick to continue playing "
+                "after the Routine has finished."),
+            label=_translate('Stop with Routine?'))
         self.params['anchor'] = Param(
             anchor, valType='str', inputType="choice", categ='Layout',
             allowedVals=['center',
@@ -106,7 +125,7 @@ class MovieComponent(BaseVisualComponent):
                          ],
             updates='constant',
             hint=_translate("Which point on the stimulus should be anchored to its exact position?"),
-            label=_translate('Anchor'))
+            label=_translate("Anchor"))
 
         # these are normally added but we don't want them for a movie
         del self.params['color']
@@ -176,6 +195,7 @@ class MovieComponent(BaseVisualComponent):
             inits = getInitVals(self.params)
         else:
             inits = copy.deepcopy(self.params)
+        inits['depth'] = -self.getPosInRoutine()
 
         noAudio = '{}'.format(inits['No audio'].val).lower()
         loop = '{}'.format(inits['loop'].val).lower()
@@ -194,50 +214,67 @@ class MovieComponent(BaseVisualComponent):
                 "  units: {units},\n"
                 "  movie: {movie},\n"
                 "  pos: {pos},\n"
+                "  anchor: {anchor},\n"
                 "  size: {size},\n"
                 "  ori: {ori},\n"
                 "  opacity: {opacity},\n"
                 "  loop: {loop},\n"
                 "  noAudio: {noAudio},\n"
+                "  depth: {depth}\n"
                 "  }});\n").format(name=inits['name'],
                                    movie=inits['movie'],
                                    units=inits['units'],
                                    pos=inits['pos'],
+                                   anchor=inits['anchor'],
                                    size=inits['size'],
                                    ori=inits['ori'],
                                    loop=loop,
                                    opacity=inits['opacity'],
-                                   noAudio=noAudio)
+                                   noAudio=noAudio,
+                                   depth=inits['depth'])
         buff.writeIndentedLines(code)
 
     def writeInitCode(self, buff):
-        # If needed then use _writeCreationCode()
-        # Movie could be created here or in writeRoutineStart()
-        if self.params['movie'].updates == 'constant':
-            # create the code using init vals
-            self._writeCreationCode(buff, useInits=True)
+        # Get init values
+        params = getInitVals(self.params)
+        params['depth'] = -self.getPosInRoutine()
+
+        # synonymise "from experiment settings" with None
+        if params["units"].val.lower() == "from exp settings":
+            params["units"].valType = "code"
+            params["units"].val = None
+
+        # Handle old backends
+        if self.params['backend'].val in ('moviepy', 'avbin', 'vlc', 'opencv'):
+            if self.params['movie'].updates == 'constant':
+                # create the code using init vals
+                self._writeCreationCode(buff, useInits=True)
+            return
+
+        code = (
+            "%(name)s = visual.MovieStim(\n"
+        )
+        buff.writeIndentedLines(code % params)
+        buff.setIndentLevel(+1, relative=True)
+        code = (
+            "win, name='%(name)s',\n"
+            "filename=%(movie)s, movieLib=%(backend)s,\n"
+            "loop=%(loop)s, volume=%(volume)s, noAudio=%(No audio)s,\n"
+            "pos=%(pos)s, size=%(size)s, units=%(units)s,\n"
+            "ori=%(ori)s, anchor=%(anchor)s,"
+            "opacity=%(opacity)s, contrast=%(contrast)s,\n"
+            "depth=%(depth)s\n"
+        )
+        buff.writeIndentedLines(code % params)
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+            ")\n"
+        )
+        buff.writeIndentedLines(code % params)
 
     def writeInitCodeJS(self, buff):
-        # If needed then use _writeCreationCodeJS()
-        # Movie could be created here or in writeRoutineStart()
-        if self.params['movie'].updates == 'constant':
-            # create the code using init vals
-            self._writeCreationCodeJS(buff, useInits=True)
-
-    def writeRoutineStartCode(self, buff):
-        # If needed then use _writeCreationCode()
-        # Movie could be created here or in writeInitCode()
-        if self.params['movie'].updates == 'set every repeat':
-            # create the code using params, not vals
-            self._writeCreationCode(buff, useInits=False)
-
-    def writeRoutineStartCodeJS(self, buff):
-        # If needed then use _writeCreationCode()
-        # Movie could be created here or in writeInitCode()
-        if self.params['movie'].updates != 'constant':
-            # create the code using params, not vals (unless set during static component)
-            useInits = 'during' in self.params['movie'].updates
-            self._writeCreationCodeJS(buff, useInits=useInits)
+        # create the code using init vals
+        self._writeCreationCodeJS(buff, useInits=True)
 
     def writeFrameCode(self, buff):
         """Write the code that will be called every frame
@@ -245,19 +282,27 @@ class MovieComponent(BaseVisualComponent):
         buff.writeIndented("\n")
         buff.writeIndented("# *%s* updates\n" % self.params['name'])
         # writes an if statement to determine whether to draw etc
-        self.writeStartTestCode(buff)
-        # buff.writeIndented(
-        #     "%s.seek(0.00001)  # make sure we're at the start\n"
-        #     % (self.params['name']))
-        buff.writeIndented("%s.setAutoDraw(True)\n" % self.params['name'])
+        indented = self.writeStartTestCode(buff)
+        if indented:
+            code = (
+                "%(name)s.setAutoDraw(True)\n"
+            )
+            if self.params['backend'].val not in ('moviepy', 'avbin', 'vlc'):
+                code += "%(name)s.play()\n"
+            buff.writeIndentedLines(code % self.params)
         # because of the 'if' statement of the time test
-        buff.setIndentLevel(-1, relative=True)
-        if self.params['stopVal'].val not in ['', None, -1, 'None']:
-            # writes an if statement to determine whether to draw etc
-            self.writeStopTestCode(buff)
-            buff.writeIndented("%(name)s.setAutoDraw(False)\n" % self.params)
-            # to get out of the if statement
-            buff.setIndentLevel(-2, relative=True)
+        buff.setIndentLevel(-indented, relative=True)
+
+        indented = self.writeStopTestCode(buff)
+        if indented:
+            code = (
+                "%(name)s.setAutoDraw(False)\n"
+            )
+            if self.params['backend'].val not in ('moviepy', 'avbin', 'vlc'):
+                code += "%(name)s.stop()\n"
+            buff.writeIndentedLines(code % self.params)
+        # to get out of the if statement
+        buff.setIndentLevel(-indented, relative=True)
         # set parameters that need updating every frame
         # do any params need updating? (this method inherited from _base)
         if self.checkNeedToUpdate('set every frame'):
@@ -269,7 +314,7 @@ class MovieComponent(BaseVisualComponent):
             buff.setIndentLevel(-1, relative=True)  # to exit the if block
         # do force end of trial code
         if self.params['forceEndRoutine'].val is True:
-            code = ("if %s.status == FINISHED:  # force-end the routine\n"
+            code = ("if %s.isFinished:  # force-end the Routine\n"
                     "    continueRoutine = False\n" %
                     self.params['name'])
             buff.writeIndentedLines(code)
@@ -307,15 +352,23 @@ class MovieComponent(BaseVisualComponent):
             buff.writeIndentedLines("}\n")
         # do force end of trial code
         if self.params['forceEndRoutine'].val is True:
-            code = ("if ({name}.status === PsychoJS.Status.FINISHED) {{  // force-end the routine\n"
+            code = ("if ({name}.status === PsychoJS.Status.FINISHED) {{  // force-end the Routine\n"
                     "    continueRoutine = false;\n"
                     "}}\n".format(**self.params))
             buff.writeIndentedLines(code)
 
     def writeRoutineEndCode(self, buff):
-        # always stop at the end of the routine. (should this be a param?)
-        buff.writeIndentedLines("{name}.stop()\n".format(**self.params))
+        if self.params['stopWithRoutine']:
+            # stop at the end of the Routine, if requested
+            code = (
+                "%(name)s.stop()  # ensure movie has stopped at end of Routine\n"
+            )
+            buff.writeIndentedLines(code % self.params)
 
     def writeRoutineEndCodeJS(self, buff):
-        # always stop at the end of the routine. (should this be a param?)
-        buff.writeIndentedLines("{name}.stop();\n".format(**self.params))
+        if self.params['stopWithRoutine']:
+            # stop at the end of the Routine, if requested
+            code = (
+                "%(name)s.stop();  // ensure movie has stopped at end of Routine\n"
+            )
+            buff.writeIndentedLines(code % self.params)
