@@ -36,7 +36,12 @@ class StaticComponent(BaseComponent):
                  startType='time (s)', startVal=0.0,
                  stopType='duration (s)', stopVal=0.5,
                  startEstim='', durationEstim=''):
-        BaseComponent.__init__(self, exp, parentName, name=name)
+        BaseComponent.__init__(
+            self, exp, parentName, name=name,
+            startType=startType, startVal=startVal,
+            stopType=stopType, stopVal=stopVal,
+            startEstim=startEstim, durationEstim=durationEstim
+        )
         self.updatesList = []  # a list of dicts {compParams, fieldName}
         self.type = 'Static'
         self.url = "https://www.psychopy.org/builder/components/static.html"
@@ -44,7 +49,7 @@ class StaticComponent(BaseComponent):
             "Custom code to be run during the static period (after updates)")
         self.params['code'] = Param("", valType='code', inputType="multi", categ='Custom',
                                     hint=hnt,
-                                    label=_localized['Custom code'])
+                                    label=_translate("Custom code"))
 
     def addComponentUpdate(self, routine, compName, fieldName):
         self.updatesList.append({'compName': compName,
@@ -93,14 +98,14 @@ class StaticComponent(BaseComponent):
         buff.writeIndented(code % self.params)
 
     def writeFrameCode(self, buff):
-        self.writeStartTestCode(buff)
-        # to get out of the if statement
-        buff.setIndentLevel(-1, relative=True)
+        if self.writeStartTestCode(buff):
+            buff.setIndentLevel(-1, relative=True)
         self.writeStopTestCode(buff)
 
     def writeFrameCodeJS(self, buff):
         # Start test
         self.writeStartTestCodeJS(buff)
+        buff.writeIndentedLines("%(name)s.status = PsychoJS.Status.STARTED;\n" % self.params)
         self.writeParamUpdates(buff, target="PsychoJS")
         buff.setIndentLevel(-1, relative=True)
         buff.writeIndentedLines("}\n")
@@ -145,6 +150,7 @@ class StaticComponent(BaseComponent):
                 buff.writeIndentedLines(code % self.params)
                 buff.setIndentLevel(-1, relative=True)
                 buff.writeIndentedLines("}\n")
+        buff.writeIndentedLines("%(name)s.status = PsychoJS.Status.FINISHED;\n" % self.params)
         # Escape stop code indent
         buff.setIndentLevel(-1, relative=True)
         buff.writeIndentedLines("}\n")
@@ -153,7 +159,7 @@ class StaticComponent(BaseComponent):
         """This will be executed as the final component in the routine
         """
         buff.writeIndented("# *%s* period\n" % (self.params['name']))
-        BaseComponent.writeStartTestCode(self, buff)
+        needsUnindent = BaseComponent.writeStartTestCode(self, buff)
 
         if self.params['stopType'].val == 'time (s)':
             durationSecsStr = "%(stopVal)s-t" % (self.params)
@@ -170,6 +176,8 @@ class StaticComponent(BaseComponent):
         vals = (self.params['name'], durationSecsStr)
         buff.writeIndented("%s.start(%s)\n" % vals)
 
+        return needsUnindent
+
     def writeStopTestCode(self, buff):
         """Test whether we need to stop
         """
@@ -216,22 +224,25 @@ class StaticComponent(BaseComponent):
             buff.writeIndented(code % self.params['name'])
             # Do updates
             for update in self.updatesList:
-                # update = {'compName':compName,'fieldName':fieldName,
-                #    'routine':routine}
                 compName = update['compName']
                 fieldName = update['fieldName']
-                # routine = self.exp.routines[update['routine']]
+                # get component
                 if hasattr(compName, 'params'):
-                    prms = compName.params  # it's already a compon so get params
+                    comp = compName
                 else:
-                    # it's a name so get compon and then get params
-                    prms = self.exp.getComponentFromName(str(compName)).params
+                    comp = self.exp.getComponentFromName(str(compName))
+                # component may be disabled or otherwise not present - skip it if so
+                if comp is None:
+                    return
+                # get params
+                prms = comp.params  # it's already a compon so get params
                 # If in JS, prepare resources
                 if target == "PsychoJS" and prms[fieldName].valType == "file":
                     # Do resource manager stuff
                     code = (
                         f"console.log('register and start downloading resources specified by component %(name)s');\n"
                         f"await psychoJS.serverManager.prepareResources(%({fieldName})s);\n"
+                        f"{self.params['name']}.status = PsychoJS.Status.STARTED;\n"
                     )
                     buff.writeIndentedLines(code % prms)
                 # Set values
