@@ -25,14 +25,20 @@ class UnknownComponent(BaseComponent):
                          'installed version of PsychoPy\n(most likely from the '
                          'future)')
 
-    def __init__(self, exp, parentName, name=''):
-        self.type = 'Unknown'
+    def __init__(self, exp, parentName, name='', compType="UnknownComponent"):
+        self.type = compType
         self.exp = exp  # so we can access the experiment if necess
         self.parentName = parentName  # to access the routine too if needed
         self.params = {}
         self.depends = []
         super(UnknownComponent, self).__init__(exp, parentName, name=name)
         self.order += []
+
+    @property
+    def _xml(self):
+        # make XML node with tag from self.type rather than class name
+        return self.makeXmlNode(self.type)
+
     # make sure nothing gets written into experiment for an unknown object
     # class!
 
