@@ -52,49 +52,49 @@ class VariableComponent(BaseComponent):
         self.params['startExpValue'] = Param(
             startExpValue, valType='code', inputType="single", allowedTypes=[], updates='constant', categ='Basic',
             hint=hnt,
-            label=_localized['startExpValue'])
-        hnt = _translate("Set the value for the beginning of each routine.")
+            label=_translate("Experiment start value"))
+        hnt = _translate("Set the value for the beginning of each Routine.")
         self.params['startRoutineValue'] = Param(
             startRoutineValue, valType='code', inputType="single", allowedTypes=[], updates='constant', categ='Basic',
             hint=hnt,
-            label=_localized['startRoutineValue'])
+            label=_translate("Routine start value"))
         hnt = _translate("Set the value for the beginning of every screen refresh.")
         self.params['startFrameValue'] = Param(
             startFrameValue, valType='code', inputType="single", allowedTypes=[], categ='Basic',
             hint=hnt,
-            label=_localized['startFrameValue'])
+            label=_translate("Frame start value"))
         # Save options
-        hnt = _translate('Save the experiment start value in data file.')
+        hnt = _translate("Save the experiment start value in data file.")
         self.params['saveStartExp'] = Param(
             False, valType='bool', inputType="bool", categ='Data',
             updates='constant',
             hint=hnt,
-            label=_localized['saveStartExp'])
-        hnt = _translate('Save the experiment end value in data file.')
+            label=_translate("Save exp start value"))
+        hnt = _translate("Save the experiment end value in data file.")
         self.params['saveEndExp'] = Param(
             False, valType='bool', inputType="bool", categ='Data',
             updates='constant',
             hint=hnt,
-            label=_localized['saveEndExp'])
-        hnt = _translate('Save the routine start value in data file.')
+            label=_translate("Save exp end value"))
+        hnt = _translate("Save the Routine start value in data file.")
         self.params['saveStartRoutine'] = Param(
             False, valType='bool', inputType="bool", categ='Data',
             updates='constant',
             hint=hnt,
-            label=_localized['saveStartRoutine'])
-        hnt = _translate('Save the routine end value in data file.')
+            label=_translate("Save Routine start value"))
+        hnt = _translate("Save the Routine end value in data file.")
         self.params['saveEndRoutine'] = Param(
             True, valType='bool', inputType="bool", categ='Data',
             updates='constant',
             hint=hnt,
-            label=_localized['saveEndRoutine'])
-        hnt = _translate('Save choice of frame value in data file.')
+            label=_translate("Save Routine end value"))
+        hnt = _translate("Save choice of frame value in data file.")
         self.params['saveFrameValue'] = Param(
             'never', valType='str', inputType="choice", categ='Data',
             allowedVals=['first', 'last', 'all', 'never'],
             updates='constant', direct=False,
             hint=hnt,
-            label=_localized['saveFrameValue'])
+            label=_translate("Save frame value"))
 
     def writeInitCode(self, buff):
         """Write variable initialisation code."""
@@ -108,9 +108,9 @@ class VariableComponent(BaseComponent):
         buff.writeIndented(code % self.params)
     #
     def writeRoutineStartCode(self, buff):
-        """Write the code that will be called at the start of the routine."""
+        """Write the code that will be called at the start of the Routine."""
         if not self.params['startRoutineValue'] == '':
-            code = ("%(name)s = %(startRoutineValue)s  # Set routine start values for %(name)s\n")
+            code = ("%(name)s = %(startRoutineValue)s  # Set Routine start values for %(name)s\n")
             if self.params['saveStartRoutine'] == True:
                 code += ("thisExp.addData('%(name)s.routineStartVal', %(name)s)  # Save exp start value\n")
             buff.writeIndentedLines(code % self.params)
@@ -168,12 +168,12 @@ class VariableComponent(BaseComponent):
             buff.writeIndentedLines(code)
 
     def writeRoutineEndCode(self, buff):
-        """Write the code that will be called at the end of the routine."""
+        """Write the code that will be called at the end of the Routine."""
         code = ''
         if self.params['saveStartExp'] == True and not self.params['startExpValue'] == '':
             code = ("thisExp.addData('%(name)s.expStartVal', %(startExpValue)s)  # Save exp start value\n")
         if self.params['saveEndRoutine'] == True and not self.params['startRoutineValue'] == '':
-            code += ("thisExp.addData('%(name)s.routineEndVal', %(name)s)  # Save end routine value\n")
+            code += ("thisExp.addData('%(name)s.routineEndVal', %(name)s)  # Save end Routine value\n")
         if not self.params['startFrameValue'] == '':
             if self.params['saveFrameValue'] == 'last':
                 code += ("thisExp.addData('%(name)s.frameEndVal', %(name)sContainer[-1])  # Save end frame value\n")
