@@ -13,6 +13,7 @@ from xml.etree.ElementTree import Element
 from psychopy.experiment import getAllStandaloneRoutines
 from psychopy.experiment.routines._base import Routine, BaseStandaloneRoutine
 from psychopy.experiment.loops import LoopTerminator, LoopInitiator
+from psychopy.tools import filetools as ft
 
 
 class Flow(list):
@@ -215,17 +216,87 @@ class Flow(list):
     def writeBody(self, script):
         """Write the rest of the code
         """
+        # Open function def
+        code = (
+            '\n'
+            'def run(expInfo, thisExp, win, inputs, globalClock=None, thisSession=None):\n'
+            '    """\n'
+            '    Run the experiment flow.\n'
+            '    \n'
+            '    Parameters\n'
+            '    ==========\n'
+            '    expInfo : dict\n'
+            '        Information about this experiment, created by the `setupExpInfo` function.\n'
+            '    thisExp : psychopy.data.ExperimentHandler\n'
+            '        Handler object for this experiment, contains the data to save and information about \n'
+            '        where to save it to.\n'
+            '    psychopy.visual.Window\n'
+            '        Window in which to run this experiment.\n'
+            '    inputs : dict\n'
+            '        Dictionary of input devices by name.\n'
+            '    globalClock : psychopy.core.clock.Clock or None\n'
+            '        Clock to get global time from - supply None to make a new one.\n'
+            '    thisSession : psychopy.session.Session or None\n'
+            '        Handle of the Session object this experiment is being run from, if any.\n'
+            '    """\n'
+        )
+        script.writeIndentedLines(code)
+        script.setIndentLevel(+1, relative=True)
+
+        # unpack inputs
+        code = (
+            "# mark experiment as started\n"
+            "thisExp.status = STARTED\n"
+            "# make sure variables created by exec are available globally\n"
+            "exec = environmenttools.setExecEnvironment(globals())\n"
+            "# get device handles from dict of input devices\n"
+            "ioServer = inputs['ioServer']\n"
+            "defaultKeyboard = inputs['defaultKeyboard']\n"
+            "eyetracker = inputs['eyetracker']\n"
+            "# make sure we're running in the directory for this experiment\n"
+            "os.chdir(_thisDir)\n"
+        )
+        script.writeIndentedLines(code)
+        # unpack filename
+        code = (
+            "# get filename from ExperimentHandler for convenience\n"
+            "filename = thisExp.dataFileName\n"
+            "frameTolerance = 0.001  # how close to onset before 'same' frame\n"
+            "endExpNow = False  # flag for 'escape' or other condition => quit the exp\n"
+        )
+        script.writeIndentedLines(code)
+        # get frame dur from frame rate
+        code = (
+            "# get frame duration from frame rate in expInfo\n"
+            "if 'frameRate' in expInfo and expInfo['frameRate'] is not None:\n"
+            "    frameDur = 1.0 / round(expInfo['frameRate'])\n"
+            "else:\n"
+            "    frameDur = 1.0 / 60.0  # could not measure, so guess\n"
+        )
+        script.writeIndentedLines(code)
+
+        # writes any components with a writeStartCode()
+        self.writeStartCode(script)
         # writeStartCode and writeInitCode:
         for entry in self:
             # NB each entry is a routine or LoopInitiator/Terminator
             self._currentRoutine = entry
+            if hasattr(entry, 'writeRunOnceInitCode'):
+                entry.writeRunOnceInitCode(script)
             entry.writeInitCode(script)
         # create clocks (after initialising stimuli)
-        code = ("\n# Create some handy timers\n"
-                "globalClock = core.Clock()  # to track the "
-                "time since experiment started\n"
-                "routineTimer = core.CountdownTimer()  # to "
-                "track time remaining of each (non-slip) routine \n")
+        code = ("\n"
+                "# create some handy timers\n"
+                "if globalClock is None:\n"
+                "    globalClock = core.Clock()  # to track the time since experiment started\n"
+                "if ioServer is not None:\n"
+                "    ioServer.syncClock(globalClock)\n"
+                "logging.setDefaultClock(globalClock)\n"
+                "routineTimer = core.Clock()  # to track time remaining of each (possibly non-slip) routine\n"
+                "win.flip()  # flip window to reset last flip timer\n"
+                "# store the exact time the global clock started\n"
+                "expInfo['expStart'] = data.getDateStr(format='%Y-%m-%d %Hh%M.%S.%f %z', fractionalSecondDigits=6)\n"
+        )
         script.writeIndentedLines(code)
         # run-time code
         for entry in self:
@@ -238,6 +309,17 @@ class Flow(list):
             self._currentRoutine = entry
             entry.writeExperimentEndCode(script)
 
+        # Mark as finished
+        code = (
+            "\n"
+            "# mark experiment as finished\n"
+            "endExperiment(thisExp, win=win, inputs=inputs)\n"
+        )
+        script.writeIndentedLines(code)
+
+        # Exit function def
+        script.setIndentLevel(-1, relative=True)
+        script.writeIndentedLines("\n")
 
     def writeFlowSchedulerJS(self, script):
         """Initialise each component and then write the per-frame code too
@@ -284,7 +366,7 @@ class Flow(list):
                             "flowScheduler.add({name}LoopEnd);\n"
                             .format(name=thisEntry.loop.params['name'].val))
                     loopStack.append(thisEntry.loop)
-                elif thisEntry.getType() == "Routine":
+                elif isinstance(thisEntry, (Routine, BaseStandaloneRoutine)):
                     code = ("flowScheduler.add({params[name]}RoutineBegin());\n"
                             "flowScheduler.add({params[name]}RoutineEachFrame());\n"
                             "flowScheduler.add({params[name]}RoutineEnd());\n"
@@ -297,18 +379,45 @@ class Flow(list):
                     loopStack.remove(thisEntry.loop)
             script.writeIndentedLines(code)
         # quit when all routines are finished
-        script.writeIndented("flowScheduler.add(quitPsychoJS, '', true);\n")
+        script.writeIndented("flowScheduler.add(quitPsychoJS, %(End Message)s, true);\n" % self.exp.settings.params)
         # handled all the flow entries
         code = ("\n// quit if user presses Cancel in dialog box:\n"
                 "dialogCancelScheduler.add(quitPsychoJS, '', false);\n\n")
         script.writeIndentedLines(code)
 
         # Write resource list
-        resourceFiles = set([resource['rel'].replace("\\", "/") for resource in self.exp.getResourceFiles()])
+        resourceFiles = []
+        for resource in self.exp.getResourceFiles():
+            if isinstance(resource, dict):
+                # Get name
+                if "https://" in resource:
+                    name = resource.split('/')[-1]
+                elif 'surveyId' in resource:
+                    name = 'surveyId'
+                elif 'name' in resource and resource['name'] in list(ft.defaultStim):
+                    name = resource['name']
+                elif 'rel' in resource:
+                    name = resource['rel']
+                else:
+                    name = ""
+
+                # Get resource
+                resourceFile = None
+                if 'rel' in resource:
+                    # If resource is a file path, add its relative path
+                    resourceFile = resource['rel'].replace("\\", "/")
+                elif 'surveyId' in resource:
+                    # If resource is a survey ID, add it and mark as a survey id
+                    resourceFile = "sid:" + resource['surveyId']
+
+                # If we have a resource, add it
+                if resourceFile is not None:
+                    resourceFiles.append((name, resourceFile))
         if self.exp.htmlFolder:
             resourceFolderStr = "resources/"
         else:
             resourceFolderStr = ""
+        # start PsychoJS
         script.writeIndented("psychoJS.start({\n")
         script.setIndentLevel(1, relative=True)
         script.writeIndentedLines("expName: expName,\n"
@@ -318,12 +427,32 @@ class Flow(list):
         if not self.exp.htmlFolder:
             script.writeIndentedLines("resources: [\n")
             script.setIndentLevel(1, relative=True)
-            code = ""
-            for idx, resource in enumerate(resourceFiles):
-                temp = "{{'name': '{0}', 'path': '{1}{0}'}}".format(resource, resourceFolderStr)
-                code += temp
-                if idx != (len(resourceFiles)-1):
-                    code += ",\n"  # Trailing comma
+            # do we need to load surveys?
+            needsSurveys = False
+            for rt in self:
+                if hasattr(rt, "type") and rt.type == "PavloviaSurvey":
+                    needsSurveys = True
+            if needsSurveys:
+                script.writeIndentedLines(
+                    "// libraries:\n"
+                    "{'surveyLibrary': true},\n"
+                )
+            code = "// resources:\n"
+            for name, resource in resourceFiles:
+                if "sid:" in resource:
+                    # Strip sid prefix from survey id
+                    resource = resource.replace("sid:", "")
+                    # Add this line
+                    code += f"{{'surveyId': '{resource}'}},\n"
+                else:
+                    if "https://" in resource:
+                        # URL paths are already fine
+                        pass
+                    else:
+                        # Anything else make it relative to resources folder
+                        resource = resourceFolderStr + resource
+                    # Add this line
+                    code += f"{{'name': '{name}', 'path': '{resource}'}},\n"
             script.writeIndentedLines(code)
             script.setIndentLevel(-1, relative=True)
             script.writeIndented("]\n")
