@@ -15,6 +15,7 @@ The code that writes out a *_lastrun.py experiment file is (in order):
         which will call the .writeBody() methods from each component
     settings.SettingsComponent.writeEndCode()
 """
+import functools
 from xml.etree.ElementTree import Element
 
 import re
@@ -212,8 +213,8 @@ class Param():
                     # If param is a path or pathlike use Path to make sure it's valid (with / not \)
                     isPathLike = bool(re.findall(r"[\\/](?!\W)", val))
                     if self.valType in ['file', 'table'] or (isPathLike and self.canBePath):
-                        val = Path(val).as_posix()
-                        val = val.replace("\\", "/").replace("//", "/")
+                        val = val.replace("\\\\", "/")
+                        val = val.replace("\\", "/")
                     # Hide escape char on escaped $ (other escaped chars are handled by wx but $ is unique to us)
                     val = re.sub(r"\\\$", "$", val)
                     # Replace line breaks with escaped line break character
@@ -274,6 +275,8 @@ class Param():
                 return self.val.strip('$')
             else:
                 return f"\"{self.val}\""
+        elif self.valType == "dict":
+            return str(self.val)
         else:
             raise TypeError("Can't represent a Param of type %s" %
                             self.valType)
@@ -345,8 +348,8 @@ class Param():
                 if len(re.findall(r"\$", val)) == len(re.findall(r"\$", inComment)):
                     # Return if all $ are commented out
                     return True, val
-                if len(re.findall(r"\$", val)) - len(re.findall(r"\$", inComment)) == len(re.findall(r"\\\$", inQuotes)):
-                    # Return if all non-commended $ are in strings and escaped
+                if len(re.findall(r"\$", val)) - len(re.findall(r"\$", inComment)) == len(re.findall(r"\$", inQuotes)):
+                    # Return if all non-commended $ are in strings
                     return True, val
             else:
                 # If value does not begin with an unescaped $, treat it as a string
@@ -362,6 +365,28 @@ class Param():
     __nonzero__ = __bool__  # for python2 compatibility
 
 
+class Partial(functools.partial):
+    """
+    Value to supply to `allowedVals` or `allowedLabels` which contains a reference
+    to a method and arguments to use when populating the control.
+
+    Parameters
+    ----------
+    method : method
+        Method to call, should return the values to be used in the relevant control.
+    args : tuple, list
+        Array of positional arguments. To use the value of another parameter, supply
+        a handle to its Param object.
+    kwargs : dict
+        Dict of keyword arguments. To use the value of another parameter, supply
+        a handle to its Param object.
+    """
+    def __init__(self, method, args=(), kwargs=dict()):
+        self.method = method
+        self.args = args
+        self.kwargs = kwargs
+
+
 def getCodeFromParamStr(val, target=None):
     """Convert a Param.val string to its intended python code
     (as triggered by special char $)
