@@ -17,11 +17,32 @@ except ImportError:
 import astunparse
 
 
+namesJS = {
+    'sin': 'Math.sin',
+    'cos': 'Math.cos',
+    'tan': 'Math.tan',
+    'pi': 'Math.PI',
+    'rand': 'Math.random',
+    'random': 'Math.random',
+    'sqrt': 'Math.sqrt',
+    'abs': 'Math.abs',
+    'floor': 'Math.floor',
+    'ceil': 'Math.ceil',
+    'randint': 'util.randint',
+    'range': 'util.range',
+    'randchoice': 'util.randchoice',
+    'round': 'util.round',  # better than Math.round, supports n DPs arg
+    'sum': 'util.sum',
+}
+
+
 class psychoJSTransformer(ast.NodeTransformer):
     """PsychoJS-specific AST transformer
     """
 
     def visit_Name(self, node):
+        if node.id in namesJS:
+            node.id = namesJS[node.id]
         # status = STOPPED --> status = PsychoJS.Status.STOPPED
         if node.id in ['STARTED', 'FINISHED', 'STOPPED'] and isinstance(node.ctx, ast.Load):
             return ast.copy_location(
@@ -97,7 +118,7 @@ class pythonTransformer(ast.NodeTransformer):
 
     # operation from the math python module or builtin operations that are available
     # in util/Util.js:
-    utilOperations = ['sum', 'average', 'randint', 'range', 'sort', 'shuffle', 'randchoice']
+    utilOperations = ['sum', 'average', 'randint', 'range', 'sort', 'shuffle', 'randchoice', 'pad', 'Clock']
 
     def visit_BinOp(self, node):
 
@@ -179,7 +200,7 @@ class pythonTransformer(ast.NodeTransformer):
             precisionCall = ast.Call(
                 func=ast.Attribute(
                     value=conversionFunc,
-                    attr='toPrecision',
+                    attr='toFixed',
                     ctx=ast.Load()
                 ),
                 args=[ast.Constant(value=precision, kind=None)],
@@ -194,7 +215,7 @@ class pythonTransformer(ast.NodeTransformer):
             )
 
             # return the node:
-            node.value = widthCall
+            node.value = self.visit_Call(widthCall)
             node.conversion = -1
             node.format_spec = None
 
@@ -213,7 +234,7 @@ class pythonTransformer(ast.NodeTransformer):
 
         # substitutable transformation, e.g. Vector.append(5) --> Vector.push(5):
         if isinstance(node.func, ast.Attribute):  # and isinstance(node.func.value, ast.Name):
-            substitutedNode = self.substitutionTransform(node.func, node.args)
+            substitutedNode = self.substitutionTransform(node.func, node.args, node.keywords)
             if substitutedNode:
                 return substitutedNode
 
@@ -226,6 +247,10 @@ class pythonTransformer(ast.NodeTransformer):
                 mathNode = self.mathTransform(attribute, node.args)
                 if mathNode:
                     return mathNode
+            elif prefix == 'core':
+                utilNode = self.utilTransform(attribute, node.args)
+                if utilNode:
+                    return utilNode
 
         # operations without prefix:
         if isinstance(node.func, ast.Name):
@@ -249,70 +274,52 @@ class pythonTransformer(ast.NodeTransformer):
         # return the node by default:
         return node
 
+    def substitutionTransform(self, func, args, keywords):
+        # Substitutions where only the function name changes (see below)
+        functionSubsJS = {
+            'lower': 'toLowerCase',
+            'append': 'push',
+            'upper': 'toUpperCase',
+            'extend': 'concat',
+        }
+        # Substitions that become util functions
+        utilSubsJS = [
+            'index',
+            'count'
+        ]
 
-    def substitutionTransform(self, func, args):
-
-        # a = 'HELLO'
-        # a.lower() --> a.toLowerCase()
-        if func.attr == 'lower':
-            func.attr = 'toLowerCase'
-            return ast.Call(
-                func=func,
-                args=args,
-                keywords=[]
-            )
-
-        # a = [1,2,3]
-        # a.append(4) --> a.push(4)
-        # value=Call(func=Attribute(value=Name(id='a', ctx=Load()), attr='append', ctx=Load()), args=[Num(n=4)], keywords=[])
-        if func.attr == 'append':
-            func.attr = 'push'
-            return ast.Call(
-                func=func,
-                args=args,
-                keywords=[]
-            )
-
-        # a = 'hello
-        # a.upper() --> a.toUpperCase()
-        # value=Call(func=Attribute(value=Name(id='a', ctx=Load()), attr='append', ctx=Load()), args=[Num(n=4)], keywords=[])
-        if func.attr == 'upper':
-            func.attr = 'toUpperCase'
-            return ast.Call(
-                func=func,
-                args=args,
-                keywords=[]
-            )
-
+        # Substitutions where only the function name changes
+        # Examples:
+        #   a = 'HELLO'
+        #   a.lower() --> a.toLowerCase()
+        #
+        #   a = [1,2,3]
+        #   a.append(4) --> a.push(4)
+        #
+        #   a = 'hello
+        #   a.upper() --> a.toUpperCase()
+        #
         # a = [1,2,3]
         # a.extend([4, 5, 6]) --> a.concat([4, 5, 6])
-        if func.attr == 'extend':
-            func.attr = 'concat'
+        if func.attr in functionSubsJS:
+            func.attr = functionSubsJS[func.attr]
             return ast.Call(
                 func=func,
                 args=args,
                 keywords=[]
             )
 
+        # Substitutions where the function is changed to a util.function and the original value becomes an argument
         # a = [1,2,3]
         # a.index(2) --> util.index(a,2)
         # value=Call(func=Attribute(value=Name(id='a', ctx=Load()), attr='index', ctx=Load()), args=[Num(n=2)], keywords=[])
         # value=Call(func=Attribute(value=Name(id='util', ctx=Load()), attr='index', ctx=Load()), args=[Name(id='a', ctx=Load()), Num(n=2)], keywords=[])
-        elif func.attr == 'index':
-            value = func.value
-            func.value = ast.Name(id='util', ctx=ast.Load())
-            args = [value, args]
-            return ast.Call(
-                func=func,
-                args=args,
-                keywords=[]
-            )
-
+        #
         # a = [1,2,3]
         # a.count(2) --> util.count(a,2)
         # value=Call(func=Attribute(value=Name(id='a', ctx=Load()), attr='count', ctx=Load()), args=[Num(n=2)], keywords=[])
         # value=Call(func=Attribute(value=Name(id='util', ctx=Load()), attr='count', ctx=Load()), args=[Name(id='a', ctx=Load()), Num(n=2)], keywords=[])
-        elif func.attr == 'count':
+        elif func.attr in utilSubsJS:
             value = func.value
             func.value = ast.Name(id='util', ctx=ast.Load())
             args = [value, args]
@@ -322,6 +329,72 @@ class pythonTransformer(ast.NodeTransformer):
                 keywords=[]
             )
 
+        # Substitutions where more than one of the function, value, and arguments change
+        # a = [1,2,3]
+        # a.pop(2) -> a.splice(2, 1);
+        # a.pop() -> a.splice(-1, 1);
+        # The second argument of splice is the number of elements to delete; pass 1 for functionality equivalent to pop.
+        # The default first argument for pop is -1 (remove the last item).
+        elif func.attr == 'pop':
+            func.attr = 'splice'
+            args = args if args else [ast.Constant(value=-1, kind=None)]
+            args = [args, [ast.Constant(value=1, kind=None)]]
+
+            return ast.Call(
+                func=func,
+                args=args,
+                keywords=[]
+            )
+
+        # a = [1, 2, 3, 4]
+        # a.insert(0, 5) -> a.splice(0, 0, 5);
+        # Note that .insert only inserts a single element, so there should always be exactly two input args).
+        elif func.attr == 'insert':
+            func.attr = 'splice'
+            args = [args[0], [ast.Constant(value=0, kind=None)], args[1]]
+
+            return ast.Call(
+                func=func,
+                args=args,
+                keywords=[]
+            )
+
+        # a = ['This', 'is', 'a', 'test']
+        # ' '.join(a) -> a.join(" ");
+        # In this case func.value and args need to be switched.
+        elif func.attr == 'join':
+            new_args = [ast.Constant(value=func.value.value, kind=None)]
+            func.value = args[0]
+            
+            return ast.Call(
+                func=func,
+                args=new_args,
+                keywords=[]
+            )
+
+        # a = "This is a test"
+        # a.split() -> a.split(" ")
+        # Note that this function translates correctly if there's an input arg; only the default requires modification.
+        elif func.attr == 'split' and not args:
+            args = [ast.Constant(value=" ", kind=None)]
+            return ast.Call(
+                func=func,
+                args=args,
+                keywords=[]
+            )
+
+        # a = [3, 7, 9, 0, 1, 5]
+        # a.sort(reverse=True) -> a.reverse()
+        # This one only needs adjustment if the reverse=True keyword argument is included.
+        elif func.attr == 'sort' and keywords and keywords[0].arg == 'reverse' and keywords[0].value.value:
+            func.attr = 'reverse'
+            return ast.Call(
+                func=func,
+                args=[],
+                keywords=[]
+            )
+
+    # Substitutions where the value on which the function is performed (not the function itself) changes
         elif isinstance(func.value, ast.Name):
             # webbrowser.open('https://pavlovia.org') --> window.open('https://pavlovia.org')
             if func.value.id == 'webbrowser':
@@ -372,7 +445,7 @@ class pythonTransformer(ast.NodeTransformer):
 
 class pythonAddonVisitor(ast.NodeVisitor):
     # operations that require an addon:
-    addonOperations = ['list', 'pad']
+    addonOperations = ['list']
 
     def __init__(self):
         self.addons = []
@@ -405,11 +478,12 @@ def transformNode(astNode):
     return pythonBuiltinTransformedNode, visitor.addons
 
 
-def transformPsychoJsCode(psychoJsCode, addons):
+def transformPsychoJsCode(psychoJsCode, addons, namespace=[]):
     """Transform the input PsychoJS code.
 
     Args:
         psychoJsCode (str): the input PsychoJS JavaScript code
+        namespace (list): list of varnames which are already defined
 
     Returns:
         (str) the transformed code
@@ -432,20 +506,6 @@ def transformPsychoJsCode(psychoJsCode, addons):
 
         """
 
-    if 'pad' in addons:
-        transformedPsychoJSCode += r"""
-        // add-on: pad(n: number, width: number): string
-        function pad(n, width) {
-            width = width || 2;
-            integerPart = Number.parseInt(n);
-            decimalPart = (n+'').match(/\.[0-9]*/);
-            if (!decimalPart)
-                decimalPart = '';
-            return (integerPart+'').padStart(width,'0') + decimalPart;
-        }
-
-        """
-
     lines = psychoJsCode.splitlines()
 
     # remove the initial variable declarations, unless it is for _pj:
@@ -455,21 +515,38 @@ def transformPsychoJsCode(psychoJsCode, addons):
     else:
         startIndex = 0
 
-    if lines[startIndex].find('var ') == 0:
-        startIndex += 1
-
     for index in range(startIndex, len(lines)):
-        transformedPsychoJSCode += lines[index]
-        transformedPsychoJSCode += '\n'
+        include = True
+
+        # Remove var defs if variable is defined earlier in experiment
+        if lines[index].startswith("var "):
+            # Get var names
+            varNames = lines[index][4:-1].split(", ")
+            validVarNames = []
+            for varName in varNames:
+                if namespace is not None and varName not in namespace:
+                    # If var name not is already in namespace, keep it in
+                    validVarNames.append(varName)
+            # If there are no var names left, remove statement altogether
+            if not len(validVarNames):
+                include = False
+            # Recombine line
+            lines[index] = f"var {', '.join(validVarNames)};"
+
+        # Append line
+        if include:
+            transformedPsychoJSCode += lines[index]
+            transformedPsychoJSCode += '\n'
 
     return transformedPsychoJSCode
 
 
-def translatePythonToJavaScript(psychoPyCode):
+def translatePythonToJavaScript(psychoPyCode, namespace=[]):
     """Translate PsychoPy python code into PsychoJS JavaScript code.
 
     Args:
         psychoPyCode (str): the input PsychoPy python code
+        namespace (list, None): list of varnames which are already defined
 
     Returns:
         str: the PsychoJS JavaScript code
@@ -511,7 +588,7 @@ def translatePythonToJavaScript(psychoPyCode):
 
     # transform the JavaScript code:
     try:
-        transformedPsychoJsCode = transformPsychoJsCode(psychoJsCode, addons)
+        transformedPsychoJsCode = transformPsychoJsCode(psychoJsCode, addons, namespace=namespace)
     except Exception as error:
         raise Exception('unable to transform the PsychoJS JavaScript code: ' + str(error))
 
