@@ -13,6 +13,8 @@ from psychopy.constants import FOREVER
 from xml.etree.ElementTree import Element
 from pathlib import Path
 
+from psychopy.experiment.components.static import StaticComponent
+from psychopy.experiment.components.routineSettings import RoutineSettingsComponent
 from psychopy.localization import _translate
 from psychopy.experiment import Param
 
@@ -23,6 +25,8 @@ class BaseStandaloneRoutine:
     iconFile = Path(__file__).parent / "unknown" / "unknown.png"
     tooltip = ""
     limit = float('inf')
+    # what version was this Routine added in?
+    version = "0.0.0"
 
     def __init__(self, exp, name='',
                  stopType='duration (s)', stopVal='',
@@ -30,21 +34,22 @@ class BaseStandaloneRoutine:
         self.params = {}
         self.name = name
         self.exp = exp
+        self.url = ""
         self.type = 'StandaloneRoutine'
         self.depends = []  # allows params to turn each other off/on
         self.order = ['stopVal', 'stopType', 'name']
 
         msg = _translate(
-            "Name of this routine (alphanumeric or _, no spaces)")
+            "Name of this Routine (alphanumeric or _, no spaces)")
         self.params['name'] = Param(name,
                                     valType='code', inputType="single", categ='Basic',
                                     hint=msg,
-                                    label=_translate('name'))
+                                    label=_translate('Name'))
 
         self.params['stopVal'] = Param(stopVal,
             valType='num', inputType="single", categ='Basic',
             updates='constant', allowedUpdates=[], allowedTypes=[],
-            hint=_translate("When does the routine end? (blank is endless)"),
+            hint=_translate("When does the Routine end? (blank is endless)"),
             label=_translate('Stop'))
 
         msg = _translate("How do you want to define your end point?")
@@ -52,14 +57,14 @@ class BaseStandaloneRoutine:
             valType='str', inputType="choice", categ='Basic',
             allowedVals=['duration (s)', 'duration (frames)', 'condition'],
             hint=msg, direct=False,
-            label=_translate('Stop Type...'))
+            label=_translate('Stop type...'))
 
         # Testing
-        msg = _translate("Disable this component")
+        msg = _translate("Disable this Routine")
         self.params['disabled'] = Param(disabled,
             valType='bool', inputType="bool", categ="Testing",
             hint=msg, allowedTypes=[], direct=False,
-            label=_translate('Disable component'))
+            label=_translate('Disable Routine'))
 
     def __repr__(self):
         _rep = "psychopy.experiment.routines.%s(name='%s', exp=%s)"
@@ -103,6 +108,14 @@ class BaseStandaloneRoutine:
 
         return element
 
+    def copy(self):
+        # Create a deep copy of self
+        dupe = copy.deepcopy(self)
+        # ...but retain original exp reference
+        dupe.exp = self.exp
+
+        return dupe
+
     def writePreCode(self, buff):
         return
 
@@ -183,7 +196,10 @@ class BaseStandaloneRoutine:
     def name(self):
         if hasattr(self, 'params'):
             if 'name' in self.params:
-                return self.params['name'].val
+                if hasattr(self.params['name'], "val"):
+                    return self.params['name'].val
+                else:
+                    return self.params['name']
         return self.type
 
     @name.setter
@@ -213,15 +229,17 @@ class Routine(list):
     """
 
     targets = ["PsychoPy", "PsychoJS"]
+    version = "0.0.0"
 
-    def __init__(self, name, exp, components=()):
+    def __init__(self, name, exp, components=(), disabled=False):
+        self.settings = RoutineSettingsComponent(exp, name, disabled=disabled)
         super(Routine, self).__init__()
-        self.params = {'name': name}
-        self.name = name
+
         self.exp = exp
         self._clockName = None  # for scripts e.g. "t = trialClock.GetTime()"
         self.type = 'Routine'
         list.__init__(self, list(components))
+        self.addComponent(self.settings)
 
     def __repr__(self):
         _rep = "psychopy.experiment.Routine(name='%s', exp=%s, components=%s)"
@@ -230,8 +248,13 @@ class Routine(list):
     def copy(self):
         # Create a new routine with the same experiment and name as this one
         dupe = type(self)(self.name, self.exp, components=())
+        # Replace duplicate Routine's setting component
+        dupe.settings.params = copy.deepcopy(self.settings.params)
         # Iterate through components
         for comp in self:
+            # Skip settings component
+            if isinstance(comp, RoutineSettingsComponent):
+                continue
             # Create a deep copy of each component...
             newComp = copy.deepcopy(comp)
             # ...but retain original exp reference
@@ -254,11 +277,18 @@ class Routine(list):
 
     @property
     def name(self):
-        return self.params['name']
+        return self.params['name'].val
 
     @name.setter
     def name(self, name):
-        self.params['name'] = name
+        self.params['name'].val = name
+        # Update references in components
+        for comp in self:
+            comp.parentName = name
+
+    @property
+    def params(self):
+        return self.settings.params
 
     def integrityCheck(self):
         """Run tests on self and on all the Components inside"""
@@ -290,6 +320,14 @@ class Routine(list):
         """Remove a component from the end of the routine"""
         name = component.params['name']
         self.remove(component)
+        # if this is a static component, we need to remove references to it
+        if isinstance(component, StaticComponent):
+            for update in component.updatesList:
+                # remove reference in component
+                comp = self.exp.getComponentFromName(update['compName'])
+                if comp:
+                    param = comp.params[update['fieldName']]
+                    param.updates = None
         # check if the component was using any Static Components for updates
         for thisParamName, thisParam in list(component.params.items()):
             if (hasattr(thisParam, 'updates') and
@@ -354,10 +392,11 @@ class Routine(list):
                 thisCompon.writeRunOnceInitCode(buff)
 
     def writeInitCode(self, buff):
-        code = '\n# Initialize components for Routine "%s"\n'
+        code = '\n# --- Initialize components for Routine "%s" ---\n'
         buff.writeIndentedLines(code % self.name)
-        self._clockName = self.name + "Clock"
-        buff.writeIndented('%s = core.Clock()\n' % self._clockName)
+
+        maxTime, useNonSlip = self.getMaxTime()
+        self._clockName = 'routineTimer'
         for thisCompon in self:
             thisCompon.writeInitCode(buff)
 
@@ -374,15 +413,15 @@ class Routine(list):
         """This defines the code for the frames of a single routine
         """
         # create the frame loop for this routine
-        code = ('\n# ------Prepare to start Routine "%s"-------\n')
+        code = ('\n# --- Prepare to start Routine "%s" ---\n')
         buff.writeIndentedLines(code % (self.name))
-        code = 'continueRoutine = True\n'
+        code = (
+            'continueRoutine = True\n'
+        )
         buff.writeIndentedLines(code)
 
         # can we use non-slip timing?
         maxTime, useNonSlip = self.getMaxTime()
-        if useNonSlip:
-            buff.writeIndented('routineTimer.add(%f)\n' % (maxTime))
 
         code = "# update component parameters for each repeat\n"
         buff.writeIndentedLines(code)
@@ -407,13 +446,20 @@ class Routine(list):
                 "# reset timers\n"
                 't = 0\n'
                 '_timeToFirstFrame = win.getFutureFlipTime(clock="now")\n'
-                '{clockName}.reset(-_timeToFirstFrame)  # t0 is time of first possible flip\n'
+                # '{clockName}.reset(-_timeToFirstFrame)  # t0 is time of first possible flip\n'
                 'frameN = -1\n'
-                '\n# -------Run Routine "{name}"-------\n')
+                '\n# --- Run Routine "{name}" ---\n')
         buff.writeIndentedLines(code.format(name=self.name,
                                             clockName=self._clockName))
+        # initial value for forceRoutineEnded (needs to happen now as Code components will have executed
+        # their Begin Routine code)
+        code = (
+            'routineForceEnded = not continueRoutine\n'
+        )
+        buff.writeIndentedLines(code)
+
         if useNonSlip:
-            code = 'while continueRoutine and routineTimer.getTime() > 0:\n'
+            code = f'while continueRoutine and routineTimer.getTime() < {maxTime}:\n'
         else:
             code = 'while continueRoutine:\n'
         buff.writeIndented(code)
@@ -441,16 +487,26 @@ class Routine(list):
 
         # allow subject to quit via Esc key?
         if self.exp.settings.params['Enable Escape'].val:
-            code = ('\n# check for quit (typically the Esc key)\n'
-                    'if endExpNow or defaultKeyboard.getKeys(keyList=["escape"]):\n'
-                    '    core.quit()\n')
+            code = (
+                '\n'
+                '# check for quit (typically the Esc key)\n'
+                'if defaultKeyboard.getKeys(keyList=["escape"]):\n'
+                '    thisExp.status = FINISHED\n'
+            )
             buff.writeIndentedLines(code)
+        code = (
+            "if thisExp.status == FINISHED or endExpNow:\n"
+            "    endExperiment(thisExp, inputs=inputs, win=win)\n"
+            "    return\n"
+        )
+        buff.writeIndentedLines(code)
 
         # are we done yet?
         code = (
             '\n# check if all components have finished\n'
             'if not continueRoutine:  # a component has requested a '
             'forced-end of Routine\n'
+            '    routineForceEnded = True\n'
             '    break\n'
             'continueRoutine = False  # will revert to True if at least '
             'one component still running\n'
@@ -472,7 +528,7 @@ class Routine(list):
         buff.setIndentLevel(-1, True)
 
         # write the code for each component for the end of the routine
-        code = ('\n# -------Ending Routine "%s"-------\n'
+        code = ('\n# --- Ending Routine "%s" ---\n'
                 'for thisComponent in %sComponents:\n'
                 '    if hasattr(thisComponent, "setAutoDraw"):\n'
                 '        thisComponent.setAutoDraw(False)\n')
@@ -480,6 +536,16 @@ class Routine(list):
         for event in self:
             event.writeRoutineEndCode(buff)
 
+        if useNonSlip:
+            code = (
+                "# using non-slip timing so subtract the expected duration of this Routine (unless ended on request)\n"
+                "if routineForceEnded:\n"
+                "    routineTimer.reset()\n"
+                "else:\n"
+                "    routineTimer.addTime(-%f)\n"
+            )
+            buff.writeIndentedLines(code % (maxTime))
+
     def writeRoutineBeginCodeJS(self, buff, modular):
 
         # create the frame loop for this routine
@@ -491,7 +557,7 @@ class Routine(list):
         buff.setIndentLevel(1, relative=True)
 
         code = ("TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date\n\n"
-                "//------Prepare to start Routine '%(name)s'-------\n"
+                "//--- Prepare to start Routine '%(name)s' ---\n"
                 "t = 0;\n"
                 "%(name)sClock.reset(); // clock\n"
                 "frameN = -1;\n"
@@ -550,7 +616,7 @@ class Routine(list):
         buff.writeIndentedLines("return async function () {\n")
         buff.setIndentLevel(1, relative=True)
 
-        code = ("//------Loop for each frame of Routine '%(name)s'-------\n"
+        code = ("//--- Loop for each frame of Routine '%(name)s' ---\n"
                 "// get current time\n"
                 "t = %(name)sClock.getTime();\n"
                 "frameN = frameN + 1;"
@@ -625,25 +691,26 @@ class Routine(list):
                     'routineTimer.reset()\n')
             buff.writeIndentedLines(code % self.name)
 
+
     def writeRoutineEndCodeJS(self, buff, modular):
         # can we use non-slip timing?
         maxTime, useNonSlip = self.getMaxTime()
 
-        code = ("\nfunction %(name)sRoutineEnd() {\n" % self.params)
+        code = ("\nfunction %(name)sRoutineEnd(snapshot) {\n" % self.params)
         buff.writeIndentedLines(code)
         buff.setIndentLevel(1, relative=True)
         buff.writeIndentedLines("return async function () {\n")
         buff.setIndentLevel(1, relative=True)
 
         if modular:
-            code = ("//------Ending Routine '%(name)s'-------\n"
+            code = ("//--- Ending Routine '%(name)s' ---\n"
                     "for (const thisComponent of %(name)sComponents) {\n"
                     "  if (typeof thisComponent.setAutoDraw === 'function') {\n"
                     "    thisComponent.setAutoDraw(false);\n"
                     "  }\n"
                     "}\n")
         else:
-            code = ("//------Ending Routine '%(name)s'-------\n"
+            code = ("//--- Ending Routine '%(name)s' ---\n"
                     "%(name)sComponents.forEach( function(thisComponent) {\n"
                     "  if (typeof thisComponent.setAutoDraw === 'function') {\n"
                     "    thisComponent.setAutoDraw(false);\n"
@@ -662,9 +729,14 @@ class Routine(list):
                     'routineTimer.reset();\n\n')
             buff.writeIndentedLines(code % self.name)
 
+        buff.writeIndentedLines(
+            "// Routines running outside a loop should always advance the datafile row\n"
+            "if (currentLoop === psychoJS.experiment) {\n"
+            "  psychoJS.experiment.nextEntry(snapshot);\n"
+            "}\n")
         buff.writeIndented('return Scheduler.Event.NEXT;\n')
         buff.setIndentLevel(-1, relative=True)
-        buff.writeIndentedLines("};\n")
+        buff.writeIndentedLines("}\n")
         buff.setIndentLevel(-1, relative=True)
         buff.writeIndentedLines("}\n")
 
@@ -724,7 +796,20 @@ class Routine(list):
                 except Exception:
                     thisT = 0
                 maxTime = max(maxTime, thisT)
-        if maxTime == 0:  # if there are no components
+        # if max set by routine, override calculated max
+        rtDur, numericStop = self.settings.getDuration()
+        if numericStop and rtDur != FOREVER:
+            maxTime = rtDur
+        # if there are no components, default to 10s
+        if maxTime == 0:
             maxTime = 10
             nonSlipSafe = False
         return maxTime, nonSlipSafe
+
+    @property
+    def disabled(self):
+        return bool(self.params['disabled'])
+
+    @disabled.setter
+    def disabled(self, value):
+        self.params['disabled'].val = value
