@@ -6,29 +6,32 @@
 #  Part of the PsychoPy library
 # Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
+import importlib
+from psychopy import logging
 
 haveQt = False  # until we confirm otherwise
-importOrder = ['PyQt5', 'PyQt4']
+importOrder = ['PyQt6', 'PyQt5']
 
 for libname in importOrder:
     try:
-        exec("import {}".format(libname))
+        importlib.import_module(f"{libname}.QtCore")
         haveQt = libname
+        logging.debug(f"psychopy.gui is using {haveQt}")
         break
-    except ImportError:
+    except ModuleNotFoundError:
         pass
 
 if not haveQt:
     # do the main import again not in a try...except to recreate error
-    exec("import {}".format(importOrder[0]))
+    import PyQt6
+elif haveQt == 'PyQt6':
+    from PyQt6 import QtWidgets
+    from PyQt6 import QtGui
+    from PyQt6.QtCore import Qt
 elif haveQt == 'PyQt5':
     from PyQt5 import QtWidgets
     from PyQt5 import QtGui
     from PyQt5.QtCore import Qt
-else:
-    from PyQt4 import QtGui  
-    QtWidgets = QtGui  # in qt4 these were all in one package
-    from PyQt4.QtCore import Qt
 
 from psychopy import logging
 import numpy as np
@@ -37,7 +40,6 @@ import sys
 import json
 from psychopy.localization import _translate
 
-OK = QtWidgets.QDialogButtonBox.Ok
 
 qtapp = QtWidgets.QApplication.instance()
 
@@ -48,6 +50,7 @@ def ensureQtApp():
     # dialog
     if qtapp is None:
         qtapp = QtWidgets.QApplication(sys.argv)
+        qtapp.setStyle('Fusion')  # use this to avoid annoying PyQt bug with OK being greyed-out
 
 
 wasMouseVisible = True
@@ -87,7 +90,7 @@ class Dlg(QtWidgets.QDialog):
                  screen=-1):
 
         ensureQtApp()
-        QtWidgets.QDialog.__init__(self, None, Qt.WindowTitleHint)
+        QtWidgets.QDialog.__init__(self, None)
 
         self.inputFields = []
         self.inputFieldTypes = {}
@@ -98,25 +101,20 @@ class Dlg(QtWidgets.QDialog):
         # QtWidgets.QToolTip.setFont(QtGui.QFont('SansSerif', 10))
 
         # add buttons for OK and Cancel
-        self.buttonBox = QtWidgets.QDialogButtonBox(Qt.Horizontal,
-                                                    parent=self)
-        self.okbutton = QtWidgets.QPushButton(labelButtonOK,
-                                              parent=self)
-        self.cancelbutton = QtWidgets.QPushButton(labelButtonCancel,
-                                                  parent=self)
-        self.buttonBox.addButton(self.okbutton,
-                                 QtWidgets.QDialogButtonBox.ActionRole)
-        self.buttonBox.addButton(self.cancelbutton,
-                                 QtWidgets.QDialogButtonBox.ActionRole)
-        self.okbutton.clicked.connect(self.accept)
-        self.cancelbutton.clicked.connect(self.reject)
+        buttons = QtWidgets.QDialogButtonBox.StandardButton.Ok | QtWidgets.QDialogButtonBox.StandardButton.Cancel
+        self.buttonBox = QtWidgets.QDialogButtonBox(buttons, parent=self)
+        self.buttonBox.accepted.connect(self.accept)
+        self.buttonBox.rejected.connect(self.reject)
+        # store references to OK and CANCEL buttons
+        self.okBtn = self.buttonBox.button(QtWidgets.QDialogButtonBox.StandardButton.Ok)
+        self.cancelBtn = self.buttonBox.button(QtWidgets.QDialogButtonBox.StandardButton.Cancel)
 
         if style:
             raise RuntimeWarning("Dlg does not currently support the "
                                  "style kwarg.")
         self.size = size
 
-        if haveQt == 'PyQt5':
+        if haveQt in ['PyQt5', 'PyQt6']:
             nScreens = len(qtapp.screens())
         else:
             nScreens = QtWidgets.QDesktopWidget().screenCount()
@@ -129,18 +127,21 @@ class Dlg(QtWidgets.QDialog):
         self.layout.setSpacing(10)
         self.layout.setColumnMinimumWidth(1, 250)
 
+        # add message about required fields (shown/hidden by validate)
+        msg = _translate("Fields marked with an asterisk (*) are required.")
+        self.requiredMsg = QtWidgets.QLabel(text=msg, parent=self)
+        self.layout.addWidget(self.requiredMsg, 0, 0, 1, -1)
+        self.irow += 1
+
         self.setLayout(self.layout)
 
         self.setWindowTitle(title)
 
-
     def addText(self, text, color='', isFieldLabel=False):
         textLabel = QtWidgets.QLabel(text, parent=self)
 
         if len(color):
-            palette = QtGui.QPalette()
-            palette.setColor(QtGui.QPalette.Foreground, QtGui.QColor(color))
-            textLabel.setPalette(palette)
+            textLabel.setStyleSheet("color: {0};".format(color))
 
         if isFieldLabel is True:
             self.layout.addWidget(textLabel, self.irow, 0, 1, 1)
@@ -151,7 +152,7 @@ class Dlg(QtWidgets.QDialog):
         return textLabel
 
     def addField(self, label='', initial='', color='', choices=None, tip='',
-                 enabled=True):
+                 required=False, enabled=True):
         """Adds a (labelled) input field to the dialogue box,
         optional text color and tooltip.
 
@@ -229,6 +230,8 @@ class Dlg(QtWidgets.QDialog):
                     logging.error(msg.format(label, thisType, self.data[ix],
                                              e))
 
+                self.validate()
+
             inputBox.textEdited.connect(handleLineEditChange)
         else:
             inputBox = QtWidgets.QComboBox(parent=self)
@@ -262,6 +265,9 @@ class Dlg(QtWidgets.QDialog):
 
             inputBox.currentIndexChanged.connect(handleCurrentIndexChanged)
 
+        # set required (attribute is checked later by validate fcn)
+        inputBox.required = required
+
         if len(color):
             inputBox.setPalette(inputLabel.palette())
         if len(tip):
@@ -293,6 +299,35 @@ class Dlg(QtWidgets.QDialog):
         """
         return self.exec_()
 
+    def validate(self):
+        """
+        Make sure that required fields have a value.
+        """
+        # start off assuming valid
+        valid = True
+        # start off assuming no required fields
+        hasRequired = False
+        # iterate through fields
+        for field in self.inputFields:
+            # if field isn't required, skip
+            if not field.required:
+                continue
+            # if we got this far, we have a required field
+            hasRequired = True
+            # validation is only relevant for text fields, others have defaults
+            if not isinstance(field, QtWidgets.QLineEdit):
+                continue
+            # check that we have text
+            if not len(field.text()):
+                valid = False
+        # if not valid, disable OK button
+        self.okBtn.setEnabled(valid)
+        # show required message if we have any required fields
+        if hasRequired:
+            self.requiredMsg.show()
+        else:
+            self.requiredMsg.hide()
+
     def show(self):
         """Presents the dialog and waits for the user to press OK or CANCEL.
 
@@ -322,21 +357,23 @@ class Dlg(QtWidgets.QDialog):
         if self.pos is None:
             # Center Dialog on appropriate screen
             frameGm = self.frameGeometry()
-            desktop = QtWidgets.QApplication.desktop()
-            qtscreen = self.screen
             if self.screen <= 0:
-                qtscreen = desktop.primaryScreen()
-            centerPoint = desktop.screenGeometry(qtscreen).center()
+                qtscreen = QtGui.QGuiApplication.primaryScreen()
+            else:
+                qtscreen = self.screen
+            centerPoint = qtscreen.availableGeometry().center()
             frameGm.moveCenter(centerPoint)
             self.move(frameGm.topLeft())
         else:
-            self.move(self.pos[0],self.pos[1])
+            self.move(self.pos[0], self.pos[1])
         QtWidgets.QDialog.show(self)
         self.raise_()
         self.activateWindow()
+        if self.inputFields:
+            self.inputFields[0].setFocus()
 
         self.OK = False
-        if QtWidgets.QDialog.exec_(self) == QtWidgets.QDialog.Accepted:
+        if QtWidgets.QDialog.exec(self):  # == QtWidgets.QDialog.accepted:
             self.OK = True
             return self.data
 
@@ -468,13 +505,32 @@ class DlgFromDict(Dlg):
             tooltip = ''
             if field in tip:
                 tooltip = tip[field]
+            # is field required?
+            required = str(label).startswith("*") or str(label).endswith("*")
+            # make field
             if field in fixed:
-                self.addFixedField(label, self.dictionary[field], tip=tooltip)
+                self.addFixedField(
+                    label,
+                    self.dictionary[field],
+                    tip=tooltip
+                )
             elif type(self.dictionary[field]) in [list, tuple]:
-                self.addField(label, choices=self.dictionary[field],
-                              tip=tooltip)
+                self.addField(
+                    label,
+                    choices=self.dictionary[field],
+                    tip=tooltip,
+                    required=required
+                )
             else:
-                self.addField(label, self.dictionary[field], tip=tooltip)
+                self.addField(
+                    label,
+                    self.dictionary[field],
+                    tip=tooltip,
+                    required=required
+                )
+
+        # validate so the required message is shown/hidden as appropriate
+        self.validate()
 
         if show:
             self.show()
@@ -686,7 +742,7 @@ if __name__ == '__main__':
 
     # Test Dict Dialog
 
-    info = {'Observer': 'jwp', 'GratingOri': 45,
+    info = {'Observer*': 'jwp', 'GratingOri': 45,
             'ExpVersion': 1.1, 'Group': ['Test', 'Control']}
     dictDlg = DlgFromDict(dictionary=info, title='TestExperiment',
                           labels={'Group': 'Participant Group'},
@@ -725,3 +781,4 @@ if __name__ == '__main__':
     # win.flip()
     # from psychopy import event
     # event.waitKeys()
+
