@@ -2,7 +2,7 @@
 # Part of the PsychoPy library
 # Copyright (C) 2012-2020 iSolver Software Solutions (C) 2021 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
-
+import importlib
 import os
 import sys
 from operator import itemgetter
@@ -31,6 +31,7 @@ from .devices import DeviceEvent, import_device
 from .devices import Computer
 from .devices.deviceConfigValidation import validateDeviceConfiguration
 getTime = Computer.getTime
+syncClock = Computer.syncClock
 
 # pylint: disable=protected-access
 # pylint: disable=broad-except
@@ -506,6 +507,12 @@ class udpServer(DatagramServer):
         the ioHub Server process."""
         return getTime()
 
+    @staticmethod
+    def syncClock(lastResetTime):
+        """See Computer.syncClock documentation, where current process will be
+        the ioHub Server process."""
+        return syncClock(lastResetTime)
+
     @staticmethod
     def setPriority(level='normal', disable_gc=False):
         """See Computer.setPriority documentation, where current process will
@@ -821,6 +828,7 @@ class ioServer():
 
         DeviceClass = None
         cls_name_start = dev_cls_name.rfind('.')
+        # define subdirectory to look in
         iohub_submod = 'psychopy.iohub.'
         iohub_submod_len = len(iohub_submod)
         dev_mod_pth = iohub_submod + 'devices.'
@@ -829,14 +837,15 @@ class ioServer():
             dev_cls_name = dev_cls_name[cls_name_start + 1:]
         else:
             dev_mod_pth += dev_cls_name.lower()
-
-        dev_file_pth = dev_mod_pth[iohub_submod_len:].replace('.', os.path.sep)
-
-        dev_conf_pth = os.path.join(IOHUB_DIRECTORY, dev_file_pth,
+        # convert subdirectory to path
+        dev_mod = importlib.import_module(dev_mod_pth)
+        dev_file_pth = os.path.dirname(dev_mod.__file__)
+        # get config from path
+        dev_conf_pth = os.path.join(dev_file_pth,
                                     'default_%s.yaml' % (dev_cls_name.lower()))
 
         self.log('Loading Device Defaults file: %s' % (dev_cls_name,))
-
+        # load config
         _dconf = yload(open(dev_conf_pth, 'r'), Loader=yLoader)
         _, def_dev_conf = _dconf.popitem()
 
