@@ -15,10 +15,6 @@ their experiment vary by locale. If they do, it is easy enough for them to
 create multiple versions of an experiment.
 """
 
-from __future__ import absolute_import, print_function
-
-from builtins import map
-from builtins import str
 import platform
 macVer = platform.mac_ver()[0]  # e.g., '10.9.5' or '' for non-Mac
 
