@@ -25,6 +25,7 @@ from psychopy import app
 from psychopy.localization import _translate
 import wx
 
+from ..app.errorDlg import exceptionCallback
 from ..tools.apptools import SortTerm
 
 try:
@@ -93,7 +94,8 @@ def login(tokenOrUsername, rememberMe=True):
     """
     currentSession = getCurrentSession()
     if not currentSession:
-        raise requests.exceptions.ConnectionError("Failed to connect to Pavlovia.org. No network?")
+        exceptionCallback(exc_type=requests.exceptions.ConnectionError)
+        return
     # would be nice here to test whether this is a token or username
     logging.debug('pavloviaTokensCurrently: {}'.format(knownUsers))
     if tokenOrUsername in knownUsers:
@@ -141,7 +143,14 @@ class User(dict):
         # Get info from Pavlovia
         if isinstance(id, (float, int, str)):
             # If given a number or string, treat it as a user ID / username
-            self.info = requests.get("https://pavlovia.org/api/v2/designers/" + str(id)).json()['designer']
+            self.info = self.session.session.get(
+                "https://pavlovia.org/api/v2/designers/" + str(id)
+            ).json()['designer']
+            # Make sure self.info has necessary keys
+            assert 'gitlabId' in self.info, _translate(
+                "Could not retrieve user info for user {}, server returned:\n"
+                "{}"
+            ).format(id,self.info)
         elif isinstance(id, dict) and 'gitlabId' in id:
             # If given a dict from Pavlovia rather than an ID, store it rather than requesting again
             self.info = id
@@ -283,24 +292,40 @@ class PavloviaSession:
             gitlabProj = self.gitlab.projects.create(projDict)
         except gitlab.exceptions.GitlabCreateError as e:
             if 'has already been taken' in str(e.error_message):
+                # If name is taken, get instead of create
+                gitlabProj = self.gitlab.projects.get(f"{self.username}/{name}")
+            else:
+                # Otherwise, raise original error
+                raise e
+            # Check for bare remote
+            bareRemote = False
+            try:
+                gitlabProj.repository_tree()
+            except gitlab.exceptions.GitlabGetError:
+                # If we've made it thus far but can't get a tree, the remote is bare
+                bareRemote = True
+            # If remote is populated, handle and return
+            if not bareRemote:
                 dlg = wx.MessageDialog(None, message=_translate(
                     "Project `{namespace}/{name}` already exists, please choose another name."
                 ).format(namespace=namespace, name=name), style=wx.ICON_WARNING)
                 dlg.ShowModal()
                 return
-            else:
-                raise e
+            # Otherwise, continue as normal (bare remote is fine to replace)
 
         # Create pavlovia project object
         pavProject = PavloviaProject(gitlabProj.get_id(), localRoot=localRoot)
         return pavProject
 
-    def getProject(self, id):
+    def getProject(self, id, localRoot=""):
         """Gets a Pavlovia project from an ID number or namespace/name
 
         Parameters
         ----------
-        id a numerical
+        id : float
+            Numeric ID of the project
+        localRoot : str or Path
+            Path of the project root
 
         Returns
         -------
@@ -308,7 +333,7 @@ class PavloviaSession:
 
         """
         if id:
-            return PavloviaProject(id)
+            return PavloviaProject(id, localRoot=localRoot)
         else:
             return None
 
@@ -353,7 +378,7 @@ class PavloviaSession:
         projIDs = []
         for proj in own + group:
             if proj.id not in projIDs and proj.id not in projs:
-                projs.append(PavloviaProject(proj))
+                projs.append(PavloviaProject(proj.id))
                 projIDs.append(proj.id)
         return projs
 
@@ -392,6 +417,7 @@ class PavloviaSession:
                         "Trying to login with token {} which is shorter "
                         "than expected length ({} not 64) for gitlab token"
                             .format(repr(token), len(token)))
+            # Setup gitlab session
             if parse_version(gitlab.__version__) > parse_version("1.4"):
                 self.gitlab = gitlab.Gitlab(rootURL, oauth_token=token, timeout=10, per_page=100)
             else:
@@ -401,18 +427,25 @@ class PavloviaSession:
             self.userID = self.gitlab.user.id  # populate when token property is set
             self.userFullName = self.gitlab.user.name
             self.authenticated = True
+            # Setup http session
+            self.session = requests.Session()
+            self.session.headers = {'OauthToken': token}
         else:
+            # Setup gitlab session
             if parse_version(gitlab.__version__) > parse_version("1.4"):
                 self.gitlab = gitlab.Gitlab(rootURL, timeout=10, per_page=100)
             else:
                 self.gitlab = gitlab.Gitlab(rootURL, timeout=10)
+            # Setup http session
+            self.session = requests.Session()
 
     @property
     def user(self):
         if not hasattr(self, "_user") or self._user is None:
-            if not hasattr(self.gitlab, "user") or self.gitlab.user.username is None:
+            try:
+                self._user = User(self.gitlab.user.username)
+            except AttributeError:
                 return None
-            self._user = User(self.gitlab.user.username)
         return self._user
 
     @user.setter
@@ -447,6 +480,10 @@ class PavloviaSearch(pandas.DataFrame):
         def __str__(self):
             # Start off with blank str
             terms = ""
+            # Synonymise visibility unspecified with visibility all ticked (compensate for back-end being
+            # exploration focussed)
+            if "Visibility" not in self or not self['Visibility']:
+                self['Visibility'] = ["owned", "private", "public"]
             # Iterate through values
             for key, value in self.items():
                 # Ensure value is iterable and mutable
@@ -476,28 +513,24 @@ class PavloviaSearch(pandas.DataFrame):
         # Ensure filter is a FilterTerm
         filterBy = self.FilterTerm(filterBy)
         # Do search
-        try:
-            if mine:
-                # Display experiments by current user
-                session = getCurrentSession()
-                data = requests.get(
-                    f"https://pavlovia.org/api/v2/designers/{session.userID}/experiments?search={term}{filterBy}",
-                    timeout=5
-                ).json()
-            elif term or filterBy:
-                data = requests.get(
-                    f"https://pavlovia.org/api/v2/experiments?search={term}{filterBy}",
-                    timeout=5
-                ).json()
-            else:
-                # Display demos for blank search
-                data = requests.get(
-                    "https://pavlovia.org/api/v2/experiments?search=demos&designer=demos",
-                    timeout=5
-                ).json()
-        except requests.exceptions.ReadTimeout:
-            msg = "Could not connect to Pavlovia server. Please check that you are connected to the internet. If you are connected, then the Pavlovia servers may be down. You can check their status here: https://pavlovia.org/status"
-            raise ConnectionError(msg)
+        session = getCurrentSession()
+        if mine:
+            # Display experiments by current user
+            data = session.session.get(
+                f"https://pavlovia.org/api/v2/designers/{session.userID}/experiments?search={term}{filterBy}",
+                timeout=10
+            ).json()
+        elif term or filterBy:
+            data = session.session.get(
+                f"https://pavlovia.org/api/v2/experiments?search={term}{filterBy}",
+                timeout=10
+            ).json()
+        else:
+            # Display demos for blank search
+            data = session.session.get(
+                "https://pavlovia.org/api/v2/experiments?search=demos&designer=demos",
+                timeout=10
+            ).json()
         # Construct dataframe
         pandas.DataFrame.__init__(self, data=data['experiments'])
         # Do any requested sorting
@@ -540,17 +573,24 @@ class PavloviaProject(dict):
     .localRoot is the path to the local root
     """
 
+    # list of keys which we expect to be datetimes
+    _datetimeKeys = ("created_at", "last_activity_at")
+
     def __init__(self, id, localRoot=None):
-        if not isinstance(id, int):
-            # If given a dict from Pavlovia rather than an ID, store it rather than requesting again
+        # Cache whatever form of ID is given, to avoid uneccesary calls to Pavlovia/GitLab later
+        if isinstance(id, int):
+            # If created using a numeric ID...
+            self.numericId = id
+        elif isinstance(id, str):
+            # If created using a "namespace/name" string...
+            self.stringId = id
+        else:
+            # If created using a dict with info from Pavlovia...
             self._info = dict(id)
             if 'gitlabId' in self._info:
-                self.id = int(self._info['gitlabId'])
-            else:
-                self.id = int(self._info['id'])
-        else:
-            # If given an ID, store this ready to fetch info when needed
-            self.id = id
+                self.numericId = int(self._info['gitlabId'])
+            if 'pathWithNamespace' in self._info:
+                self.stringId = self._info['pathWithNamespace']
         # Set local root
         if localRoot is not None:
             self.localRoot = localRoot
@@ -567,9 +607,8 @@ class PavloviaProject(dict):
             else:
                 value = None
         # Transform datetimes
-        dtRegex = re.compile("\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d(.\d\d\d)?\w?")
-        if dtRegex.match(str(value)):
-            value = pandas.to_datetime(value, format="%Y-%m-%d %H:%M:%S.%f")
+        if key in self._datetimeKeys:
+            value = pandas.to_datetime(value, format=None, errors='coerce')
 
         return value
 
@@ -577,6 +616,17 @@ class PavloviaProject(dict):
         dict.__setitem__(self, key, value)
         self.project.__setattr__(key, value)
 
+    @property
+    def id(self):
+        """
+        ID of this project - will be either a numeric ID or a "namespace/name" string depending
+        on which is available. If both are available, "namespace/name" is prioritised.
+        """
+        if hasattr(self, "stringId"):
+            return self.stringId
+        elif hasattr(self, "numericId"):
+            return self.numericId
+
     @property
     def info(self):
         """
@@ -599,20 +649,26 @@ class PavloviaProject(dict):
         self._info = None
         # for a new project it may take time for Pavlovia to register the new ID so try for a while
         while self._info is None and (time.time() - start) < 30:
-            requestVal = requests.get(f"https://pavlovia.org/api/v2/experiments/{self.project.id}",
-                                      headers={'OauthToken': self.session.getToken()}).json()
+            requestVal = self.session.session.get(
+                f"https://pavlovia.org/api/v2/experiments/{self.id}",
+            ).json()
             self._info = requestVal['experiment']
         if self._info is None:
             raise ValueError(f"Could not find project with id `{self.id}` on Pavlovia: {requestVal}")
-        # Store own id
-        dict.__init__(self, self.project.attributes)
-        # Convert datetime
-        dtRegex = re.compile("\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d(.\d\d\d)?")
-        for key in self._info:
-            if dtRegex.match(str(self.info[key])):
-                self._info[key] = pandas.to_datetime(self._info[key], format="%Y-%m-%d %H:%M:%S.%f")
-        # Update base dict
-        self.update(self.project.attributes)
+        # Store received ID values from Pavlovia
+        if 'gitlabId' in self._info:
+            self.numericId = int(self._info['gitlabId'])
+        if 'pathWithNamespace' in self._info:
+            self.stringId = self._info['pathWithNamespace']
+        if self.project is not None:
+            # Reinitialise dict
+            dict.__init__(self, self.project.attributes)
+            # Convert datetime
+            for key in self._info:
+                if key in self._datetimeKeys:
+                    self._info[key] = pandas.to_datetime(self._info[key], format=None, errors='coerce')
+            # Update base dict
+            self.update(self.project.attributes)
 
     @property
     def session(self):
@@ -633,7 +689,18 @@ class PavloviaProject(dict):
             self._project = self.session.gitlab.projects.get(self.id)
             return self._project
         except gitlab.exceptions.GitlabGetError as e:
-            raise KeyError(f"Could not find GitLab project with id {self.id}.")
+            # dlg = wx.MessageDialog(
+            #     parent=None,
+            #     message=_translate(
+            #         "Could not find GitLab project with id {}.\n"
+            #         "\n"
+            #         "Please check that the project exists on Pavlovia, that you are logged in as the correct user in "
+            #         "the PsychoPy app, and that your account has access to the project."
+            #     ).format(self.id),
+            #     style=wx.ICON_ERROR
+            # )
+            # dlg.ShowModal()
+            return None
 
     @property
     def editable(self):
@@ -713,29 +780,35 @@ class PavloviaProject(dict):
 
     @property
     def localRoot(self):
-        if self.project.path_with_namespace in knownProjects:
+        if hasattr(self, "stringId") and self.stringId in knownProjects:
             # If project has known local store, return its root
-            return knownProjects[self.project.path_with_namespace]['localRoot']
+            return knownProjects[self.stringId]['localRoot']
+        elif hasattr(self, "_localRootStatic"):
+            # If project has local root stored as static value, return it
+            return self._localRootStatic
         else:
             # Otherwise, return blank
             return ""
 
     @localRoot.setter
     def localRoot(self, value):
-        if self.project.path_with_namespace in knownProjects:
+        if hasattr(self, "stringId") and self.stringId in knownProjects:
             # If project has known local store, update its root
-            knownProjects[self.project.path_with_namespace]['localRoot'] = value
+            knownProjects[self.stringId]['localRoot'] = str(value)
             knownProjects.save()
-        else:
-            # If project has no known local store, create one
-            knownProjects[self.project.path_with_namespace] = {
-                'id': self['path_with_namespace'],
-                'idNumber': self.id,
-                'localRoot': value,
-                'remoteHTTPS': f"https://gitlab.pavlovia.org/{self['path_with_namespace']}.git",
-                'remoteSSH': f"git@gitlab.pavlovia.org:{self['path_with_namespace']}.git"
+        elif hasattr(self, "stringId") and hasattr(self, "numericId"):
+            # If project has no known local store and we are able to, create one
+            knownProjects[self.stringId] = {
+                'id': self.stringId,
+                'idNumber': self.numericId,
+                'localRoot': str(value),
+                'remoteHTTPS': f"https://gitlab.pavlovia.org/{self.stringId}.git",
+                'remoteSSH': f"git@gitlab.pavlovia.org:{self.stringId}.git"
             }
             knownProjects.save()
+        else:
+            # If we don't have enough info to save to knownProjects, just store the value given
+            self._localRootStatic = value
 
     def sync(self, infoStream=None):
         """Performs a pull-and-push operation on the remote
@@ -749,27 +822,44 @@ class PavloviaProject(dict):
         """
         # Error catch local root
         if not self.localRoot:
-            raise gitlab.GitlabGetError("Can't sync project without a local root.")
-        # Reset local repo so it checks again (rather than erroring if it's been deleted without an app restart)
-        self._repo = None
-        # Jot down start time
-        t0 = time.time()
-        # If first commit, do initial push
-        if not bool(self.project.attributes['default_branch']):
-            self.firstPush(infoStream=infoStream)
-        # Pull and push
-        self.pull(infoStream)
-        self.push(infoStream)
-        # Write updates
-        t1 = time.time()
-        msg = ("Successful sync at: {}, took {:.3f}s"
-               .format(time.strftime("%H:%M:%S", time.localtime()), t1 - t0))
-        logging.info(msg)
-        if infoStream:
-            infoStream.write("\n" + msg)
-            time.sleep(0.5)
-        # Refresh info
-        self.refresh()
+            dlg = wx.MessageDialog(self, message=_translate(
+                "Can't sync project without a local root. Please specify a local root then try again."
+            ), style=wx.ICON_ERROR | wx.OK)
+            dlg.ShowModal()
+            return
+        # Error catch logged out
+        if not self.session.user:
+            dlg = wx.MessageDialog(self, message=_translate(
+                "You are not logged in to Pavlovia. Please log in to sync project."
+            ), style=wx.ICON_ERROR | wx.OK)
+            dlg.ShowModal()
+            return
+        if self.project is not None:
+            # Reset local repo so it checks again (rather than erroring if it's been deleted without an app restart)
+            self._repo = None
+            # Jot down start time
+            t0 = time.time()
+            # If first commit, do initial push
+            if not bool(self.project.attributes['default_branch']):
+                self.firstPush(infoStream=infoStream)
+            # Pull and push
+            self.pull(infoStream)
+            self.push(infoStream)
+            # Write updates
+            t1 = time.time()
+            msg = ("Successful sync at: {}, took {:.3f}s"
+                   .format(time.strftime("%H:%M:%S", time.localtime()), t1 - t0))
+            logging.info(msg)
+            if infoStream:
+                infoStream.write("\n" + msg)
+                time.sleep(0.5)
+            # Refresh info
+            self.refresh()
+        else:
+            # If project doesn't exist, tell the user
+            infoStream.write(
+                _translate("\n\nSync failed - could not find project with id {}").format(self.id)
+            )
 
         return 1
 
@@ -1086,7 +1176,20 @@ class PavloviaProject(dict):
 
     def save(self):
         """Saves the metadata to gitlab.pavlovia.org"""
-        self.project.save()
+        try:
+            self.project.save()
+            return True
+        except gitlab.GitlabUpdateError as err:
+            msgRoot = "Could not sync project.\n\n"
+            if err.response_code == 400:
+                # Error: Avatar is too big
+                msg = msgRoot + _translate("Avatar is too big, should be at most 200 KB.")
+                dlg = wx.MessageDialog(None, msg, style=wx.ICON_ERROR)
+                dlg.ShowModal()
+                # Reset avatar
+                self['avatar_url'] = ""
+            return False
+
         # note that saving info locally about known projects is done
         # by the knownProjects DictStorage class
 
@@ -1211,7 +1314,6 @@ def getNameWithNamespace(p):
         return None
 
 
-
 def getProject(filename):
     """Will try to find (locally synced) pavlovia Project for the filename
     """
@@ -1225,10 +1327,36 @@ def getProject(filename):
     path = getNameWithNamespace(filename)
     # Get session
     session = getCurrentSession()
+    # Start off with proj as None
+    proj = None
     # If already found, return
     if (knownProjects is not None) and (path in knownProjects) and ('idNumber' in knownProjects[path]):
+        # Make sure we are logged in
+        nameSpace, projectName = path.split("/")
+        # Try to log in if not logged in
+        if not session.user:
+            if nameSpace in knownUsers:
+                # Log in if user is known
+                login(nameSpace, rememberMe=True)
+            else:
+                # Check whether project repo is found in any of the known users accounts
+                for user in knownUsers:
+                    try:
+                        login(user)
+                    except requests.exceptions.ConnectionError:
+                        break
+
+        thisId = knownProjects[path]['id']
+        # Check that project still exists on Pavlovia
+        requestVal = session.session.get(
+            f"https://pavlovia.org/api/v2/experiments/{thisId}",
+        ).json()
+        if requestVal['experiment'] is None:
+            # If project has been deleted, return None
+            return None
+        # If project is still there, get it
         try:
-            return PavloviaProject(knownProjects[path]['idNumber'])
+            return PavloviaProject(thisId, localRoot=gitRoot)
         except LookupError as err:
             # If project not found, print warning and return None
             logging.warn(str(err))
@@ -1240,18 +1368,58 @@ def getProject(filename):
         for remote in localRepo.remotes:
             for url in remote.urls:
                 if "gitlab.pavlovia.org" in url:
-                    # Get Namespace/Name from standard style url
-                    nameSearch = re.search(r"(?<=https:\/\/gitlab\.pavlovia\.org\/).*\/.*(?=\.git)", url)
-                elif "git@gitlab.pavlovia.org:" in url:
-                    # Get Namespace/Name from @ stye url
-                    nameSearch = re.search(r"(?<=git@gitlab\.pavlovia\.org:).*\/.*(?=\.git)", url)
-                else:
-                    # Attempt to get Namespace/Name from unhandled style
-                    nameSearch = re.search(r"[\w\-]*\\[\w\-]*\.git", url)
-                if nameSearch is not None:
-                    name = nameSearch.group(0)
-                    project = session.gitlab.projects.get(name)
-                    return PavloviaProject(project.id)
+                    # Get namespace from url
+                    # could be 'https://gitlab.pavlovia.org/NameSpace/Name.git'
+                    # or may be 'git@gitlab.pavlovia.org:NameSpace/Name.git'
+                    namespaceName = url.split('gitlab.pavlovia.org')[1]
+                    # remove the first char if it's : or /
+                    if namespaceName[0] in ['/', ':']:
+                        namespaceName = namespaceName[1:]
+                    # Remove .git
+                    namespaceName = namespaceName.replace(".git", "")
+                    # Split to get namespace
+                    nameSpace, projectName = namespaceName.split('/')
+                    # Get current session
+                    pavSession = getCurrentSession()
+                    # Try to log in if not logged in
+                    if not pavSession.user:
+                        if nameSpace in knownUsers:
+                            # Log in if user is known
+                            login(nameSpace, rememberMe=True)
+                        else:
+                            # Check whether project repo is found in any of the known users accounts
+                            for user in knownUsers:
+                                try:
+                                    login(user)
+                                except requests.exceptions.ConnectionError:
+                                    break
+                                foundProject = False
+                                for repo in pavSession.findUserProjects():
+                                    if namespaceName in repo:
+                                        foundProject = True
+                                        logging.info("Logging in as {}".format(user))
+                                        break
+                                if not foundProject:
+                                    logging.warning("Could not find {namespace} in your Pavlovia accounts. "
+                                                    "Logging in as {user}.".format(namespace=namespaceName,
+                                                                                   user=user))
+
+                    if pavSession.user:
+                        # Get PavloviaProject via id
+                        proj = pavSession.getProject(namespaceName, localRoot=gitRoot)
+                        proj.repo = localRepo
+                    else:
+                        # If we are still logged out, prompt user
+                        logging.warning(_translate("We found a repository pointing to {} "
+                                                   "but no user is logged in for us to check it".format(url)))
+                    return proj
+
+            if proj is None:
+                # Warn user if still no project
+                logging.warning("We found a repository at {} but it "
+                                "doesn't point to gitlab.pavlovia.org. "
+                                "You could create that as a remote to "
+                                "sync from PsychoPy.".format(gitRoot))
 
 
 global _existingSession
