@@ -9,10 +9,13 @@ import os
 import time
 import re
 import weakref
+from pathlib import Path
 
 from psychopy import prefs, logging, exceptions
 from psychopy.constants import (STARTED, PAUSED, FINISHED, STOPPING,
                                 NOT_STARTED)
+from psychopy.tools import systemtools
+from psychopy.tools import filetools as ft
 from .exceptions import SoundFormatError, DependencyError
 from ._base import _SoundBase, HammingWindow
 
@@ -56,8 +59,6 @@ else:
 defaultInput = None
 defaultOutput = audioDevice
 
-
-travisCI = bool(str(os.environ.get('TRAVIS')).lower() == 'true')
 logging.info("Loaded psychtoolbox audio version {}"
              .format(audio.get_version_info()['version']))
 
@@ -89,7 +90,7 @@ def getDevices(kind=None):
     else:
         deviceTypes = None
     devs = {}
-    if travisCI:  # travis-CI testing does not have a sound device
+    if systemtools.isVM_CI():  # GitHub actions VM does not have a sound device
         return devs
     else:
         allDevs = audio.get_devices(device_type=deviceTypes)
@@ -221,7 +222,7 @@ class _MasterStream(audio.Stream):
         self.takeTimeStamp = False
         self.frameN = 1
         # self.frameTimes = range(5)  # DEBUGGING: store the last 5 callbacks
-        if not travisCI:  # travis-CI testing does not have a sound device
+        if not systemtools.isVM_CI():  # Github Actions VM does not have a sound device
             try:
                 audio.Stream.__init__(self, device_id=deviceID, mode=mode+8,
                                     latency_class=audioLatencyClass,
@@ -330,8 +331,20 @@ class SoundPTB(_SoundBase):
         # setSound (determines sound type)
         self.setSound(value, secs=self.secs, octave=self.octave,
                       hamming=self.hamming)
+        self._isPlaying = False  # set `True` after `play()` is called
+        self._isFinished = False
         self.status = NOT_STARTED
 
+    @property
+    def isPlaying(self):
+        """`True` if the audio playback is ongoing."""
+        return self._isPlaying
+
+    @property
+    def isFinished(self):
+        """`True` if the audio playback has completed."""
+        return self._isFinished
+
     def _getDefaultSampleRate(self):
         """Check what streams are open and use one of these"""
         if len(streams):
@@ -351,6 +364,7 @@ class SoundPTB(_SoundBase):
         NOT_STARTED, STARTED, FINISHED, PAUSED
 
         Psychtoolbox sounds also have a statusDetailed property with further info"""
+
         if self.__dict__['status']==STARTED:
             # check portaudio to see if still playing
             pa_status = self.statusDetailed
@@ -426,6 +440,9 @@ class SoundPTB(_SoundBase):
         _SoundBase.setSound(self, value, secs, octave, hamming, log)
 
     def _setSndFromFile(self, filename):
+        # alias default names (so it always points to default.png)
+        if filename in ft.defaultStim:
+            filename = Path(prefs.paths['resources']) / ft.defaultStim[filename]
         self.sndFile = f = sf.SoundFile(filename)
         self.sourceType = 'file'
         self.sampleRate = f.samplerate
@@ -510,13 +527,27 @@ class SoundPTB(_SoundBase):
                 "experiment settings**".format(self.channels, array.shape[1]))
             logging.error(msg)
             raise ValueError(msg)
+        
+    def _checkPlaybackFinished(self):
+        """Checks whether playback has finished by looking up the status.
+        """
+        pa_status = self.statusDetailed
+        self._isFinished = not pa_status['Active'] and pa_status['State'] == 0
+        return self._isFinished
 
     def play(self, loops=None, when=None, log=True):
-        """Start the sound playing
+        """Start the sound playing.
+
+        Calling this after the sound has finished playing will restart the
+        sound.
+
         """
+        if self._checkPlaybackFinished():
+            self.stop(reset=True)
+        
         if loops is not None and self.loops != loops:
             self.setLoops(loops)
-        self.status = STARTED
+
         self._tSoundRequestPlay = time.time()
 
         if hasattr(when, 'getFutureFlipTime'):
@@ -528,45 +559,54 @@ class SoundPTB(_SoundBase):
         else:
             logTime = None
         self.track.start(repetitions=loops, when=when)
+        self._isPlaying = True
+        self._isFinished = False
         # time.sleep(0.)
         if log and self.autoLog:
             logging.exp(u"Sound %s started" % (self.name), obj=self, t=logTime)
 
-    def pause(self):
-        """Stop the sound but play will continue from here if needed
+    def pause(self, log=True):
+        """Stops the sound without reset, so that play will continue from here if needed
         """
-        self.status = PAUSED
-        self.track.stop()
+        if self.isPlaying:
+            self.stop(reset=False)
+            if log and self.autoLog:
+                logging.exp(u"Sound %s paused" % (self.name), obj=self)
 
     def stop(self, reset=True, log=True):
         """Stop the sound and return to beginning
         """
-        if self.status == FINISHED:
+        # this uses FINISHED for some reason, all others use STOPPED
+        if not self.isPlaying:
             return
+
         self.track.stop()
+        self._isPlaying = False
+
         if reset:
             self.seek(0)
         if log and self.autoLog:
             logging.exp(u"Sound %s stopped" % (self.name), obj=self)
-        self.status = FINISHED
 
     def seek(self, t):
         self.t = t
         self.frameN = int(round(t * self.sampleRate))
         if self.sndFile and not self.sndFile.closed:
             self.sndFile.seek(self.frameN)
+        self._isFinished = t >= self.duration
 
     def _EOS(self, reset=True, log=True):
         """Function called on End Of Stream
         """
-        self.status = STOPPING
         self._loopsFinished += 1
         if self.loops == 0:
             self.stop(reset=reset, log=False)
-        elif self.loops > 0 and self._loopsFinished >= self.loops:
+            self._isFinished = True
+        elif 0 < self.loops <= self._loopsFinished:
             self.stop(reset=reset, log=False)
+
         if log and self.autoLog:
-            logging.exp(u"Sound %s reached end of file" % (self.name), obj=self)
+            logging.exp(u"Sound %s reached end of file" % self.name, obj=self)
 
     @property
     def stream(self):
