@@ -1,11 +1,12 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+from pathlib import Path
 
-from psychopy import logging
+from psychopy import logging, prefs
 from .exceptions import DependencyError
 from psychopy.constants import (STARTED, PLAYING, PAUSED, FINISHED, STOPPED,
                                 NOT_STARTED, FOREVER)
-from psychopy.tools import attributetools
+from psychopy.tools import attributetools, filetools as ft
 from ._base import _SoundBase
 
 try:
@@ -183,6 +184,13 @@ class SoundPySoundCard(_SoundBase):
         self.requestedLoops = self.loops = int(loops)
         self.setSound(value=value, secs=secs, octave=octave)
 
+        self._isPlaying = False
+
+    @property
+    def isPlaying(self):
+        """`True` if the audio playback is ongoing."""
+        return self._isPlaying
+
     def play(self, fromStart=True, log=True, loops=None, when=None):
         """Starts playing the sound on an available channel.
 
@@ -209,18 +217,24 @@ class SoundPySoundCard(_SoundBase):
             will be played over each other.
 
         """
+        if self.isPlaying:
+            return
+
         if loops is not None:
             self.loops = loops
         self._stream.start()
-        self.status = STARTED
+        self._isPlaying = True
         if log and self.autoLog:
             logging.exp("Sound %s started" % (self.name), obj=self)
         return self
 
     def stop(self, log=True):
         """Stops the sound immediately"""
+        if not self.isPlaying:  # already stopped
+            return
+
         self._stream.abort()  # _stream.stop() finishes current buffer
-        self.status = STOPPED
+        self._isPlaying = False
         if log and self.autoLog:
             logging.exp("Sound %s stopped" % (self.name), obj=self)
 
@@ -229,8 +243,8 @@ class SoundPySoundCard(_SoundBase):
         Don't know why you would do this in psychophysics but it's easy
         and fun to include as a possibility :)
         """
-        pass  # todo
-        self.status = STOPPED
+        # todo
+        self._isPlaying = False
 
     def getDuration(self):
         """Gets the duration of the current sound in secs
@@ -250,6 +264,9 @@ class SoundPySoundCard(_SoundBase):
         return value  # this is returned for historical reasons
 
     def _setSndFromFile(self, fileName):
+        # alias default names (so it always points to default.png)
+        if fileName in ft.defaultStim:
+            fileName = Path(prefs.paths['resources']) / ft.defaultStim[fileName]
         # load the file
         if not path.isfile(fileName):
             msg = "Sound file %s could not be found." % fileName
@@ -316,7 +333,7 @@ class SoundPySoundCard(_SoundBase):
     def _onEOS(self, log=True):
         if log and self.autoLog:
             logging.exp("Sound %s finished" % (self.name), obj=self)
-        self.status = FINISHED
+        self._isPlaying = False
 
     def __del__(self):
         self._stream.close()
