@@ -12,7 +12,7 @@ __all__ = ['Microphone']
 
 import sys
 import psychopy.logging as logging
-from psychopy.constants import NOT_STARTED, STARTED
+from psychopy.constants import NOT_STARTED
 from psychopy.preferences import prefs
 from .audioclip import *
 from .audiodevice import *
@@ -45,7 +45,7 @@ class RecordingBuffer:
     ----------
     sampleRateHz : int
         Sampling rate for audio recording in Hertz (Hz). By default, 48kHz
-        (``sampleRateHz=480000``) is used which is adequate for most consumer
+        (``sampleRateHz=48000``) is used which is adequate for most consumer
         grade microphones (headsets and built-in).
     channels : int
         Number of channels to record samples to `1=Mono` and `2=Stereo`.
@@ -313,7 +313,7 @@ class Microphone:
     Creating an instance of this class will open a stream using the specified
     device. Streams should remain open for the duration of your session. When a
     stream is opened, a buffer is allocated to store samples coming off it.
-    Samples from the input stream will written to the buffer once
+    Samples from the input stream will writen to the buffer once
     :meth:`~Microphone.start()` is called.
 
     Parameters
@@ -323,7 +323,7 @@ class Microphone:
         (`int`) or descriptor (`AudioDevice`).
     sampleRateHz : int
         Sampling rate for audio recording in Hertz (Hz). By default, 48kHz
-        (``sampleRateHz=480000``) is used which is adequate for most consumer
+        (``sampleRateHz=48000``) is used which is adequate for most consumer
         grade microphones (headsets and built-in).
     channels : int
         Number of channels to record samples to `1=Mono` and `2=Stereo`.
@@ -346,9 +346,9 @@ class Microphone:
         be used.
     audioRunMode : int
         Run mode for the recording device. Default is standby-mode (`0`) which
-        allows the system to put the device to sleep. However when the device is
-        needed, waking the device results in some latency. Using a run mode of
-        `1` will keep the microphone running (or 'hot') with reduces latency
+        allows the system to put the device to sleep. However, when the device
+        is needed, waking the device results in some latency. Using a run mode
+        of `1` will keep the microphone running (or 'hot') with reduces latency
         when th recording is started. Cannot be set when after initialization at
         this time.
 
@@ -391,7 +391,7 @@ class Microphone:
     def __init__(self,
                  device=None,
                  sampleRateHz=None,
-                 channels=2,
+                 channels=None,
                  streamBufferSecs=2.0,
                  maxRecordingSize=24000,
                  policyWhenFull='warn',
@@ -402,21 +402,47 @@ class Microphone:
             raise ModuleNotFoundError(
                 "Microphone audio capture requires package `psychtoolbox` to "
                 "be installed.")
+        
+        def _getDeviceByIndex(deviceIndex):
+            """Subroutine to get a device by index. Used to handle the case 
+            where the user specifies a device by index.
 
-        # get information about the selected device
-        devices = Microphone.getDevices()
-        if isinstance(device, AudioDeviceInfo):
-            self._device = device
-        elif isinstance(device, (int, float)):
-            devicesByIndex = {d.deviceIndex: d for d in devices}
-            if device in devicesByIndex:
-                self._device = devicesByIndex[device]
+            Parameters
+            ----------
+            deviceIndex : int, float or str
+                Index of the device to get.
+            
+            Returns
+            -------
+            AudioDeviceInfo
+                Audio device information object.
+            
+            """
+            # convert to `int` first, sometimes strings can specify the enum
+            # value from builder
+            deviceIndex = int(deviceIndex)
+            # get all audio devices
+            devices_ = Microphone.getDevices()
+
+            # get information about the selected device
+            devicesByIndex = {d.deviceIndex: d for d in devices_}
+            if deviceIndex in devicesByIndex:
+                useDevice = devicesByIndex[deviceIndex]
             else:
                 raise AudioInvalidCaptureDeviceError(
                     'No suitable audio recording devices found matching index '
-                    '{}.'.format(device))
+                    '{}.'.format(deviceIndex))
+            
+            return useDevice
+
+        # get information about the selected device
+        if isinstance(device, AudioDeviceInfo):
+            self._device = device
+        elif isinstance(device, (int, float, str)):
+            self._device = _getDeviceByIndex(device)
         else:
             # get default device, first enumerated usually
+            devices = Microphone.getDevices()
             if not devices:
                 raise AudioInvalidCaptureDeviceError(
                     'No suitable audio recording devices found on this system. '
@@ -505,7 +531,7 @@ class Microphone:
             'Allocated stream buffer to hold {} seconds of data'.format(
                 self._streamBufferSecs))
 
-        # status flag
+        # status flag for Builder
         self._statusFlag = NOT_STARTED
 
         # setup recording buffer
@@ -521,6 +547,7 @@ class Microphone:
         self.lastClip = None
         self.scripts = {}
         self.lastScript = None
+        self._isStarted = False  # internal state
 
         logging.debug('Audio capture device #{} ready'.format(
             self._device.deviceIndex))
@@ -637,6 +664,10 @@ class Microphone:
         """Status flag for the microphone. Value can be one of
         ``psychopy.constants.STARTED`` or ``psychopy.constants.NOT_STARTED``.
 
+        This is attribute is used by Builder and does not correspond to the
+        actual state of the microphone. Use `streamStatus` and `isStarted`
+        instead.
+
         For detailed stream status information, use the
         :attr:`~psychopy.sound.microphone.Microphone.streamStatus` property.
 
@@ -690,7 +721,13 @@ class Microphone:
     @property
     def isStarted(self):
         """``True`` if stream recording has been started (`bool`)."""
-        return self.status == STARTED
+        return self._isStarted
+
+    @property
+    def isRecording(self):
+        """``True`` if stream recording has been started (`bool`). Alias of
+        `isStarted`."""
+        return self.isStarted
 
     def start(self, when=None, waitForStart=0, stopTime=None):
         """Start an audio recording.
@@ -738,7 +775,7 @@ class Microphone:
             stop_time=stopTime)
 
         # recording has begun or is scheduled to do so
-        self._statusFlag = STARTED
+        self._isStarted = True
 
         logging.debug(
             'Scheduled start of audio capture for device #{} at t={}.'.format(
@@ -793,14 +830,16 @@ class Microphone:
 
         Returns
         -------
-        tuple
+        tuple or None
             Tuple containing `startTime`, `endPositionSecs`, `xruns` and
-            `estStopTime`.
+            `estStopTime`. Returns `None` if `stop` or `pause` was called
+            previously before `start`.
 
         """
+        # This function must be idempotent since it can be invoked at any time
+        # whether a stream is started or not.
         if not self.isStarted:
-            raise AudioStreamError(
-                "Cannot stop a stream that has not been started.")
+            return
 
         # poll remaining samples, if any
         if not self.isRecBufferFull:
@@ -809,7 +848,7 @@ class Microphone:
         startTime, endPositionSecs, xruns, estStopTime = self._stream.stop(
             block_until_stopped=int(blockUntilStopped),
             stopTime=stopTime)
-        self._statusFlag = NOT_STARTED
+        self._isStarted = False
 
         logging.debug(
             ('Device #{} stopped capturing audio samples at estimated time '
@@ -835,9 +874,10 @@ class Microphone:
 
         Returns
         -------
-        tuple
+        tuple or None
             Tuple containing `startTime`, `endPositionSecs`, `xruns` and
-            `estStopTime`.
+            `estStopTime`. Returns `None` if `stop()` or `pause()` was called
+            previously before `start()`.
 
         """
         return self.stop(blockUntilStopped=blockUntilStopped, stopTime=stopTime)
@@ -912,14 +952,20 @@ class Microphone:
         # make sure the tag exists in both clips and transcripts dicts
         if tag not in self.clips:
             self.clips[tag] = []
+
         if tag not in self.scripts:
             self.scripts[tag] = []
+
         # append current recording to clip list according to tag
         self.lastClip = self.getRecording()
         self.clips[tag].append(self.lastClip)
+
         # synonymise null values
-        if transcribe in ('undefined', 'NONE', 'None', 'none', 'False', 'false', 'FALSE'):
+        nullVals = (
+            'undefined', 'NONE', 'None', 'none', 'False', 'false', 'FALSE')
+        if transcribe in nullVals:
             transcribe = False
+
         # append current clip's transcription according to tag
         if transcribe:
             if transcribe in ('Built-in', True, 'BUILT_IN', 'BUILT-IN',
@@ -927,13 +973,21 @@ class Microphone:
                 engine = "sphinx"
             elif type(transcribe) == str:
                 engine = transcribe
+            else:
+                raise ValueError(
+                    "Invalid transcription engine {} specified.".format(
+                        transcribe))
+
             self.lastScript = self.lastClip.transcribe(
                 engine=engine, **kwargs)
         else:
             self.lastScript = "Transcription disabled."
+
         self.scripts[tag].append(self.lastScript)
+
         # clear recording buffer
         self._recording.clear()
+
         # return banked items
         if transcribe:
             return self.lastClip, self.lastScript
@@ -952,7 +1006,6 @@ class Microphone:
         """Get a copy of all banked clips, then clear the clips from storage."""
         # get copy of clips dict
         clips = self.clips.copy()
-        # clear
         self.clear()
 
         return clips
