@@ -12,8 +12,7 @@ from psychopy.hardware import emulator
 from psychopy.hardware.emulator import *
 
 # launchScan sound is not tested, nor included coverage
-
-travis = bool("{}".format(os.environ.get('TRAVIS')).lower() == 'true')
+from psychopy.tests import skip_under_vm
 
 BASE_MR_SETTINGS = {
     'TR': 0.5,    # duration (sec) per volume
@@ -25,7 +24,7 @@ BASE_MR_SETTINGS = {
 @pytest.mark.emulator
 class TestLaunchScan():
     '''A base class to test launchScan with different MR_settings'''
-    def setup(self):
+    def setup_method(self):
         self.win = visual.Window(fullscr=False, autoLog=False)
         self.globalClock = core.Clock()
         self.MR_settings = BASE_MR_SETTINGS.copy()
@@ -81,9 +80,8 @@ class TestLaunchScan():
         r.stop()
         core.wait(.1, 0)
 
+    @skip_under_vm
     def test_misc(self):
-        if travis:
-            pytest.skip()
         MR_settings = BASE_MR_SETTINGS.copy()
         MR_settings.update({'sync': 'equal'})
         vol = launchScan(self.win, MR_settings, globalClock=self.globalClock,
