@@ -7,9 +7,10 @@ from psychopy.iohub import launchHubServer, Computer
 
 getTime = Computer.getTime
 
-from psychopy.tests.utils import skip_under_travis
+from psychopy.tests import skip_under_vm
 
-@skip_under_travis
+
+@skip_under_vm
 def startHubProcess():
     io = launchHubServer()
     assert io != None
@@ -20,7 +21,8 @@ def startHubProcess():
 
     return io
 
-@skip_under_travis
+
+@skip_under_vm
 def stopHubProcess():
     from psychopy.iohub.client import ioHubConnection
 
