@@ -17,7 +17,8 @@ To add a new stimulus test use _base so that it gets tested in all contexts
 
 """
 
-from psychopy.tests import _travisTesting, skip_under_vm, _vmTesting
+from psychopy.tests import skip_under_vm
+from psychopy.tools import systemtools
 
 
 class Test_Window():
@@ -55,6 +56,32 @@ class Test_Window():
         self.win.callOnFlip(assertThisIs2, 2)
         self.win.flip()
 
+    def test_resetViewport(self):
+        # Check if the `Window.resetViewport()` method works correctly. Not
+        # checking if the OpenGL state is correct here, just if the property
+        # setter `Window.viewport` updates accordingly.
+        #
+        # bugfix: https://github.com/psychopy/psychopy/issues/5135
+        #
+        viewportOld = self.win.viewport.copy()  # store old viewport value
+
+        # Create a new viewport, ensure that the test value never equals the
+        # windows size.
+        viewportNew = [0, 0] + [max(int(v / 2.0), 1) for v in viewportOld[2:]]
+        self.win.viewport = viewportNew
+
+        # assert that the change has been made correctly after setting
+        assert numpy.allclose(self.win.viewport, viewportNew), \
+            "Failed to change viewport, expected `{}` got `{}`.".format(
+                viewportNew, list(self.win.viewport))  # show as list
+
+        # reset the viewport and check if the value is reset to original
+        self.win.resetViewport()
+
+        assert numpy.allclose(self.win.viewport, viewportOld), \
+            "Failed to reset viewport, expected `{}` got `{}`.".format(
+                viewportOld, list(self.win.viewport))
+
 
 class _baseVisualTest():
     #this class allows others to be created that inherit all the tests for
@@ -69,7 +96,7 @@ class _baseVisualTest():
     def teardown_class(self):#run once for each test class (window)
         self.win.close()#shutil.rmtree(self.temp_dir)
 
-    def setup(self):#this is run for each test individually
+    def setup_method(self):#this is run for each test individually
         #make sure we start with a clean window
         self.win.flip()
 
@@ -475,7 +502,7 @@ class _baseVisualTest():
         text.draw()
         grat1.draw()
         grat2.draw()
-        if _vmTesting:
+        if systemtools.isVM_CI():
             pytest.skip("Blendmode='add' doesn't work under a virtual machine for some reason")
         if self.win.winType != 'pygame':
             utils.compareScreenshot('blend_add_%s.png' %self.contextName,
@@ -498,10 +525,7 @@ class _baseVisualTest():
         mov = visual.MovieStim3(win, fileName, pos=pos, noAudio=True)
         mov.setFlipVert(True)
         mov.setFlipHoriz(True)
-        if sys.platform == 'darwin':
-            threshold = 30
-        else:
-            threshold = 11
+        threshold = 30
         for frameN in range(10):
             mov.draw()
 
@@ -704,7 +728,7 @@ class _baseVisualTest():
         grating.draw()
         utils.compareScreenshot('aperture1_%s.png' %(self.contextName), win)
         #aperture should automatically disable on exit
-        for shape, nVert, pos in [(None, 120, (0,0)), ('circle', 17, (.2, -.7)),
+        for shape, nVert, pos in [(None, 4, (0,0)), ('circle', 72, (.2, -.7)),
                                   ('square', 4, (-.5,-.5)), ('triangle', 3, (1,1))]:
             aperture = visual.Aperture(win, pos=pos, shape=shape, nVert=nVert)
             assert len(aperture.vertices) == nVert  # true for BaseShapeStim; expect (nVert-2)*3 if tesselated
@@ -772,7 +796,7 @@ class TestPygletHexColor(_baseVisualTest):
         self.contextName='normHexbackground'
         self.scaleFactor=1#applied to size/pos values
 
-if not _travisTesting:
+if not systemtools.isVM_CI():
     class TestPygletBlendAdd(_baseVisualTest):
         @classmethod
         def setup_class(self):
@@ -785,7 +809,7 @@ if not _travisTesting:
 class TestPygletNormFBO(_baseVisualTest):
     @classmethod
     def setup_class(self):
-        self.win = visual.Window([128,128], winType='pyglet', pos=[50,50],
+        self.win = visual.Window([128,128], units="norm", winType='pyglet', pos=[50,50],
                                  allowStencil=True, autoLog=False, useFBO=True)
         self.contextName='norm'
         self.scaleFactor=1#applied to size/pos values
@@ -794,7 +818,7 @@ class TestPygletNormFBO(_baseVisualTest):
 class TestPygletHeight(_baseVisualTest):
     @classmethod
     def setup_class(self):
-        self.win = visual.Window([128,64], winType='pyglet', pos=[50,50],
+        self.win = visual.Window([128,64], units="height", winType='pyglet', pos=[50,50],
                                  allowStencil=False, autoLog=False)
         self.contextName='height'
         self.scaleFactor=1#applied to size/pos values
@@ -803,7 +827,7 @@ class TestPygletHeight(_baseVisualTest):
 class TestPygletNormStencil(_baseVisualTest):
     @classmethod
     def setup_class(self):
-        self.win = visual.Window([128,128], monitor='testMonitor',
+        self.win = visual.Window([128,128], units="norm", monitor='testMonitor',
                                  winType='pyglet', pos=[50,50],
                                  allowStencil=True, autoLog=False)
         self.contextName='stencil'
@@ -817,8 +841,9 @@ class TestPygletPix(_baseVisualTest):
         mon.setDistance(57)
         mon.setWidth(40.0)
         mon.setSizePix([1024,768])
-        self.win = visual.Window([128,128], monitor=mon, winType='pyglet', pos=[50,50], allowStencil=True,
-            units='pix', autoLog=False)
+        self.win = visual.Window([128,128], units="pix", monitor=mon,
+                                 winType='pyglet', pos=[50, 50],
+                                 allowStencil=True, autoLog=False)
         self.contextName='pix'
         self.scaleFactor=60#applied to size/pos values
 
@@ -830,8 +855,9 @@ class TestPygletCm(_baseVisualTest):
         mon.setDistance(57.0)
         mon.setWidth(40.0)
         mon.setSizePix([1024,768])
-        self.win = visual.Window([128,128], monitor=mon, winType='pyglet', pos=[50,50], allowStencil=False,
-            units='cm', autoLog=False)
+        self.win = visual.Window([128,128], units="cm", monitor=mon,
+                                 winType='pyglet', pos=[50,50],
+                                 allowStencil=False, autoLog=False)
         self.contextName='cm'
         self.scaleFactor=2#applied to size/pos values
 
@@ -843,8 +869,9 @@ class TestPygletDeg(_baseVisualTest):
         mon.setDistance(57.0)
         mon.setWidth(40.0)
         mon.setSizePix([1024,768])
-        self.win = visual.Window([128,128], monitor=mon, winType='pyglet', pos=[50,50], allowStencil=True,
-            units='deg', autoLog=False)
+        self.win = visual.Window([128,128], units="deg", monitor=mon,
+                                 winType='pyglet', pos=[50,50], allowStencil=True,
+                                 autoLog=False)
         self.contextName='deg'
         self.scaleFactor=2#applied to size/pos values
 
@@ -856,8 +883,9 @@ class TestPygletDegFlat(_baseVisualTest):
         mon.setDistance(10.0) #exaggerate the effect of flatness by setting the monitor close
         mon.setWidth(40.0)
         mon.setSizePix([1024,768])
-        self.win = visual.Window([128,128], monitor=mon, winType='pyglet', pos=[50,50], allowStencil=True,
-            units='degFlat', autoLog=False)
+        self.win = visual.Window([128,128], units="degFlat", monitor=mon,
+                                 winType='pyglet', pos=[50,50],
+                                 allowStencil=True, autoLog=False)
         self.contextName='degFlat'
         self.scaleFactor=4#applied to size/pos values
 
@@ -869,8 +897,9 @@ class TestPygletDegFlatPos(_baseVisualTest):
         mon.setDistance(10.0) #exaggerate the effect of flatness by setting the monitor close
         mon.setWidth(40.0)
         mon.setSizePix([1024,768])
-        self.win = visual.Window([128,128], monitor=mon, winType='pyglet', pos=[50,50], allowStencil=True,
-            units='degFlatPos', autoLog=False)
+        self.win = visual.Window([128,128], units='degFlatPos', monitor=mon,
+                                 winType='pyglet', pos=[50,50],
+                                 allowStencil=True, autoLog=False)
         self.contextName='degFlatPos'
         self.scaleFactor=4#applied to size/pos values
 
