@@ -1,9 +1,11 @@
 import pytest
 from psychopy import visual
 from .test_basevisual import _TestColorMixin, _TestUnitsMixin
+from psychopy.tests.test_experiment.test_component_compile_python import _TestBoilerplateMixin
 
 
-class TestShape(_TestColorMixin, _TestUnitsMixin):
+
+class TestShape(_TestColorMixin, _TestUnitsMixin, _TestBoilerplateMixin):
 
     @classmethod
     def setup_class(self):
