@@ -1,5 +1,5 @@
 import sys
-from os.path import abspath, basename, dirname, isfile, join as pjoin
+from os.path import abspath, basename, dirname, isfile, isdir, join as pjoin
 import os.path
 from pathlib import Path
 import shutil
@@ -18,14 +18,51 @@ import pytest
 # so tests could be ran from any location
 TESTS_PATH = abspath(dirname(__file__))
 TESTS_DATA_PATH = pjoin(TESTS_PATH, 'data')
+TESTS_FAILS_PATH = pjoin(TESTS_PATH, 'fails', sys.platform)
 TESTS_FONT = pjoin(TESTS_DATA_PATH, 'DejaVuSerif.ttf')
 
+# Make sure all paths exist
+if not isdir(TESTS_FAILS_PATH):
+    os.makedirs(TESTS_FAILS_PATH)
+
 # Some regex shorthand
 _q = r"[\"']"  # quotes
 _lb = r"[\[\(]"  # left bracket
 _rb = r"[\]\)]"  # right bracket
 _d = r"\$"  # dollar (escaped for re)
-_sl = "\\"  # back slash
+_sl = r"\\\\"  # back slash
+
+
+def getFailFilenames(fileName, tag=""):
+    """
+    Create variant of given filename for a failed test
+
+    Parameters
+    ==========
+    fileName : str or Path
+        Path to original file
+    tag : str
+        Optional tag to append to the file stem
+
+    Returns
+    ==========
+    str
+        Path to the local copy
+    str
+        Path to the copy of the exemplar in the fails folder
+    """
+    # Path-ise filename
+    fileName = Path(fileName)
+    # Create new stem
+    if tag:
+        tag = "_" + tag
+    stem = fileName.stem + tag
+    # Construct new filename for local copy
+    localFileName = pjoin(TESTS_FAILS_PATH, stem + "_local" + fileName.suffix)
+    # Construct new filename for exemplar copy
+    exemplarFileName = pjoin(TESTS_FAILS_PATH, fileName.stem + fileName.suffix)
+
+    return localFileName, exemplarFileName
 
 
 def compareScreenshot(fileName, win, tag="", crit=5.0):
@@ -57,18 +94,18 @@ def compareScreenshot(fileName, win, tag="", crit=5.0):
             imgDat = np.array(frame.getdata())
             crit += 5  # be more relaxed because of the interpolation
         rms = np.std(imgDat-expDat)
-        if tag:
-            tag = "_" + tag
-        filenameLocal = fileName.replace('.png',f'{tag}_local.png')
+        localFileName, exemplarFileName = getFailFilenames(fileName, tag=tag)
         if rms >= crit/2:
             #there was SOME discrepancy
             logging.warning('PsychoPyTests: RMS=%.3g at threshold=%3.g'
                   % (rms, crit))
         if not rms<crit: #don't do `if rms>=crit because that doesn't catch rms=nan
-            frame.save(filenameLocal, optimize=1)
-            logging.warning('PsychoPyTests: Saving local copy into %s' % filenameLocal)
+            # If test fails, save local copy and copy of exemplar to fails folder
+            frame.save(localFileName, optimize=1)
+            expected.save(exemplarFileName, optimize=1)
+            logging.warning('PsychoPyTests: Saving local copy into %s' % localFileName)
         assert rms<crit, \
-            "RMS=%.3g at threshold=%.3g. Local copy in %s" % (rms, crit, filenameLocal)
+            "RMS=%.3g at threshold=%.3g. Local copy in %s" % (rms, crit, localFileName)
 
 
 def compareTextFiles(pathToActual, pathToCorrect, delim=None,
@@ -155,8 +192,7 @@ def compareTextFiles(pathToActual, pathToCorrect, delim=None,
                             %(lineN, wordN, repr(wordActual), repr(wordCorrect))
 
     except AssertionError as err:
-        pathToLocal, ext = os.path.splitext(pathToCorrect)
-        pathToLocal = pathToLocal+'_local'+ext
+        pathToLocal, pathToExemplar = getFailFilenames(pathToCorrect)
 
         # Set assertion type
         if not nLinesMatch:  # Fail if number of lines not equal
@@ -165,7 +201,8 @@ def compareTextFiles(pathToActual, pathToCorrect, delim=None,
             msg = 'Number of differences in {failed} exceeds the {tol}% tolerance'.format(failed=pathToActual,
                                                                                           tol=tolerance or 0)
         else:
-            shutil.copyfile(pathToActual,pathToLocal)
+            shutil.copyfile(pathToActual, pathToLocal)
+            shutil.copyfile(pathToCorrect, pathToExemplar)
             msg = "txtActual != txtCorr: Saving local copy to {}".format(pathToLocal)
         logging.error(msg)
         raise AssertionError(err)
@@ -213,10 +250,10 @@ def compareXlsxFiles(pathToActual, pathToCorrect):
                 error = "nf Cell %s: %s != %s" %(key, expVal, actVal)
                 break
     if error:
-        pathToLocal, ext = os.path.splitext(pathToCorrect)
-        pathToLocal = pathToLocal+'_local'+ext
-        shutil.copyfile(pathToActual,pathToLocal)
-        logging.warning("xlsxActual!=xlsxCorr: Saving local copy to %s" %pathToLocal)
+        pathToLocal, pathToExemplar = getFailFilenames(pathToCorrect)
+        shutil.copyfile(pathToActual, pathToLocal)
+        shutil.copyfile(pathToCorrect, pathToExemplar)
+        logging.warning("xlsxActual!=xlsxCorr: Saving local copy to %s" % pathToLocal)
         raise IOError(error)
 
 
@@ -250,5 +287,32 @@ def comparePixelColor(screen, color, coord=(0, 0), context="color_comparison"):
     # Assert
     cond = all(c for c in color == pixCol) or closeEnough
     if not cond:
-        frame.save(Path(TESTS_DATA_PATH) / (context + "_local.png"))
+        frame.save(Path(TESTS_FAILS_PATH) / (context + ".png"))
         raise AssertionError(f"Pixel color {pixCol} at {ogCoord} (x{screen.getContentScaleFactor()}) not equal to target color {color}")
+
+
+def forceBool(value, handler=any):
+    """
+    Force a value to a boolean, accounting for the possibility that it is an
+    array of booleans.
+
+    Parameters
+    ----------
+    value
+        Value to force
+    mode : str
+        Method to apply to values which fail builtin `bool` function, e.g. `any`
+        or `all` for boolean arrays.
+
+    Returns
+    -------
+    bool
+    """
+    try:
+        # attempt to make bool
+        value = bool(value)
+    except ValueError:
+        # if this fails,
+        value = handler(value)
+
+    return value
