@@ -2,22 +2,17 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """Functions and classes related to attribute handling
 """
-from __future__ import absolute_import, division, print_function
 
-from past.builtins import basestring
-from past.utils import old_div
-from builtins import object
 import numpy
-
 from psychopy import logging
 
 
-class attributeSetter(object):
+class attributeSetter:
     """Makes functions appear as attributes. Takes care of autologging.
     """
 
@@ -91,15 +86,13 @@ def setAttribute(self, attrib, value, log,
 
         # Apply operation except for the case when new or old value
         # are None or string-like
-        if (value is not None and
-                not isinstance(value, basestring) and
-                oldValue is not None and
-                not isinstance(oldValue, basestring)):
+        if (value is not None and type(value)!=str
+                and oldValue is not None and type(oldValue)!=str):
             value = numpy.array(value, float)
 
             # Calculate new value using operation
             if operation in ('', None):
-                if (value.shape is () and
+                if (value.shape == () and
                         not isinstance(oldValue, attributeSetter)):  # scalar
                     # Preserves dimensions in case oldValue is array-like.
                     value = oldValue * 0 + value
@@ -135,7 +128,7 @@ def setAttribute(self, attrib, value, log,
         # Trick to control logging of attributeSetter. Set logging in
         # self.autoLog
         autoLogOrig = self.autoLog  # save original value
-        # set to desired logging. log=None dafaults to autoLog
+        # set to desired logging. log=None defaults to autoLog
         self.__dict__['autoLog'] = log or autoLogOrig and log is None
         # set attribute, calling attributeSetter if it exists
         setattr(self, attrib, value)
@@ -156,8 +149,7 @@ def logAttrib(obj, log, attrib, value=None):
 
         # for numpy arrays bigger than 2x2 repr is slow (up to 1ms) so just
         # say it was an array
-        if isinstance(value, numpy.ndarray) \
-                and (value.ndim > 2 or len(value) > 2):
+        if isinstance(value, numpy.ndarray) and (value.ndim > 2 or value.size > 2):
             valStr = repr(type(value))
         else:
             valStr = value.__repr__()
