@@ -2,17 +2,24 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-"""Functions and classes related to color space conversion.
+"""Tools related to working with various color spaces.
+
+The routines provided in the module are used to transform color coordinates
+between spaces. Most of the functions here are *vectorized*, allowing for array
+inputs to convert multiple color values at once.
+
+**As of version 2021.0 of PsychoPy**, users ought to use the
+:class:`~psychopy.colors.Color` class for working with color coordinate values.
+
 """
-from __future__ import absolute_import, division, print_function
 
 __all__ = ['srgbTF', 'rec709TF', 'cielab2rgb', 'cielch2rgb', 'dkl2rgb',
-           'dklCart2rgb', 'rgb2dklCart', 'hsv2rgb', 'rgb2lms', 'lms2rgb']
+           'dklCart2rgb', 'rgb2dklCart', 'hsv2rgb', 'rgb2lms', 'lms2rgb',
+           'rgb2hsv', 'rescaleColor']
 
-from past.utils import old_div
 import numpy
 from psychopy import logging
 from psychopy.tools.coordinatetools import sph2cart
@@ -56,6 +63,69 @@ def unpackColors(colors):  # used internally, not exported by __all__
     return colors, orig_shape, orig_dim
 
 
+def rescaleColor(rgb, convertTo='signed', clip=False):
+    """Rescale RGB colors.
+
+    This function can be used to convert RGB value triplets from the PsychoPy
+    signed color format to the normalized OpenGL format.
+
+    PsychoPy represents colors using values between -1 and 1. However, colors
+    are commonly represented using values between 0 and 1 when working with
+    OpenGL and various other contexts. This function simply rescales values to
+    switch between these formats.
+
+    Parameters
+    ----------
+    rgb : `array_like`
+        1-, 2-, 3-D vector of RGB coordinates to convert. The last dimension
+        should be length-3 in all cases, specifying a single coordinate.
+    convertTo : `str`
+        If 'signed', this function will assume `rgb` is in OpenGL format [0:1]
+        and rescale them to PsychoPy's format [-1:1]. If 'unsigned', input
+        values are treated as OpenGL format and will be rescaled to use
+        PsychoPy's. Default is 'signed'.
+    clip : bool
+        Clip values to the range that can be represented on a display. This is
+        an optional step. Default is `False`.
+
+    Returns
+    -------
+    ndarray
+        Rescaled values with the same shape as `rgb`.
+
+    Notes
+    -----
+    The `convertTo` argument also accepts strings 'opengl' and 'psychopy'
+    as substitutes for 'signed' and 'unsigned', respectively. This might be more
+    explicit in some contexts.
+
+    Examples
+    --------
+    Convert a signed RGB value to unsigned format::
+
+        rgb_signed = [-1, 0, 1]
+        rgb_unsigned = rescaleColor(rgb_signed, convertTo='unsigned')
+
+    """
+    # While pretty simple, this operation is done often enough to justify having
+    # its own function to avoid writing it out all the time. It also explicitly
+    # shows the direction of which values are being rescaled to make code more
+    # readable.
+
+    if convertTo == 'signed' or convertTo == 'psychopy':
+        rgb_out = rgb * 2 - 1   # from OpenGL to PsychoPy format
+    elif convertTo == 'unsigned' or convertTo == 'opengl':
+        rgb_out = (rgb + 1) / 2.  # from PsychoPy to OpenGL
+    else:
+        raise ValueError("Invalid value for `convertTo`, can either be "
+                         "'signed' or 'unsigned'.")
+
+    if clip:
+        rgb_out = numpy.clip(rgb_out, -1 if convertTo == 'signed' else 0, 1)
+
+    return rgb_out
+
+
 def srgbTF(rgb, reverse=False, **kwargs):
     """Apply sRGB transfer function (or gamma) to linear RGB values.
 
@@ -101,10 +171,10 @@ def srgbTF(rgb, reverse=False, **kwargs):
 
 
 def rec709TF(rgb, **kwargs):
-    """Apply the Rec. 709 transfer function (or gamma) to linear RGB values.
+    """Apply the Rec 709 transfer function (or gamma) to linear RGB values.
 
     This transfer function is defined in the ITU-R BT.709 (2015) recommendation
-    document (http://www.itu.int/rec/R-REC-BT.709-6-201506-I/en) and is
+    document (https://www.itu.int/rec/R-REC-BT.709-6-201506-I/en) and is
     commonly used with HDTV televisions.
 
     Parameters
@@ -200,7 +270,7 @@ def cielab2rgb(lab,
     if conversionMatrix is None:
         # XYZ -> sRGB conversion matrix, assumes D65 white point
         # mdc - computed using makeXYZ2RGB with sRGB primaries
-        conversionMatrix = numpy.asmatrix([
+        conversionMatrix = numpy.asarray([
             [3.24096994, -1.53738318, -0.49861076],
             [-0.96924364, 1.8759675, 0.04155506],
             [0.05563008, -0.20397696, 1.05697151]
@@ -253,7 +323,7 @@ def cielab2rgb(lab,
     elif orig_dim == 3:
         rgb_out = numpy.reshape(rgb_out, orig_shape)
 
-    return rgb_out * 2.0 - 1.0
+    return rescaleColor(rgb_out, convertTo='psychopy')
 
 
 def cielch2rgb(lch,
@@ -262,12 +332,12 @@ def cielch2rgb(lch,
                transferFunc=None,
                clip=False,
                **kwargs):
-    """Transform CIE L*C*h* coordinates to RGB tristimulus values.
+    """Transform CIE `L*C*h*` coordinates to RGB tristimulus values.
 
     Parameters
     ----------
     lch : tuple, list or ndarray
-        1-, 2-, 3-D vector of CIE L*C*h* coordinates to convert. The last
+        1-, 2-, 3-D vector of CIE `L*C*h*` coordinates to convert. The last
         dimension should be length-3 in all cases specifying a single
         coordinate. The hue angle *h is expected in degrees.
     whiteXYZ : tuple, list or ndarray
@@ -331,12 +401,17 @@ def dkl2rgb(dkl, conversionMatrix=None):
     an accurate representation of the color space unless you supply a
     conversion matrix).
 
-    usage::
+    Examples
+    --------
+    Converting a single DKL color to RGB::
 
-        rgb(Nx3) = dkl2rgb(dkl_Nx3(el,az,radius), conversionMatrix)
-        rgb(NxNx3) = dkl2rgb(dkl_NxNx3(el,az,radius), conversionMatrix)
+        dkl = [90, 0, 1]
+        rgb = dkl2rgb(dkl, conversionMatrix)
 
     """
+    # make sure the input is an array
+    dkl = numpy.asarray(dkl)
+
     if conversionMatrix is None:
         conversionMatrix = numpy.asarray([
             # (note that dkl has to be in cartesian coords first!)
@@ -397,6 +472,92 @@ def dklCart2rgb(LUM, LM, S, conversionMatrix=None):
     return numpy.reshape(numpy.transpose(rgb), NxNx3)
 
 
+def rgb2hsv(rgb):
+    """Convert values from linear RGB to HSV colorspace.
+
+    Parameters
+    ----------
+    rgb : `array_like`
+        1-, 2-, 3-D vector of RGB coordinates to convert. The last dimension
+        should be length-3 in all cases, specifying a single coordinate.
+
+    Returns
+    -------
+    ndarray
+        HSV values with the same shape as the input.
+
+    """
+    # Based on https://www.geeksforgeeks.org/program-change-rgb-color-model-hsv-color-model/
+    rgb, orig_shape, orig_dim = unpackColors(rgb)
+
+    # need to rescale RGB values to 0.0 and 1.0
+    rgb = rescaleColor(rgb, convertTo='unsigned')
+
+    # get row min/max indices
+    rmax = numpy.argmax(rgb, axis=1)
+    rmin = numpy.argmin(rgb, axis=1)
+
+    # get min/max values for each color coordinate
+    sel = numpy.arange(len(rgb))
+    cmax = rgb[sel, rmax]
+    cmin = rgb[sel, rmin]
+
+    # compute the difference between the max and min color value
+    delta = cmax - cmin
+
+    # vector to return HSV values
+    hsv_out = numpy.zeros_like(rgb, dtype=float)
+
+    # --- calculate vibrancy ---
+    dzero = delta == 0  # if delta is zero the color is a shade of grey
+    inv_dzero = None
+    if numpy.any(dzero):  # vibrancy is 1
+        hsv_out[dzero, 2] = numpy.sum(rgb[dzero], axis=1) / 3.
+        inv_dzero = ~dzero
+
+    if inv_dzero is not None:
+        hsv_out[inv_dzero, 2] = cmax[inv_dzero]
+    else:
+        hsv_out[:, 2] = cmax[:]  # no B/W colors
+
+    # --- calculate saturation ---
+    hsv_out[:, 1] = numpy.where(cmax > 0.0, delta / cmax, 0.0)
+
+    # --- calculate hues ---
+    # views of each column
+    r = rgb[:, 0]
+    g = rgb[:, 1]
+    b = rgb[:, 2]
+
+    # select on rows where the RGB gun value is max and not `dzero`
+    sel_r = (rmax == 0) & inv_dzero if inv_dzero is not None else rmax == 0
+    sel_g = (rmax == 1) & inv_dzero if inv_dzero is not None else rmax == 1
+    sel_b = (rmax == 2) & inv_dzero if inv_dzero is not None else rmax == 2
+
+    if numpy.any(sel_r):  # if red == cmax
+        hsv_out[sel_r, 0] = \
+            (60 * ((g[sel_r] - b[sel_r]) / delta[sel_r]) + 360) % 360
+
+    if numpy.any(sel_g):  # if green == cmax
+        hsv_out[sel_g, 0] = \
+            (60 * ((b[sel_g] - r[sel_g]) / delta[sel_g]) + 120) % 360
+
+    if numpy.any(sel_b):  # if blue == cmax
+        hsv_out[sel_b, 0] = \
+            (60 * ((r[sel_b] - g[sel_b]) / delta[sel_b]) + 240) % 360
+
+    # round the hue angle value
+    hsv_out[:, 0] = numpy.round(hsv_out[:, 0])
+
+    # make the output match the dimensions/shape of input
+    if orig_dim == 1:
+        hsv_out = hsv_out[0]
+    elif orig_dim == 3:
+        hsv_out = numpy.reshape(hsv_out, orig_shape)
+
+    return hsv_out
+
+
 def hsv2rgb(hsv_Nx3):
     """Convert from HSV color space to RGB gun values.
 
@@ -412,14 +573,14 @@ def hsv2rgb(hsv_Nx3):
     PsychoPy functions.
     """
     # based on method in
-    # http://en.wikipedia.org/wiki/HSL_and_HSV#Converting_to_RGB
+    # https://en.wikipedia.org/wiki/HSL_and_HSV#Converting_to_RGB
 
     hsv_Nx3 = numpy.asarray(hsv_Nx3, dtype=float)
     # we expect a 2D array so convert there if needed
     origShape = hsv_Nx3.shape
     hsv_Nx3 = hsv_Nx3.reshape([-1, 3])
 
-    H_ = old_div((hsv_Nx3[:, 0] % 360), 60.0)  # this is H' in the wikipedia version
+    H_ = (hsv_Nx3[:, 0] % 360) / 60.0  # this is H' in the wikipedia version
     # multiply H and V to give chroma (color intensity)
     C = hsv_Nx3[:, 1] * hsv_Nx3[:, 2]
     X = C * (1 - abs(H_ % 2 - 1))
@@ -482,6 +643,40 @@ def lms2rgb(lms_Nx3, conversionMatrix=None):
     return numpy.transpose(rgb)  # return in the shape we received it
 
 
+def rgb2lms(rgb_Nx3, conversionMatrix=None):
+    """Convert from RGB to cone space (LMS).
+
+    Requires a conversion matrix, which will be generated from generic
+    Sony Trinitron phosphors if not supplied (note that you will not get
+    an accurate representation of the color space unless you supply a
+    conversion matrix)
+
+    usage::
+
+        lms_Nx3 = rgb2lms(rgb_Nx3(el,az,radius), conversionMatrix)
+
+    """
+
+    # its easier to use in the other orientation!
+    rgb_3xN = numpy.transpose(rgb_Nx3)
+
+    if conversionMatrix is None:
+        cones_to_rgb = numpy.asarray([
+            # L        M        S
+            [4.97068857, -4.14354132, 0.17285275],  # R
+            [-0.90913894, 2.15671326, -0.24757432],  # G
+            [-0.03976551, -0.14253782, 1.18230333]])  # B
+
+        logging.warning('This monitor has not been color-calibrated. '
+                        'Using default LMS conversion matrix.')
+    else:
+        cones_to_rgb = conversionMatrix
+    rgb_to_cones = numpy.linalg.inv(cones_to_rgb)
+
+    lms = numpy.dot(rgb_to_cones, rgb_3xN)
+    return numpy.transpose(lms)  # return in the shape we received it
+
+
 def rgb2dklCart(picture, conversionMatrix=None):
     """Convert an RGB image into Cartesian DKL space.
     """
@@ -516,38 +711,5 @@ def rgb2dklCart(picture, conversionMatrix=None):
 
     # Reshape the picture so that it's back to it's original shape
     dklPicture = numpy.reshape(numpy.transpose(dkl), origShape)
+
     return dklPicture
-
-
-def rgb2lms(rgb_Nx3, conversionMatrix=None):
-    """Convert from RGB to cone space (LMS).
-
-    Requires a conversion matrix, which will be generated from generic
-    Sony Trinitron phosphors if not supplied (note that you will not get
-    an accurate representation of the color space unless you supply a
-    conversion matrix)
-
-    usage::
-
-        lms_Nx3 = rgb2lms(rgb_Nx3(el,az,radius), conversionMatrix)
-
-    """
-
-    # its easier to use in the other orientation!
-    rgb_3xN = numpy.transpose(rgb_Nx3)
-
-    if conversionMatrix is None:
-        cones_to_rgb = numpy.asarray([
-            # L        M        S
-            [4.97068857, -4.14354132, 0.17285275],  # R
-            [-0.90913894, 2.15671326, -0.24757432],  # G
-            [-0.03976551, -0.14253782, 1.18230333]])  # B
-
-        logging.warning('This monitor has not been color-calibrated. '
-                        'Using default LMS conversion matrix.')
-    else:
-        cones_to_rgb = conversionMatrix
-    rgb_to_cones = numpy.linalg.inv(cones_to_rgb)
-
-    lms = numpy.dot(rgb_to_cones, rgb_3xN)
-    return numpy.transpose(lms)  # return in the shape we received it
