@@ -10,6 +10,7 @@
 import os
 import sys
 import subprocess  # for git commandline invocation
+from collections import OrderedDict
 from subprocess import CalledProcessError
 import psychopy  # for currently loaded version
 from psychopy import prefs
@@ -17,6 +18,7 @@ from psychopy import prefs
 from psychopy import logging, tools, web, constants, preferences
 from pkg_resources import parse_version
 from importlib import reload
+from packaging.version import Version
 
 USERDIR = prefs.paths['userPrefsDir']
 VER_SUBDIR = 'versions'
@@ -26,6 +28,93 @@ VERSIONSDIR = os.path.join(USERDIR, VER_SUBDIR)
 _localVersionsCache = []
 _remoteVersionsCache = []
 
+# define ranges of PsychoPy versions which support each Python version
+versionMap = OrderedDict({
+    Version('2.7'): (Version("0.0"), Version("2020.2.0")),
+    Version('3.6'): (Version("1.9"), Version("2022.1.0")),
+    Version('3.8'): (Version("2022.1.0"), Version("2024.1.0")),
+    Version('3.10'): (Version("2023.2.0"), None),
+})
+# fill out intermediate versions
+for n in range(13):
+    v = Version(f"3.{n}")
+    av = max([key for key in versionMap if key <= v])
+    versionMap[v] = versionMap[av]
+
+
+class VersionRange:
+    def __init__(self, first=None, last=None):
+        self.first = first
+        self.last = last
+
+    @property
+    def first(self):
+        return self._first
+
+    @first.setter
+    def first(self, value):
+        self._first = value
+        if self._first is not None:
+            self._first = Version(self._first)
+
+    @property
+    def last(self):
+        return self._last
+
+    @last.setter
+    def last(self, value):
+        self._last = value
+        if self._last is not None:
+            self._last = Version(self._last)
+
+    def __contains__(self, item):
+        # enforce Version type
+        if isinstance(item, str):
+            item = Version(item)
+        # if not less than or greater than, assume contains
+        lt = self > item
+        gt = self < item
+        return not any((lt, gt))
+
+    def __eq__(self, other):
+        return other in self
+
+    def __lt__(self, other):
+        # if no first version, nothing is less than
+        if self.last is None:
+            return False
+        # enforce Version type
+        if isinstance(other, str):
+            other = Version(other)
+        # otherwise compare to first version
+        return self.last < other
+
+    def __le__(self, other):
+        return self < other or other == self
+
+    def __gt__(self, other):
+        # if no last version, nothing is greater than
+        if self.first is None:
+            return False
+        # enforce Version type
+        if isinstance(other, str):
+            other = Version(other)
+        # otherwise compare to first version
+        return self.first > other
+
+    def __ge__(self, other):
+        return self > other or other in self
+
+    def __str__(self):
+        first = self.first
+        if first is None:
+            first = "up"
+        last = self.last
+        if last is None:
+            last = "latest"
+
+        return _translate("{} to {}").format(first, last)
+
 
 # ideally want localization for error messages
 # but don't want to have the lib/ depend on app/, drat
@@ -59,7 +148,7 @@ def getPsychoJSVersionStr(currentVersion, preferredVersion=''):
         # e.g. 2021.1.0 not 2021.1.0.dev3
         useVerStr = '.'.join(useVerStr.split('.')[:3])
     # PsychoJS doesn't have additional rc1 or dev1 releases
-    for versionSuffix in ["rc", "dev"]:
+    for versionSuffix in ["rc", "dev", "a", "b"]:
         if versionSuffix in useVerStr:
             useVerStr = useVerStr.split(versionSuffix)[0]
 
@@ -95,6 +184,8 @@ def useVersion(requestedVersion):
     See also:
         ensureMinimal()
     """
+    requestedVersion = str(requestedVersion)
+
     # Sanity Checks
     imported = _psychopyComponentsImported()
     if imported:
@@ -102,6 +193,9 @@ def useVersion(requestedVersion):
                          "PsychoPy modules. (Found: {})")
         raise RuntimeError(msg.format(imported))
 
+    # make sure PsychoPy and Python versions match
+    ensurePythonCompatibility(requestedVersion)
+
     # Get a proper full-version tag from a partial tag:
     reqdMajorMinorPatch = fullVersion(requestedVersion)
     logging.exp('Requested: useVersion({}) = {}'.format(requestedVersion,
@@ -113,17 +207,6 @@ def useVersion(requestedVersion):
     if not os.path.isdir(VERSIONSDIR):
         _clone(requestedVersion)  # Allow the versions subdirectory to be built
 
-    py3Compatible = _versionFilter(versionOptions(local=False), None)
-    py3Compatible += _versionFilter(availableVersions(local=False), None)
-    py3Compatible.sort(reverse=True)
-
-    if reqdMajorMinorPatch not in py3Compatible:
-        msg = _translate("Please request a version of PsychoPy that is compatible with Python 3. "
-                         "You can choose from the following versions: {}. "
-                         "Alternatively, run a Python 2 installation of PsychoPy < v1.9.0.\n")
-        logging.error(msg.format(py3Compatible))
-        return
-
     if psychopy.__version__ != reqdMajorMinorPatch:
         # Switching required, so make sure `git` is available.
         if not _gitPresent():
@@ -148,6 +231,41 @@ def useVersion(requestedVersion):
     return psychopy.__version__
 
 
+def ensurePythonCompatibility(requestedVersion):
+    """
+    Ensure that the requested version of PsychoPy is compatible with the currently running version of Python, raising
+    an EnvironmentError if not.
+
+    Parameters
+    ----------
+    requestedVersion : str
+        PsychoPy version being requested (e.g. "2023.2.0")
+    """
+    requestedVersion = Version(requestedVersion)
+
+    # get Python version
+    pyVersion = Version(".".join(
+        [str(sys.version_info.major), str(sys.version_info.minor)]
+    ))
+    # get first and last PsychoPy version to support it
+    firstVersion, lastVersion = versionMap.get(pyVersion, (None, None))
+    # check supported
+    _msg = _translate(
+        "Requested PsychoPy version {requested} does not support installed Python version {py}. The {mode} version "
+        "of PsychoPy to support {py} was version {key}.\n"
+        "\n"
+        "Try either choosing a different version of PsychoPy or installing a different version of Python - some "
+        "standalone PsychoPy releases include installers for multiple versions."
+    ).format(requested=requestedVersion, py=pyVersion, mode="{mode}", key="{key}")
+    if firstVersion is not None and firstVersion > requestedVersion:
+        # if Python version is too new for PsychoPy...
+        raise EnvironmentError(_msg.format(mode="first", key=firstVersion))
+
+    if lastVersion is not None and lastVersion < requestedVersion:
+        # if PsychoPy version is too new for Python...
+        raise EnvironmentError(_msg.format(mode="last", key=lastVersion))
+
+
 def ensureMinimal(requiredVersion):
     """Raise a RuntimeError if the current version < `requiredVersion`.
 
@@ -189,7 +307,7 @@ def _switchToVersion(requestedVersion):
         else:
             _clone(requestedVersion)
     except (CalledProcessError, OSError) as e:
-        if 'did not match any file(s) known to git' in e.output:
+        if 'did not match any file(s) known to git' in str(e):
             msg = _translate("'{}' is not a valid PsychoPy version.")
             logging.error(msg.format(requestedVersion))
             raise RuntimeError(msg)
@@ -351,9 +469,16 @@ def _checkout(requestedVersion):
         # Grab new tags
         msg = _translate("Couldn't find version {} locally. Trying github...")
         logging.info(msg.format(requestedVersion))
-        subprocess.check_output('git fetch github --tags'.split(),
-                                cwd=VERSIONSDIR,
-                                env=constants.ENVIRON).decode('UTF-8')
+
+        out, stdout, stderr = _call_process(f"git fetch github --tags")
+
+        # check error code
+        if out.returncode != 0:
+            logging.error(stderr)
+            raise ChildProcessError(
+                'Error: process exited with code {}, check log for '
+                'output.'.format(out.returncode))
+
         # is requested here now? forceCheck to refresh cache
         if requestedVersion not in _localVersions(forceCheck=True):
             msg = _translate("{} is not currently available.")
@@ -361,13 +486,18 @@ def _checkout(requestedVersion):
             return ''
 
     # Checkout the requested tag
-    cmd = ['git', 'checkout', requestedVersion]
-    out = subprocess.check_output(cmd,
-                                  stderr=subprocess.STDOUT,
-                                  cwd=VERSIONSDIR,
-                                  env=constants.ENVIRON).decode('UTF-8')
-    logging.debug(out)
-    logging.exp('Success:  ' + ' '.join(cmd))
+    out, stdout, stderr = _call_process(f"git reset --hard") # in case of any accidental local changes
+    out, stdout, stderr = _call_process(f"git checkout {requestedVersion}") #
+
+    # check error code
+    if out.returncode != 0:
+        logging.error(stderr)
+        raise ChildProcessError(
+            'Error: process exited with code {}, check log for '
+            'output.'.format(out.returncode))
+
+    logging.exp('Success:  ' + ' '.join(f"git checkout {requestedVersion}"))
+
     return requestedVersion
 
 
@@ -400,3 +530,19 @@ def _gitPresent():
 
 def _psychopyComponentsImported():
     return [name for name in globals() if name in psychopy.__all__]
+
+def _call_process(cmd, log=True):
+    """Convenience call to open subprocess, and pipe stdout to debug"""
+    if type(cmd) in [str, bytes]:
+        cmd = cmd.split()
+    out = subprocess.Popen(
+        cmd,
+        stderr=subprocess.PIPE,
+        stdout=subprocess.PIPE,
+        cwd=VERSIONSDIR,
+        env=constants.ENVIRON)
+    stdout, stderr = out.communicate()
+    if log:
+        logging.debug(stdout)
+
+    return out, stdout, stderr
\ No newline at end of file
