@@ -6,19 +6,17 @@ independently controlled. Suitable for creating 'global form' stimuli or more
 detailed random dot stimuli."""
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, division, print_function
-
-from builtins import str
-from past.utils import old_div
-
 # Ensure setting pyglet.options['debug_gl'] to False is done prior to any
 # other calls to pyglet or pyglet submodules, otherwise it may not get picked
 # up by the pyglet GL engine and have no effect.
 # Shaders will work but require OpenGL2.0 drivers AND PyOpenGL3.0+
 import pyglet
+
+from ..colors import Color
+
 pyglet.options['debug_gl'] = False
 import ctypes
 GL = pyglet.gl
@@ -33,13 +31,13 @@ from psychopy.tools.arraytools import val2array
 from psychopy.tools.attributetools import attributeSetter, logAttrib, setAttribute
 from psychopy.tools.monitorunittools import convertToPix
 from psychopy.visual.helpers import setColor
-from psychopy.visual.basevisual import MinimalStim, TextureMixin
+from psychopy.visual.basevisual import MinimalStim, TextureMixin, ColorMixin
 from . import globalVars
 
 import numpy
 
 
-class ElementArrayStim(MinimalStim, TextureMixin):
+class ElementArrayStim(MinimalStim, TextureMixin, ColorMixin):
     """This stimulus class defines a field of elements whose behaviour can
     be independently controlled. Suitable for creating 'global form' stimuli
     or more detailed random dot stimuli.
@@ -115,12 +113,12 @@ class ElementArrayStim(MinimalStim, TextureMixin):
         self.verticesBase = xys
         self._needVertexUpdate = True
         self._needColorUpdate = True
-        self.useShaders = True
+        self._RGBAs = None
         self.interpolate = interpolate
         self.__dict__['fieldDepth'] = fieldDepth
         self.__dict__['depths'] = depths
-        if self.win.winType != 'pyglet':
-            raise TypeError('ElementArrayStim requires a pyglet context')
+        if self.win.winType == 'pygame':
+            raise TypeError('ElementArrayStim is not supported in a pygame context')
         if not self.win._haveShaders:
             raise Exception("ElementArrayStim requires shaders support"
                             " and floating point textures")
@@ -235,24 +233,24 @@ class ElementArrayStim(MinimalStim, TextureMixin):
             rand = numpy.random.rand
             if self.fieldShape in ('sqr', 'square'):
                 # initialise a random array of X,Y
-                self.__dict__['xys'] = rand(self.nElements, 2) * fsz - old_div(fsz, 2)
+                self.__dict__['xys'] = rand(self.nElements, 2) * fsz - (fsz / 2)
                 # gone outside the square
-                xxx = (self.xys[:, 0] + old_div(fsz[0], 2)) % fsz[0]
-                yyy = (self.xys[:, 1] + old_div(fsz[1], 2)) % fsz[1]
-                self.__dict__['xys'][:, 0] = xxx - old_div(fsz[0], 2)
-                self.__dict__['xys'][:, 1] = yyy - old_div(fsz[1], 2)
-            elif self.fieldShape is 'circle':
+                xxx = (self.xys[:, 0] + (fsz[0] / 2)) % fsz[0]
+                yyy = (self.xys[:, 1] + (fsz[1] / 2)) % fsz[1]
+                self.__dict__['xys'][:, 0] = xxx - (fsz[0] / 2)
+                self.__dict__['xys'][:, 1] = yyy - (fsz[1] / 2)
+            elif self.fieldShape == 'circle':
                 # take twice as many elements as we need (and cull the ones
                 # outside the circle)
                 # initialise a random array of X,Y
-                xys = rand(self.nElements * 2, 2) * fsz - old_div(fsz, 2)
+                xys = rand(self.nElements * 2, 2) * fsz - (fsz / 2)
                 # gone outside the square
-                xys[:, 0] = ((xys[:, 0] + old_div(fsz[0], 2)) % fsz[0]) - old_div(fsz[0], 2)
-                xys[:, 1] = ((xys[:, 1] + old_div(fsz[1], 2)) % fsz[1]) - old_div(fsz[1], 2)
+                xys[:, 0] = ((xys[:, 0] + (fsz[0] / 2)) % fsz[0]) - (fsz[0] / 2)
+                xys[:, 1] = ((xys[:, 1] + (fsz[1] / 2)) % fsz[1]) - (fsz[1] / 2)
                 # use a circular envelope and flips dot to opposite edge
                 # if they fall beyond radius.
                 # NB always circular - uses fieldSize in X only
-                normxy = old_div(xys, (old_div(fsz, 2.0)))
+                normxy = xys / (fsz / 2.0)
                 dotDist = numpy.sqrt((normxy[:, 0]**2.0 + normxy[:, 1]**2.0))
                 self.__dict__['xys'] = xys[dotDist < 1.0, :][0:self.nElements]
         else:
@@ -391,8 +389,8 @@ class ElementArrayStim(MinimalStim, TextureMixin):
         """
         self.setColors(value, operation)
 
-    @attributeSetter
-    def colors(self, color):
+    @property
+    def colors(self):
         """Specifying the color(s) of the elements.
         Should be Nx1 (different intensities), Nx3 (different colors) or
         1x3 (for a single color).
@@ -405,41 +403,33 @@ class ElementArrayStim(MinimalStim, TextureMixin):
         Use ``setColors()`` if you want to set colors and colorSpace
         simultaneously or use operations on colors.
         """
-        self.setColors(color)
+        if hasattr(self, '_colors'):
+            # Return array of rendered colors
+            return self._colors.render(self.colorSpace)
+    @colors.setter
+    def colors(self, value):
+        # Create blank array of colors
+        self._colors = Color(value, self.colorSpace, self.contrast)
+        self._needColorUpdate = True
 
-    @attributeSetter
-    def colorSpace(self, colorSpace):
-        """The type of color specified is the same as those in other stimuli
-        ('rgb','dkl','lms'...) but note that for this stimulus you cannot
-        currently use text-based colors (e.g. names or hex values).
-
-        Keeping this exception in mind, see :ref:`colorspaces` for more info.
-        """
-        self.__dict__['colorSpace'] = colorSpace
-
-    def setColors(self, color, colorSpace=None, operation='', log=None):
+    def setColors(self, colors, colorSpace=None, operation='', log=None):
         """See ``color`` for more info on the color parameter  and
         ``colorSpace`` for more info in the colorSpace parameter.
         """
-        setColor(self, color, colorSpace=colorSpace, operation=operation,
-                 rgbAttrib='rgbs',  # or 'fillRGB' etc
-                 colorAttrib='colors',
-                 colorSpaceAttrib='colorSpace')
-        logAttrib(self, log, 'colors', value='%s (%s)' % (self.colors,
-                                                          self.colorSpace))
+        self.colorSpace = colorSpace
+        self.colors = colors
 
-        # check shape
-        if self.rgbs.shape in ((), (1,), (3,)):
-            self.rgbs = numpy.resize(self.rgbs, [self.nElements, 3])
-        elif self.rgbs.shape in ((self.nElements,), (self.nElements, 1)):
-            self.rgbs.shape = (self.nElements, 1)  # set to be 2D
-            self.rgbs = self.rgbs.repeat(3, 1)  # repeat once on dim 1
-        elif self.rgbs.shape == (self.nElements, 3):
-            pass  # all is good
-        else:
-            raise ValueError("New value for setRgbs should be either "
-                             "Nx1, Nx3 or a single value")
-        self._needColorUpdate = True
+    @property
+    def opacity(self):
+        if hasattr(self, "_opacity"):
+            return self._opacity
+
+    @opacity.setter
+    def opacity(self, value):
+        self._opacity = value
+        if hasattr(self, "_colors"):
+            # Set the alpha value of each color to be the desired opacity
+            self._colors.alpha = value
 
     @attributeSetter
     def contrs(self, value):
@@ -450,7 +440,14 @@ class ElementArrayStim(MinimalStim, TextureMixin):
 
         :ref:`Operations <attrib-operations>` are supported.
         """
-        self.__dict__['contrs'] = self._makeNx1(value)
+        # Convert to an Nx1 numpy array
+        value = self._makeNx1(value)
+        # If colors is too short, extend it
+        self._colors.rgb = numpy.resize(self._colors.rgb, (len(value), 3))
+        # Set
+        self._colors.contrast = value
+        # Store value and update
+        self.__dict__['contrs'] = value
         self._needColorUpdate = True
 
     def setContrs(self, value, operation='', log=None):
@@ -632,19 +629,10 @@ class ElementArrayStim(MinimalStim, TextureMixin):
         each vertex of each element.
         """
         N = self.nElements
-        self._RGBAs = numpy.zeros([N, 4], 'd')
-        if self.colorSpace in ('rgb', 'dkl', 'lms', 'hsv'):
-            # these spaces are 0-centred
-            self._RGBAs[:, 0:3] = (self.rgbs[:, :] *
-                self.contrs[:].reshape([N, 1]).repeat(3, 1) / 2 + 0.5)
-        else:
-            self._RGBAs[:, 0:3] = (self.rgbs *
-                self.contrs[:].reshape([N, 1]).repeat(3, 1) / 255.0)
-
-        self._RGBAs[:, -1] = self.opacities.reshape([N, ])
-        # repeat for the 4 vertices in the grid
-        self._RGBAs = self._RGBAs.reshape([N, 1, 4]).repeat(4, 1)
-
+        _RGBAs = numpy.zeros([len(self.verticesPix), 4], 'd')
+        _RGBAs[:,:] = self._colors.render('rgba1')
+        _RGBAs[:, -1] = self.opacities.reshape([N, ])
+        self._RGBAs = _RGBAs.reshape([len(self.verticesPix), 1, 4]).repeat(4, 1)
         self._needColorUpdate = False
 
     def updateTextureCoords(self):
@@ -659,10 +647,10 @@ class ElementArrayStim(MinimalStim, TextureMixin):
         # for the main texture
         # sf is dependent on size (openGL default)
         if self.units in ['norm', 'pix', 'height']:
-            L = old_div(-self.sfs[:, 0], 2) - self.phases[:, 0] + 0.5
-            R = old_div(+self.sfs[:, 0], 2) - self.phases[:, 0] + 0.5
-            T = old_div(+self.sfs[:, 1], 2) - self.phases[:, 1] + 0.5
-            B = old_div(-self.sfs[:, 1], 2) - self.phases[:, 1] + 0.5
+            L = (-self.sfs[:, 0] / 2) - self.phases[:, 0] + 0.5
+            R = (+self.sfs[:, 0] / 2) - self.phases[:, 0] + 0.5
+            T = (+self.sfs[:, 1] / 2) - self.phases[:, 1] + 0.5
+            B = (-self.sfs[:, 1] / 2) - self.phases[:, 1] + 0.5
         else:
             # we should scale to become independent of size
             L = (-self.sfs[:, 0] * self.sizes[:, 0] / 2
@@ -733,5 +721,5 @@ class ElementArrayStim(MinimalStim, TextureMixin):
         # remove textures from graphics card to prevent OpenGl memory leak
         try:
             self.clearTextures()
-        except ModuleNotFoundError:
+        except (ImportError, ModuleNotFoundError, TypeError):
             pass  # has probably been garbage-collected already
