@@ -138,6 +138,7 @@ class Form(BaseVisualStim, ContainerMixin, ColorMixin):
                  units='height',
                  randomize=False,
                  autoLog=True,
+                 depth=0,
                  # legacy
                  color=None,
                  foreColor=None
@@ -154,7 +155,7 @@ class Form(BaseVisualStim, ContainerMixin, ColorMixin):
         self.itemPadding = itemPadding
         self.scrollSpeed = self.setScrollSpeed(self.items, 4)
         self.units = units
-        self.depth = 0
+        self.depth = depth
 
         # Appearance
         self.colorSpace = colorSpace
@@ -701,7 +702,7 @@ class Form(BaseVisualStim, ContainerMixin, ColorMixin):
             The aperture setting viewable area for forms
         """
         aperture = psychopy.visual.Aperture(win=self.win,
-                                            name='aperture',
+                                            name=f"{self.name}_aperture",
                                             units=self.units,
                                             shape='square',
                                             size=self.size,
@@ -875,7 +876,7 @@ class Form(BaseVisualStim, ContainerMixin, ColorMixin):
         # draw the box and scrollbar
         self._drawExternalDecorations()
         # enable aperture
-        self.aperture.enable()
+        self.aperture._reset()
         # draw the box and scrollbar
         self._drawDecorations()
         # Draw question and response objects
@@ -914,6 +915,24 @@ class Form(BaseVisualStim, ContainerMixin, ColorMixin):
         self._complete = (nIncomplete == 0)
         return copy.copy(self.items)  # don't want users changing orig
 
+    def reset(self):
+        """
+        Clear all responses and set all items to their initial values.
+        """
+        # Iterate through all items
+        for item in self.items:
+            # If item doesn't have a response ctrl, skip it
+            if "responseCtrl" not in item:
+                continue
+            # If response ctrl is a slider, set its rating to None
+            if isinstance(item['responseCtrl'], psychopy.visual.Slider):
+                item['responseCtrl'].rating = None
+            # If response ctrl is a textbox, set its text to blank
+            elif isinstance(item['responseCtrl'], psychopy.visual.TextBox2):
+                item['responseCtrl'].text = ""
+        # Set scrollbar to top
+        self.scrollbar.rating = 1
+
     def addDataToExp(self, exp, itemsAs='rows'):
         """Gets the current Form data and inserts into an
         :class:`~psychopy.experiment.ExperimentHandler` object either as rows
