@@ -11,6 +11,7 @@ from psychopy import logging
 from psychopy.tools.attributetools import attributeSetter, setAttribute
 from psychopy.visual.basevisual import WindowMixin, ColorMixin
 from psychopy.visual.helpers import setColor
+from psychopy.colors import Color, colorSpaces
 import psychopy.tools.mathtools as mt
 import psychopy.tools.gltools as gt
 import psychopy.tools.arraytools as at
@@ -45,6 +46,7 @@ class LightSource:
                  specularColor=(1., 1., 1.),
                  ambientColor=(0., 0., 0.),
                  colorSpace='rgb',
+                 contrast=1.0,
                  lightType='point',
                  attenuation=(1, 0, 0)):
         """
@@ -65,8 +67,11 @@ class LightSource:
             Specular light color.
         ambientColor : array_like
             Ambient light color.
-        colorSpace : str
-            Colorspace for `diffuse`, `specular`, and `ambient` colors.
+        colorSpace : str or None
+            Colorspace for diffuse, specular, and ambient color components.
+        contrast : float
+            Contrast of the lighting color components. This acts as a 'gain'
+            factor which scales color values. Must be between 0.0 and 1.0.
         attenuation : array_like
             Values for the constant, linear, and quadratic terms of the lighting
             attenuation formula. Default is (1, 0, 0) which results in no
@@ -76,26 +81,37 @@ class LightSource:
         self.win = win
 
         self._pos = np.zeros((4,), np.float32)
-        self._diffuseColor = np.zeros((3,), np.float32)
-        self._specularColor = np.zeros((3,), np.float32)
-        self._ambientColor = np.zeros((3,), np.float32)
+        self._diffuseColor = Color()
+        self._specularColor = Color()
+        self._ambientColor = Color()
+        self._lightType = None  # set later
 
         # internal RGB values post colorspace conversion
         self._diffuseRGB = np.array((0., 0., 0., 1.), np.float32)
         self._specularRGB = np.array((0., 0., 0., 1.), np.float32)
         self._ambientRGB = np.array((0., 0., 0., 1.), np.float32)
 
+        self.contrast = contrast
         self.colorSpace = colorSpace
+
+        # set the colors
         self.diffuseColor = diffuseColor
         self.specularColor = specularColor
         self.ambientColor = ambientColor
 
-        self._lightType = lightType
+        self.lightType = lightType
         self.pos = pos
 
         # attenuation factors
         self._kAttenuation = np.asarray(attenuation, np.float32)
 
+    # --------------------------------------------------------------------------
+    # Lighting
+    #
+    # Properties about the lighting position and type. This affects the shading
+    # of the material.
+    #
+
     @property
     def pos(self):
         """Position of the light source in the scene in scene units."""
@@ -106,13 +122,13 @@ class LightSource:
         self._pos = np.zeros((4,), np.float32)
         self._pos[:3] = value
 
-        if self._lightType == 'point':
+        if self._lightType == 'point':  # if a point source then `w` == 1.0
             self._pos[3] = 1.0
 
     @property
     def lightType(self):
         """Type of light source, can be 'point' or 'directional'."""
-        return self._pos[:3]
+        return self._lightType
 
     @lightType.setter
     def lightType(self, value):
@@ -127,79 +143,6 @@ class LightSource:
                 "Unknown `lightType` specified, must be 'directional' or "
                 "'point'.")
 
-
-    @property
-    def diffuseColor(self):
-        """Diffuse color of the material."""
-        return self._diffuseColor
-
-    @diffuseColor.setter
-    def diffuseColor(self, value):
-        self._diffuseColor = np.asarray(value, np.float32)
-        setColor(self, value, colorSpace=self.colorSpace, operation=None,
-                 rgbAttrib='diffuseRGB', colorAttrib='diffuseColor',
-                 colorSpaceAttrib='colorSpace')
-
-    @property
-    def diffuseRGB(self):
-        """Diffuse color of the material."""
-        return self._diffuseRGB[:3]
-
-    @diffuseRGB.setter
-    def diffuseRGB(self, value):
-        # make sure the color we got is 32-bit float
-        self._diffuseRGB = np.zeros((4,), np.float32)
-        self._diffuseRGB[:3] = (value + 1) / 2.0
-        self._diffuseRGB[3] = 1.0
-
-    @property
-    def specularColor(self):
-        """Specular color of the material."""
-        return self._specularColor
-
-    @specularColor.setter
-    def specularColor(self, value):
-        self._specularColor = np.asarray(value, np.float32)
-        setColor(self, value, colorSpace=self.colorSpace, operation=None,
-                 rgbAttrib='specularRGB', colorAttrib='specularColor',
-                 colorSpaceAttrib='colorSpace')
-
-    @property
-    def specularRGB(self):
-        """Diffuse color of the material."""
-        return self._specularRGB[:3]
-
-    @specularRGB.setter
-    def specularRGB(self, value):
-        # make sure the color we got is 32-bit float
-        self._specularRGB = np.zeros((4,), np.float32)
-        self._specularRGB[:3] = (value + 1) / 2.0
-        self._specularRGB[3] = 1.0
-
-    @property
-    def ambientColor(self):
-        """Ambient color of the material."""
-        return self._ambientColor
-
-    @ambientColor.setter
-    def ambientColor(self, value):
-        self._ambientColor = np.asarray(value, np.float32)
-        setColor(self, value, colorSpace=self.colorSpace, operation=None,
-                 rgbAttrib='ambientRGB', colorAttrib='ambientColor',
-                 colorSpaceAttrib='colorSpace')
-
-    @property
-    def ambientRGB(self):
-        """Diffuse color of the material."""
-        return self._ambientRGB[:3]
-
-    @ambientRGB.setter
-    def ambientRGB(self, value):
-        # make sure the color we got is 32-bit float
-        self._ambientRGB = np.zeros((4,), np.float32)
-        self._ambientRGB[:3] = (value + 1) / 2.0
-        self._ambientRGB[3] = 1.0
-
     @property
     def attenuation(self):
         """Values for the constant, linear, and quadratic terms of the lighting
@@ -211,6 +154,251 @@ class LightSource:
     def attenuation(self, value):
         self._kAttenuation = np.asarray(value, np.float32)
 
+    # --------------------------------------------------------------------------
+    # Lighting colors
+    #
+
+    @property
+    def colorSpace(self):
+        """The name of the color space currently being used (`str` or `None`).
+
+        For strings and hex values this is not needed. If `None` the default
+        `colorSpace` for the stimulus is used (defined during initialisation).
+
+        Please note that changing `colorSpace` does not change stimulus
+        parameters. Thus, you usually want to specify `colorSpace` before
+        setting the color.
+
+        """
+        if hasattr(self, '_colorSpace'):
+            return self._colorSpace
+        else:
+            return 'rgba'
+
+    @colorSpace.setter
+    def colorSpace(self, value):
+        if value in colorSpaces:
+            self._colorSpace = value
+        else:
+            logging.error(f"'{value}' is not a valid color space")
+
+    @property
+    def contrast(self):
+        """A value that is simply multiplied by the color (`float`).
+
+        This may be used to adjust the gain of the light source. This is applied
+        to all lighting color components.
+
+        Examples
+        --------
+        Basic usage::
+
+            stim.contrast =  1.0  # unchanged contrast
+            stim.contrast =  0.5  # decrease contrast
+            stim.contrast =  0.0  # uniform, no contrast
+            stim.contrast = -0.5  # slightly inverted
+            stim.contrast = -1.0  # totally inverted
+
+        Setting contrast outside range -1 to 1 is permitted, but may
+        produce strange results if color values exceeds the monitor limits.::
+
+            stim.contrast =  1.2  # increases contrast
+            stim.contrast = -1.2  # inverts with increased contrast
+
+        """
+        return self._diffuseColor.contrast
+
+    @contrast.setter
+    def contrast(self, value):
+        self._diffuseColor.contrast = value
+        self._specularColor.contrast = value
+        self._ambientColor.contrast = value
+
+    @property
+    def diffuseColor(self):
+        """Diffuse color for the light source (`psychopy.color.Color`,
+        `ArrayLike` or None).
+        """
+        return self._diffuseColor.render(self.colorSpace)
+
+    @diffuseColor.setter
+    def diffuseColor(self, value):
+        if isinstance(value, Color):
+            self._diffuseColor = value
+        else:
+            self._diffuseColor = Color(
+                value,
+                self.colorSpace,
+                contrast=self.contrast)
+
+        if not self._diffuseColor:
+            # If given an invalid color, set as transparent and log error
+            self._diffuseColor = Color()
+            logging.error(f"'{value}' is not a valid {self.colorSpace} color")
+
+        # set the RGB values
+        self._diffuseRGB[:3] = self._diffuseColor.rgb1
+        self._diffuseRGB[3] = self._diffuseColor.opacity
+
+    def setDiffuseColor(self, color, colorSpace=None, operation='', log=None):
+        """Set the diffuse color for the light source. Use this function if you
+        wish to supress logging or apply operations on the color component.
+
+        Parameters
+        ----------
+        color : ArrayLike or `~psychopy.colors.Color`
+            Color to set as the diffuse component of the light source.
+        colorSpace : str or None
+            Colorspace to use. This is only used to set the color, the value of
+            `diffuseColor` after setting uses the color space of the object.
+        operation : str
+            Operation string.
+        log : bool or None
+            Enable logging.
+
+        """
+        setColor(
+            obj=self,
+            colorAttrib="diffuseColor",
+            color=color,
+            colorSpace=colorSpace or self.colorSpace,
+            operation=operation,
+            log=log)
+
+    @property
+    def specularColor(self):
+        """Specular color of the light source (`psychopy.color.Color`,
+        `ArrayLike` or None).
+        """
+        return self._specularColor.render(self.colorSpace)
+
+    @specularColor.setter
+    def specularColor(self, value):
+        if isinstance(value, Color):
+            self._specularColor = value
+        else:
+            self._specularColor = Color(
+                value,
+                self.colorSpace,
+                contrast=self.contrast)
+
+        if not self._specularColor:
+            # If given an invalid color, set as transparent and log error
+            self._specularColor = Color()
+            logging.error(f"'{value}' is not a valid {self.colorSpace} color")
+
+        self._specularRGB[:3] = self._specularColor.rgb1
+        self._specularRGB[3] = self._specularColor.opacity
+
+    def setSpecularColor(self, color, colorSpace=None, operation='', log=None):
+        """Set the diffuse color for the light source. Use this function if you
+        wish to supress logging or apply operations on the color component.
+
+        Parameters
+        ----------
+        color : ArrayLike or `~psychopy.colors.Color`
+            Color to set as the specular component of the light source.
+        colorSpace : str or None
+            Colorspace to use. This is only used to set the color, the value of
+            `diffuseColor` after setting uses the color space of the object.
+        operation : str
+            Operation string.
+        log : bool or None
+            Enable logging.
+
+        """
+        setColor(
+            obj=self,
+            colorAttrib="specularColor",
+            color=color,
+            colorSpace=colorSpace or self.colorSpace,
+            operation=operation,
+            log=log)
+
+    @property
+    def ambientColor(self):
+        """Ambient color of the light source (`psychopy.color.Color`,
+        `ArrayLike` or None).
+
+        The ambient color component is used to simulate indirect lighting caused
+        by the light source. For instance, light bouncing off adjacent surfaces
+        or atmospheric scattering if the light source is a sun. This is
+        independent of the global ambient color.
+
+        """
+        return self._ambientColor.render(self.colorSpace)
+
+    @ambientColor.setter
+    def ambientColor(self, value):
+        if isinstance(value, Color):
+            self._ambientColor = value
+        else:
+            self._ambientColor = Color(
+                value,
+                self.colorSpace,
+                contrast=self.contrast)
+
+        if not self._ambientColor:
+            # If given an invalid color, set as transparent and log error
+            self._ambientColor = Color()
+            logging.error(f"'{value}' is not a valid {self.colorSpace} color")
+
+        self._ambientRGB[:3] = self._ambientColor.rgb1
+        self._ambientRGB[3] = self._ambientColor.opacity
+
+    def setAmbientColor(self, color, colorSpace=None, operation='', log=None):
+        """Set the ambient color for the light source.
+
+        Use this function if you wish to supress logging or apply operations on
+        the color component.
+
+        Parameters
+        ----------
+        color : ArrayLike or `~psychopy.colors.Color`
+            Color to set as the ambient component of the light source.
+        colorSpace : str or None
+            Colorspace to use. This is only used to set the color, the value of
+            `ambientColor` after setting uses the color space of the object.
+        operation : str
+            Operation string.
+        log : bool or None
+            Enable logging.
+
+        """
+        setColor(
+            obj=self,
+            colorAttrib="ambientColor",
+            color=color,
+            colorSpace=colorSpace or self.colorSpace,
+            operation=operation,
+            log=log)
+
+    # --------------------------------------------------------------------------
+    # Lighting RGB colors
+    #
+    # These are the color values for the light which will be passed to the
+    # shader. We protect these values since we don't want the user changing the
+    # array type or size.
+    #
+
+    @property
+    def diffuseRGB(self):
+        """Diffuse RGB1 color of the material. This value is passed to OpenGL.
+        """
+        return self._diffuseRGB
+
+    @property
+    def specularRGB(self):
+        """Specular RGB1 color of the material. This value is passed to OpenGL.
+        """
+        return self._specularRGB
+
+    @property
+    def ambientRGB(self):
+        """Ambient RGB1 color of the material. This value is passed to OpenGL.
+        """
+        return self._ambientRGB
+
 
 class SceneSkybox:
     """Class to render scene skyboxes.
@@ -404,7 +592,8 @@ class BlinnPhongMaterial:
     primitives with respect to lighting, such as color (diffuse, specular,
     ambient, and emission), shininess, and textures. Simple materials are
     intended to work with features supported by the fixed-function OpenGL
-    pipeline.
+    pipeline. However, one may use shaders that implement the Blinn-Phong
+    shading model for per-pixel lighting.
 
     If shaders are enabled, the colors of objects will appear different than
     without. This is due to the lighting/material colors being computed on a
@@ -434,6 +623,37 @@ class BlinnPhongMaterial:
     in the lighting object. Values `k0=1.0, k1=0.0, and k2=0.0` results in a
     light that does not fall-off with distance.
 
+    Parameters
+    ----------
+    win : `~psychopy.visual.Window` or `None`
+        Window this material is associated with, required for shaders and some
+        color space conversions.
+    diffuseColor : array_like
+        Diffuse material color (r, g, b) with values between -1.0 and 1.0.
+    specularColor : array_like
+        Specular material color (r, g, b) with values between -1.0 and 1.0.
+    ambientColor : array_like
+        Ambient material color (r, g, b) with values between -1.0 and 1.0.
+    emissionColor : array_like
+        Emission material color (r, g, b) with values between -1.0 and 1.0.
+    shininess : float
+        Material shininess, usually ranges from 0.0 to 128.0.
+    colorSpace : str
+        Color space for `diffuseColor`, `specularColor`, `ambientColor`, and
+        `emissionColor`. This is no longer used.
+    opacity : float
+        Opacity of the material. Ranges from 0.0 to 1.0 where 1.0 is fully
+        opaque.
+    contrast : float
+        Contrast of the material colors.
+    diffuseTexture : TexImage2D
+        Optional 2D texture to apply to the material. Color values from the
+        texture are blended with the `diffuseColor` of the material. The target
+        primitives must have texture coordinates to specify how texels are
+        mapped to the surface.
+    face : str
+        Face to apply material to. Values are `front`, `back` or `both`.
+
     Warnings
     --------
     This class is experimental and may result in undefined behavior.
@@ -441,7 +661,7 @@ class BlinnPhongMaterial:
     """
     def __init__(self,
                  win=None,
-                 diffuseColor=(.5, .5, .5),
+                 diffuseColor=(-1., -1., -1.),
                  specularColor=(-1., -1., -1.),
                  ambientColor=(-1., -1., -1.),
                  emissionColor=(-1., -1., -1.),
@@ -451,47 +671,15 @@ class BlinnPhongMaterial:
                  opacity=1.0,
                  contrast=1.0,
                  face='front'):
-        """
-        Parameters
-        ----------
-        win : `~psychopy.visual.Window` or `None`
-            Window this material is associated with, required for shaders and
-            some color space conversions.
-        diffuseColor : array_like
-            Diffuse material color (r, g, b, a) with values between 0.0 and 1.0.
-        specularColor : array_like
-            Specular material color (r, g, b, a) with values between 0.0 and
-            1.0.
-        ambientColor : array_like
-            Ambient material color (r, g, b, a) with values between 0.0 and 1.0.
-        emissionColor : array_like
-            Emission material color (r, g, b, a) with values between 0.0 and
-            1.0.
-        shininess : float
-            Material shininess, usually ranges from 0.0 to 128.0.
-        colorSpace : float
-            Color space for `diffuseColor`, `specularColor`, `ambientColor`, and
-            `emissionColor`.
-        diffuseTexture : TexImage2D
-        opacity : float
-            Opacity of the material. Ranges from 0.0 to 1.0 where 1.0 is fully
-            opaque.
-        contrast : float
-            Contrast of the material colors.
-        face : str
-            Face to apply material to. Values are `front`, `back` or `both`.
-        textures : dict, optional
-            Texture maps associated with this material. Textures are specified
-            as a list. The index of textures in the list will be used to set
-            the corresponding texture unit they are bound to.
-        """
+
         self.win = win
 
-        self._diffuseColor = np.zeros((3,), np.float32)
-        self._specularColor = np.zeros((3,), np.float32)
-        self._ambientColor = np.zeros((3,), np.float32)
-        self._emissionColor = np.zeros((3,), np.float32)
+        self._diffuseColor = Color()
+        self._specularColor = Color()
+        self._ambientColor = Color()
+        self._emissionColor = Color()
         self._shininess = float(shininess)
+        self._face = None  # set later
 
         # internal RGB values post colorspace conversion
         self._diffuseRGB = np.array((0., 0., 0., 1.), np.float32)
@@ -505,151 +693,381 @@ class BlinnPhongMaterial:
         self._ptrAmbient = None
         self._ptrEmission = None
 
-        # which faces to apply the material
-        if face == 'front':
-            self._face = GL.GL_FRONT
-        elif face == 'back':
-            self._face = GL.GL_BACK
-        elif face == 'both':
-            self._face = GL.GL_FRONT_AND_BACK
-        else:
-            raise ValueError("Invalid `face` specified, must be 'front', "
-                             "'back' or 'both'.")
-
-        self.colorSpace = colorSpace
-        self.opacity = opacity
-        self.contrast = contrast
-
         self.diffuseColor = diffuseColor
         self.specularColor = specularColor
         self.ambientColor = ambientColor
         self.emissionColor = emissionColor
 
+        self.colorSpace = colorSpace
+        self.opacity = opacity
+        self.contrast = contrast
+        self.face = face
+
         self._diffuseTexture = diffuseTexture
         self._normalTexture = None
 
         self._useTextures = False  # keeps track if textures are being used
 
+    # --------------------------------------------------------------------------
+    # Material colors and other properties
+    #
+    # These properties are used to set the color components of various material
+    # properties.
+    #
+
+    @property
+    def colorSpace(self):
+        """The name of the color space currently being used (`str` or `None`).
+
+        For strings and hex values this is not needed. If `None` the default
+        `colorSpace` for the stimulus is used (defined during initialisation).
+
+        Please note that changing `colorSpace` does not change stimulus
+        parameters. Thus, you usually want to specify `colorSpace` before
+        setting the color.
+
+        """
+        if hasattr(self, '_colorSpace'):
+            return self._colorSpace
+        else:
+            return 'rgba'
+
+    @colorSpace.setter
+    def colorSpace(self, value):
+        if value in colorSpaces:
+            self._colorSpace = value
+        else:
+            logging.error(f"'{value}' is not a valid color space")
+
+    @property
+    def contrast(self):
+        """A value that is simply multiplied by the color (`float`).
+
+        This may be used to adjust the lightness of the material. This is
+        applied to all material color components.
+
+        Examples
+        --------
+        Basic usage::
+
+            stim.contrast =  1.0  # unchanged contrast
+            stim.contrast =  0.5  # decrease contrast
+            stim.contrast =  0.0  # uniform, no contrast
+            stim.contrast = -0.5  # slightly inverted
+            stim.contrast = -1.0  # totally inverted
+
+        Setting contrast outside range -1 to 1 is permitted, but may
+        produce strange results if color values exceeds the monitor limits.::
+
+            stim.contrast =  1.2  # increases contrast
+            stim.contrast = -1.2  # inverts with increased contrast
+
+        """
+        return self._diffuseColor.contrast
+
+    @contrast.setter
+    def contrast(self, value):
+        self._diffuseColor.contrast = value
+        self._specularColor.contrast = value
+        self._ambientColor.contrast = value
+        self._emissionColor.contrast = value
+
+    @property
+    def shininess(self):
+        """Material shininess coefficient (`float`).
+
+        This is used to specify the 'tightness' of the specular highlights.
+        Values usually range between 0 and 128, but the range depends on the
+        specular highlight formula used by the shader.
+
+        """
+        return self._shininess
+
+    @shininess.setter
+    def shininess(self, value):
+        self._shininess = float(value)
+
+    @property
+    def face(self):
+        """Face to apply the material to (`str`). Possible values are one of
+        `'front'`, `'back'` or `'both'`.
+        """
+        return self._face
+
+    @face.setter
+    def face(self, value):
+        # which faces to apply the material
+        if value == 'front':
+            self._face = GL.GL_FRONT
+        elif value == 'back':
+            self._face = GL.GL_BACK
+        elif value == 'both':
+            self._face = GL.GL_FRONT_AND_BACK
+        else:
+            raise ValueError(
+                "Invalid value for `face` specified, must be 'front', 'back' "
+                "or 'both'.")
+
+    @property
+    def diffuseColor(self):
+        """Diffuse color `(r, g, b)` for the material (`psychopy.color.Color`,
+        `ArrayLike` or `None`).
+        """
+        return self._diffuseColor.render(self.colorSpace)
+
+    @diffuseColor.setter
+    def diffuseColor(self, value):
+        if isinstance(value, Color):
+            self._diffuseColor = value
+        else:
+            self._diffuseColor = Color(
+                value,
+                self.colorSpace,
+                contrast=self.contrast)
+
+        if not self._diffuseColor:
+            # If given an invalid color, set as transparent and log error
+            self._diffuseColor = Color()
+            logging.error(f"'{value}' is not a valid {self.colorSpace} color")
+
+        # compute RGB values for the shader
+        self._diffuseRGB[:3] = self._diffuseColor.rgb1
+        self._diffuseRGB[3] = self._diffuseColor.opacity
+
+        # need to create a pointer for the shader
+        self._ptrDiffuse = np.ctypeslib.as_ctypes(self._diffuseRGB)
+
+    def setDiffuseColor(self, color, colorSpace=None, operation='', log=None):
+        """Set the diffuse color for the material.
+
+        Use this method if you wish to supress logging or apply operations on
+        the color component.
+
+        Parameters
+        ----------
+        color : ArrayLike or `~psychopy.colors.Color`
+            Color to set as the diffuse component of the material.
+        colorSpace : str or None
+            Colorspace to use. This is only used to set the color, the value of
+            `diffuseColor` after setting uses the color space of the object.
+        operation : str
+            Operation string.
+        log : bool or None
+            Enable logging.
+
+        """
+        setColor(
+            obj=self,
+            colorAttrib="diffuseColor",
+            color=color,
+            colorSpace=colorSpace or self.colorSpace,
+            operation=operation,
+            log=log)
+
+    @property
+    def specularColor(self):
+        """Specular color `(r, g, b)` of the material (`psychopy.color.Color`,
+        `ArrayLike` or `None`).
+        """
+        return self._specularColor.render(self.colorSpace)
+
+    @specularColor.setter
+    def specularColor(self, value):
+        if isinstance(value, Color):
+            self._specularColor = value
+        else:
+            self._specularColor = Color(
+                value,
+                self.colorSpace,
+                contrast=self.contrast)
+
+        if not self._specularColor:
+            # If given an invalid color, set as transparent and log error
+            self._specularColor = Color()
+            logging.error(f"'{value}' is not a valid {self.colorSpace} color")
+
+        self._specularRGB[:3] = self._specularColor.rgb1
+        self._specularRGB[3] = self._specularColor.opacity
+
+        self._ptrSpecular = np.ctypeslib.as_ctypes(self._specularRGB)
+
+    def setSpecularColor(self, color, colorSpace=None, operation='', log=None):
+        """Set the diffuse color for the material. Use this function if you
+        wish to supress logging or apply operations on the color component.
+
+        Parameters
+        ----------
+        color : ArrayLike or `~psychopy.colors.Color`
+            Color to set as the specular component of the light source.
+        colorSpace : str or None
+            Colorspace to use. This is only used to set the color, the value of
+            `diffuseColor` after setting uses the color space of the object.
+        operation : str
+            Operation string.
+        log : bool or None
+            Enable logging.
+
+        """
+        setColor(
+            obj=self,
+            colorAttrib="specularColor",
+            color=color,
+            colorSpace=colorSpace or self.colorSpace,
+            operation=operation,
+            log=log)
+
+    @property
+    def ambientColor(self):
+        """Ambient color `(r, g, b)` of the material (`psychopy.color.Color`,
+        `ArrayLike` or `None`).
+        """
+        return self._ambientColor.render(self.colorSpace)
+
+    @ambientColor.setter
+    def ambientColor(self, value):
+        if isinstance(value, Color):
+            self._ambientColor = value
+        else:
+            self._ambientColor = Color(
+                value,
+                self.colorSpace,
+                contrast=self.contrast)
+
+        if not self._ambientColor:
+            # If given an invalid color, set as transparent and log error
+            self._ambientColor = Color()
+            logging.error(f"'{value}' is not a valid {self.colorSpace} color")
+
+        self._ambientRGB[:3] = self._ambientColor.rgb1
+        self._ambientRGB[3] = self._ambientColor.opacity
+
+        self._ptrAmbient = np.ctypeslib.as_ctypes(self._ambientRGB)
+
+    def setAmbientColor(self, color, colorSpace=None, operation='', log=None):
+        """Set the ambient color for the material.
+
+        Use this function if you wish to supress logging or apply operations on
+        the color component.
+
+        Parameters
+        ----------
+        color : ArrayLike or `~psychopy.colors.Color`
+            Color to set as the ambient component of the light source.
+        colorSpace : str or None
+            Colorspace to use. This is only used to set the color, the value of
+            `ambientColor` after setting uses the color space of the object.
+        operation : str
+            Operation string.
+        log : bool or None
+            Enable logging.
+
+        """
+        setColor(
+            obj=self,
+            colorAttrib="ambientColor",
+            color=color,
+            colorSpace=colorSpace or self.colorSpace,
+            operation=operation,
+            log=log)
+
+    @property
+    def emissionColor(self):
+        """Emission color `(r, g, b)` of the material (`psychopy.color.Color`,
+        `ArrayLike` or `None`).
+        """
+        return self._emissionColor.render(self.colorSpace)
+
+    @emissionColor.setter
+    def emissionColor(self, value):
+        if isinstance(value, Color):
+            self._emissionColor = value
+        else:
+            self._emissionColor = Color(
+                value,
+                self.colorSpace,
+                contrast=self.contrast)
+
+        if not self._emissionColor:
+            # If given an invalid color, set as transparent and log error
+            self._emissionColor = Color()
+            logging.error(f"'{value}' is not a valid {self.colorSpace} color")
+
+        self._emissionRGB[:3] = self._emissionColor.rgb1
+        self._emissionRGB[3] = self._emissionColor.opacity
+
+        self._ptrEmission = np.ctypeslib.as_ctypes(self._emissionRGB)
+
+    def setEmissionColor(self, color, colorSpace=None, operation='', log=None):
+        """Set the emission color for the material.
+
+        Use this function if you wish to supress logging or apply operations on
+        the color component.
+
+        Parameters
+        ----------
+        color : ArrayLike or `~psychopy.colors.Color`
+            Color to set as the ambient component of the light source.
+        colorSpace : str or None
+            Colorspace to use. This is only used to set the color, the value of
+            `ambientColor` after setting uses the color space of the object.
+        operation : str
+            Operation string.
+        log : bool or None
+            Enable logging.
+
+        """
+        setColor(
+            obj=self,
+            colorAttrib="emissionColor",
+            color=color,
+            colorSpace=colorSpace or self.colorSpace,
+            operation=operation,
+            log=log)
+
+    # --------------------------------------------------------------------------
+    # Material RGB colors
+    #
+    # These are the color values formatted for use in OpenGL.
+    #
+
+    @property
+    def diffuseRGB(self):
+        """RGB values of the diffuse color of the material (`numpy.ndarray`).
+        """
+        return self._diffuseRGB[:3]
+
+    @property
+    def specularRGB(self):
+        """RGB values of the specular color of the material (`numpy.ndarray`).
+        """
+        return self._specularRGB[:3]
+
+    @property
+    def ambientRGB(self):
+        """RGB values of the ambient color of the material (`numpy.ndarray`).
+        """
+        return self._ambientRGB[:3]
+
+    @property
+    def emissionRGB(self):
+        """RGB values of the emission color of the material (`numpy.ndarray`).
+        """
+        return self._emissionRGB[:3]
+
+    # Texture setter -----------------------------------------------------------
+
     @property
     def diffuseTexture(self):
-        """Diffuse color of the material."""
+        """Diffuse texture of the material (`psychopy.tools.gltools.TexImage2D`
+        or `None`).
+        """
         return self._diffuseTexture
 
     @diffuseTexture.setter
     def diffuseTexture(self, value):
         self._diffuseTexture = value
 
-    @property
-    def diffuseColor(self):
-        """Diffuse color of the material."""
-        return self._diffuseColor
-
-    @diffuseColor.setter
-    def diffuseColor(self, value):
-        self._diffuseColor = np.asarray(value, np.float32)
-        setColor(self, value, colorSpace=self.colorSpace, operation=None,
-                 rgbAttrib='diffuseRGB', colorAttrib='diffuseColor',
-                 colorSpaceAttrib='colorSpace')
-
-    @property
-    def diffuseRGB(self):
-        """Diffuse color of the material."""
-        return self._diffuseRGB[:3]
-
-    @diffuseRGB.setter
-    def diffuseRGB(self, value):
-        # make sure the color we got is 32-bit float
-        self._diffuseRGB = np.zeros((4,), np.float32)
-        self._diffuseRGB[:3] = (value * self.contrast + 1) / 2.0
-        self._diffuseRGB[3] = self.opacity
-
-        self._ptrDiffuse = np.ctypeslib.as_ctypes(self._diffuseRGB)
-
-    @property
-    def specularColor(self):
-        """Specular color of the material."""
-        return self._specularColor
-
-    @specularColor.setter
-    def specularColor(self, value):
-        self._specularColor = np.asarray(value, np.float32)
-        setColor(self, value, colorSpace=self.colorSpace, operation=None,
-                 rgbAttrib='specularRGB', colorAttrib='specularColor',
-                 colorSpaceAttrib='colorSpace')
-
-    @property
-    def specularRGB(self):
-        """Diffuse color of the material."""
-        return self._specularRGB[:3]
-
-    @specularRGB.setter
-    def specularRGB(self, value):
-        # make sure the color we got is 32-bit float
-        self._specularRGB = np.zeros((4,), np.float32)
-        self._specularRGB[:3] = (value * self.contrast + 1) / 2.0
-        self._specularRGB[3] = self.opacity
-
-        self._ptrSpecular = np.ctypeslib.as_ctypes(self._specularRGB)
-
-    @property
-    def ambientColor(self):
-        """Ambient color of the material."""
-        return self._ambientColor
-
-    @ambientColor.setter
-    def ambientColor(self, value):
-        self._ambientColor = np.asarray(value, np.float32)
-        setColor(self, value, colorSpace=self.colorSpace, operation=None,
-                 rgbAttrib='ambientRGB', colorAttrib='ambientColor',
-                 colorSpaceAttrib='colorSpace')
-
-    @property
-    def ambientRGB(self):
-        """Diffuse color of the material."""
-        return self._ambientRGB[:3]
-
-    @ambientRGB.setter
-    def ambientRGB(self, value):
-        # make sure the color we got is 32-bit float
-        self._ambientRGB = np.zeros((4,), np.float32)
-        self._ambientRGB[:3] = (value * self.contrast + 1) / 2.0
-        self._ambientRGB[3] = self.opacity
-
-        self._ptrAmbient = np.ctypeslib.as_ctypes(self._ambientRGB)
-
-    @property
-    def emissionColor(self):
-        """Emission color of the material."""
-        return self._emissionColor
-
-    @emissionColor.setter
-    def emissionColor(self, value):
-        self._emissionColor = np.asarray(value, np.float32)
-        setColor(self, value, colorSpace=self.colorSpace, operation=None,
-                 rgbAttrib='emissionRGB', colorAttrib='emissionColor',
-                 colorSpaceAttrib='colorSpace')
-
-    @property
-    def emissionRGB(self):
-        """Diffuse color of the material."""
-        return self._emissionRGB[:3]
-
-    @emissionRGB.setter
-    def emissionRGB(self, value):
-        # make sure the color we got is 32-bit float
-        self._emissionRGB = np.zeros((4,), np.float32)
-        self._emissionRGB[:3] = (value * self.contrast + 1) / 2.0
-        self._emissionRGB[3] = self.opacity
-
-        self._ptrEmission = np.ctypeslib.as_ctypes(self._emissionRGB)
-
-    @property
-    def shininess(self):
-        return self._shininess
-
-    @shininess.setter
-    def shininess(self, value):
-        self._shininess = float(value)
+    # --------------------------------------------------------------------------
 
     def begin(self, useTextures=True):
         """Use this material for successive rendering calls.
@@ -663,8 +1081,9 @@ class BlinnPhongMaterial:
         GL.glDisable(GL.GL_COLOR_MATERIAL)  # disable color tracking
         face = self._face
 
-        # number of scene lights
-        nLights = len(self.win.lights)
+        # check if lighting is enabled, otherwise don't render lights
+        nLights = len(self.win.lights) if self.win.useLights else 0
+
         useTextures = useTextures and self.diffuseTexture is not None
         shaderKey = (nLights, useTextures)
         gt.useProgram(self.win._shaders['stim3d_phong'][shaderKey])
@@ -1374,7 +1793,6 @@ class BaseRigidBodyStim(ColorMixin, WindowMixin):
             imaginary and `w` is real.
 
         """
-        self.autoLog = autoLog
         self.name = name
 
         super(BaseRigidBodyStim, self).__init__()
@@ -1391,6 +1809,8 @@ class BaseRigidBodyStim(ColorMixin, WindowMixin):
 
         self._vao = None
 
+        self.autoLog = autoLog
+
     @property
     def thePose(self):
         """The pose of the rigid body. This is a class which has `pos` and `ori`
