@@ -28,8 +28,9 @@ from psychopy import logging
 # (JWP has no idea why!)
 from psychopy.tools.monitorunittools import cm2pix, deg2pix, convertToPix
 from psychopy.tools.attributetools import attributeSetter, setAttribute
-from psychopy.visual.basevisual import (BaseVisualStim, ForeColorMixin,
-                                        ContainerMixin, WindowMixin)
+from psychopy.visual.basevisual import (
+    BaseVisualStim, DraggingMixin, ForeColorMixin, ContainerMixin, WindowMixin
+)
 from psychopy.colors import Color
 
 # for displaying right-to-left (possibly bidirectional) text correctly:
@@ -37,7 +38,7 @@ from bidi import algorithm as bidi_algorithm # sufficient for Hebrew
 # extra step needed to reshape Arabic/Farsi characters depending on
 # their neighbours:
 try:
-    import arabic_reshaper
+    from arabic_reshaper import ArabicReshaper
     haveArabic = True
 except ImportError:
     haveArabic = False
@@ -70,7 +71,7 @@ defaultWrapWidth = {'cm': 15.0,
                     'pixels': 500}
 
 
-class TextStim(BaseVisualStim, ForeColorMixin, ContainerMixin):
+class TextStim(BaseVisualStim, DraggingMixin, ForeColorMixin, ContainerMixin):
     """Class of text stimuli to be displayed in a
     :class:`~psychopy.visual.Window`
     """
@@ -101,8 +102,10 @@ class TextStim(BaseVisualStim, ForeColorMixin, ContainerMixin):
                  flipHoriz=False,
                  flipVert=False,
                  languageStyle='LTR',
+                 draggable=False,
                  name=None,
-                 autoLog=None):
+                 autoLog=None,
+                 autoDraw=False):
         """
         **Performance OBS:** in general, TextStim is slower than many other
         visual stimuli, i.e. it takes longer to change some attributes.
@@ -156,6 +159,7 @@ class TextStim(BaseVisualStim, ForeColorMixin, ContainerMixin):
 
         super(TextStim, self).__init__(
             win, units=units, name=name, autoLog=False)
+        self.draggable = draggable
 
         if win.blendMode=='add':
             logging.warning("Pyglet text does not honor the Window setting "
@@ -175,6 +179,10 @@ class TextStim(BaseVisualStim, ForeColorMixin, ContainerMixin):
         self.__dict__['flipHoriz'] = flipHoriz
         self.__dict__['flipVert'] = flipVert
         self.__dict__['languageStyle'] = languageStyle
+        if languageStyle.lower() == 'arabic':
+            arabic_config = {'delete_harakat': False,  # if present, retain any diacritics
+                             'shift_harakat_position': True}  # shift by 1 to be compatible with the bidi algorithm
+            self.__dict__['arabic_reshaper'] = ArabicReshaper(configuration = arabic_config)
         self._pygletTextObj = None
         self.pos = pos
         # deprecated attributes
@@ -222,6 +230,8 @@ class TextStim(BaseVisualStim, ForeColorMixin, ContainerMixin):
         self.setText(text, log=False)
         self._needUpdate = True
 
+        self.autoDraw = autoDraw
+
         # set autoLog now that params have been initialised
         wantLog = autoLog is None and self.win.autoLog
         self.__dict__['autoLog'] = autoLog or wantLog
@@ -359,8 +369,8 @@ class TextStim(BaseVisualStim, ForeColorMixin, ContainerMixin):
             if style == 'arabic' and haveArabic:
                 # reshape Arabic characters from their isolated form so that
                 # they flow and join correctly to their neighbours:
-                text = arabic_reshaper.reshape(text)
-            if style == 'rtl' or style == 'arabic' and haveArabic:
+                text = self.arabic_reshaper.reshape(text)
+            if style == 'rtl' or (style == 'arabic' and haveArabic):
                 # deal with right-to-left text presentation by applying the
                 # bidirectional algorithm:
                 text = bidi_algorithm.get_display(text)
