@@ -11,6 +11,8 @@ from distutils.core import setup
 from pkg_resources import parse_version
 import bdist_mpkg  # noqa: needed to build bdist, even though not explicitly used here
 import py2app  # noqa: needed to build app bundle, even though not explicitly used here
+from ctypes.util import find_library
+import importlib
 
 import psychopy
 version = psychopy.__version__
@@ -33,10 +35,11 @@ if platform != 'darwin':
 
 resources = glob.glob('psychopy/app/Resources/*')
 frameworks = [ # these installed using homebrew
-              "/usr/local/opt/libevent/lib/libevent.dylib", 
-              "/usr/local/opt/lame/lib/libmp3lame.0.dylib",
+              find_library("libevent"),
+              find_library("libmp3lame"),
+              find_library("libglfw"),
+              # libffi comes in the system
               "/usr/local/opt/libffi/lib/libffi.dylib",
-              "/usr/local/opt/libglfw/lib/libglfw.3.2.dylib",
               ]
 opencvLibs = glob.glob(os.path.join(sys.exec_prefix, 'lib', 'libopencv*.2.4.dylib'))
 frameworks.extend(opencvLibs)
@@ -57,65 +60,89 @@ if parse_version(macholib.__version__) <= parse_version('1.7'):
     macholib.MachOGraph.dyld_find = dyld_find
 
 includes = ['_sitebuiltins',  # needed for help()
-            'Tkinter', 'tkFileDialog',
             'imp', 'subprocess', 'shlex',
             'shelve',  # for scipy.io
             '_elementtree', 'pyexpat',  # for openpyxl
-            'hid',
             'pyo', 'greenlet', 'zmq', 'tornado',
             'psutil',  # for iohub
             'tobii_research',  # need tobii_research file and tobiiresearch pkg
-            'pysoundcard', 'soundfile', 'sounddevice', 'readline',
-            'hid',
+            'soundfile', 'sounddevice', 'readline',
             'xlwt',  # writes excel files for pandas
-            'vlc',  # install with pip install python-vlc
             'msgpack_numpy',
             'configparser',
+            'ntplib',  # for egi-pynetstation
             ]
 packages = ['pydoc',  # needed for help()
+            'setuptools', 'wheel', # for plugin installing
             'wx', 'psychopy',
-            'pyglet', 'pytz', 'OpenGL', 'glfw',
-            'scipy', 'matplotlib', 'openpyxl',
-            'xml', 'xmlschema', 'elementpath',
-            'moviepy', 'imageio', 'imageio_ffmpeg',
+            'PyQt6',
+            'pyglet', 'pytz',
+            'scipy', 'matplotlib', 'openpyxl', 'pandas',
+            'xml', 'xmlschema',
+            'ffpyplayer', 'cython', 'AVFoundation',
+            'imageio', 'imageio_ffmpeg',
             '_sounddevice_data', '_soundfile_data',
             'cffi', 'pycparser',
             'PIL',  # 'Image',
-            'objc', 'Quartz', 'AppKit', 'QTKit', 'Cocoa',
+            'freetype',
+            'objc', 'Quartz', 'AppKit', 'Cocoa',
             'Foundation', 'CoreFoundation',
             'pkg_resources',  # needed for objc
-            'pyo',
             'requests', 'certifi', 'cryptography',
-            # for unit testing
-            'coverage',
-            # handy external science libs
-            'serial',
-            'egi', 'pylink', 'tobiiresearch',
-            'pyxid2', 'ftd2xx',  # ftd2xx is used by cedrus
-            'pandas', 'tables',  # 'cython',
-            'msgpack', 'yaml', 'gevent',  # for ioHub
-            # these aren't needed, but liked
-            'bidi', 'arabic_reshaper',  # for right-left language conversions
-            # for Py3 compatibility
-            'ujson',  # faster than built-in json
             'json_tricks',  # allows saving arrays/dates in json
             'git', 'gitlab',
+            'msgpack', 'yaml', 'gevent',  # for ioHub
             'astunparse', 'esprima',  # for translating/adapting py/JS
+            'metapensiero.pj', 'dukpy', 
+            'jedi', 'parso',
+            'bidi', 'arabic_reshaper', 'charset_normalizer', # for (natural) language conversions
+            'ujson',  # faster than built-in json
+            'six',  # needed by configobj
+            # hardware
+            'serial',
+            # handy science tools
+            'tables',  # 'cython',
+            # these aren't needed, but liked
             'pylsl', 'pygaze',
             'smite',  # https://github.com/marcus-nystrom/SMITE (not pypi!)
             'cv2',
-            'badapted', 'darc_toolbox',  # adaptive methods from Ben Vincent
             'questplus',
-            'metapensiero.pj', 'dukpy', 'macropy',
-            'jedi', 'parso',
             'psychtoolbox',
-            'freetype', 'h5py',
-            'markdown_it',
-            'speech_recognition', 'googleapiclient', 'pocketsphinx',
-            'six',  # needed by configobj
-            'PyQt5',
+            'h5py',
+            'markdown_it'
             ]
 
+# Add packages that older PsychoPy (<=2023.1.x) shipped, for useVersion() compatibility
+# In PsychoPy 2023.2.0 these packages were removed from Standalone Py3.10+ builds
+if sys.version_info < (3, 9):
+    packages.extend(
+        [
+            'moviepy', 
+            'OpenGL', 'glfw',
+            'speech_recognition', 'googleapiclient', 'pocketsphinx',
+            'badapted', #'darc_toolbox',  # adaptive methods from Ben Vincent
+            'egi_pynetstation', 'pylink', 'tobiiresearch',
+            'pyxid2', 'ftd2xx',  # ftd2xx is used by cedrus
+            'Phidget22',
+            'hid',
+            'macropy',
+        ]
+    )
+    packages.append('PyQt5')
+    packages.remove('PyQt6')  # PyQt6 is not compatible with earlier PsychoPy versions
+
+# check the includes and packages are all available
+for pkg in includes+packages:
+    missing_pkgs = []
+    try:
+        importlib.import_module(pkg)
+    except ImportError:
+        missing_pkgs.append(pkg)
+if missing_pkgs:
+    raise ImportError("Missing packages: %s" % missing_pkgs)
+else:
+    print("All packages appear to be present. Proceeding to build...")
+
 setup(
     app=['psychopy/app/psychopyApp.py'],
     options=dict(py2app=dict(
@@ -125,6 +152,9 @@ setup(
                       'libsz.2.dylib', 'pygame',
                       # 'stringprep',
                       'functools32',
+                      'sympy',
+                      '/usr/lib/libffi.dylib',
+                      'libwebp.7.dylib'
                       ],  # anything we need to forcibly exclude?
             resources=resources,
             argv_emulation=False,  # must be False or app bundle pauses (py2app 0.21 and 0.24 tested)
@@ -142,6 +172,9 @@ setup(
                   NSHumanReadableCopyright   = "Open Science Tools Limited",
                   CFBundleDocumentTypes=[dict(CFBundleTypeExtensions=['*'],
                                               CFBundleTypeRole='Editor')],
+                  CFBundleURLTypes=[dict(CFBundleURLName='psychopy',  # respond to psychopy://
+                                         CFBundleURLSchemes='psychopy',
+                                         CFBundleTypeRole='Editor')],
                   LSEnvironment=dict(PATH="/usr/local/git/bin:/usr/local/bin:"
                                           "/usr/local:/usr/bin:/usr/sbin"),
             ),
